/*
** 2000-05-29
**
** The author disclaims copyright to this source code.  In place of
** a legal notice, here is a blessing:
**
**    May you do good and not evil.
**    May you find forgiveness for yourself and forgive others.
**    May you share freely, never taking more than you give.
**
*************************************************************************
** Driver template for the LEMON parser generator.
**
** The "lemon" program processes an LALR(1) input grammar file, then uses
** this template to construct a parser.  The "lemon" program inserts text
** at each "%%" line.  Also, any "P-a-r-s-e" identifer prefix (without the
** interstitial "-" characters) contained in this template is changed into
** the value of the %name directive from the grammar.  Otherwise, the content
** of this template is copied straight through into the generate parser
** source file.
**
** The following is the concatenation of all %include directives from the
** input grammar file:
*/
#include <stdio.h>
#include <string.h>
#include <assert.h>

#ifdef _MSC_VER
#define CDECL __cdecl
#else
#define CDECL
#endif

/************ Begin %include sections from the grammar ************************/
#line 40 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"

// Allocates a new AST node off the parse state's arena.
#define NEW_AST_NODE(type,name,tok) \
	ZCC_##type *name = static_cast<ZCC_##type *>(stat->InitNode(sizeof(ZCC_##type), AST_##type)); \
	SetNodeLine(name, tok)

static void SetNodeLine(ZCC_TreeNode *name, ZCCToken &tok)
{
	name->SourceLoc = tok.SourceLoc;
}

static void SetNodeLine(ZCC_TreeNode *name, ZCC_TreeNode *node)
{
	if (name == nullptr || node == nullptr)
	{
		I_Error("Fatal parse error");
	}
	name->SourceLoc = node->SourceLoc;
}

static void SetNodeLine(ZCC_TreeNode *name, int line)
{
	name->SourceLoc = line;
}

// If a is non-null, appends b to a. Otherwise, sets a to b.
#define SAFE_APPEND(a,b) \
	if (a == NULL) a = b; else AppendTreeNodeSibling(a, b);

#define UNARY_EXPR(X,T) NEW_AST_NODE(ExprUnary, expr1, X); expr1->Operation = T; expr1->Operand = X; expr1->Type = NULL
#define BINARY_EXPR(X,Y,T) NEW_AST_NODE(ExprBinary, expr2, X); expr2->Operation = T; expr2->Type = NULL; expr2->Left = X; expr2->Right = Y

#define NEW_INTCONST_NODE(name,type,val,tok) \
	NEW_AST_NODE(ExprConstant, name, tok); \
	name->Operation = PEX_ConstValue; \
	name->Type = type; \
	name->IntVal = val

	struct ClassFlagsBlock {
		VM_UWORD Flags;
		ZCC_Identifier *Replaces;
		VersionInfo Version;
	};

	struct StateOpts {
		ZCC_Expression *Offset;
		ZCC_ExprConstant *Lights;
		bool Bright;
		bool Fast;
		bool Slow;
		bool NoDelay;
		bool CanRaise;

		void Zero() {
			Offset = nullptr;
			Lights = nullptr;
			Bright = false;
			Fast = false;
			Slow = false;
			NoDelay = false;
			CanRaise = false;
		}
	};

	struct VarOrFun
	{
		ZCC_VarName *VarNames;
		ZCC_FuncParamDecl *FuncParams;
		ZCC_CompoundStmt *FuncBody;
		ENamedName FuncName;
		int FuncFlags;
		int SourceLoc;
	};
#line 111 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
/**************** End of %include directives **********************************/
/* These constants specify the various numeric values for terminal symbols
** in a format understandable to "makeheaders".  This section is blank unless
** "lemon" is run with the "-m" command-line option.
***************** Begin makeheaders token definitions *************************/
/**************** End makeheaders token definitions ***************************/

/* The next section is a series of control #defines.
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used to store the integer codes
**                       that represent terminal and non-terminal symbols.
**                       "unsigned char" is used if there are fewer than
**                       256 symbols.  Larger types otherwise.
**    YYNOCODE           is a number of type YYCODETYPE that is not used for
**                       any terminal or nonterminal symbol.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       (also known as: "terminal symbols") have fall-back
**                       values which should be used if the original symbol
**                       would not parse.  This permits keywords to sometimes
**                       be used as identifiers, for example.
**    YYACTIONTYPE       is the data type used for "action codes" - numbers
**                       that indicate what to do in response to the next
**                       token.
**    ZCCParseTOKENTYPE     is the data type used for minor type for terminal
**                       symbols.  Background: A "minor type" is a semantic
**                       value associated with a terminal or non-terminal
**                       symbols.  For example, for an "ID" terminal symbol,
**                       the minor type might be the name of the identifier.
**                       Each non-terminal can have a different minor type.
**                       Terminal symbols all have the same minor type, though.
**                       This macros defines the minor type for terminal 
**                       symbols.
**    YYMINORTYPE        is the data type used for all minor types.
**                       This is typically a union of many types, one of
**                       which is ZCCParseTOKENTYPE.  The entry in the union
**                       for terminal symbols is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    ZCCParseARG_SDECL     A static variable declaration for the %extra_argument
**    ZCCParseARG_PDECL     A parameter declaration for the %extra_argument
**    ZCCParseARG_STORE     Code to store %extra_argument into yypParser
**    ZCCParseARG_FETCH     Code to extract %extra_argument from yypParser
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YY_MAX_SHIFT       Maximum value for shift actions
**    YY_MIN_SHIFTREDUCE Minimum value for shift-reduce actions
**    YY_MAX_SHIFTREDUCE Maximum value for shift-reduce actions
**    YY_MIN_REDUCE      Maximum value for reduce actions
**    YY_ERROR_ACTION    The yy_action[] code for syntax error
**    YY_ACCEPT_ACTION   The yy_action[] code for accept
**    YY_NO_ACTION       The yy_action[] code for no-op
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/************* Begin control #defines *****************************************/
#define YYCODETYPE unsigned short int
#define YYNOCODE 268
#define YYACTIONTYPE unsigned short int
#define ZCCParseTOKENTYPE  ZCCToken 
typedef union {
  int yyinit;
  ZCCParseTOKENTYPE yy0;
  ZCC_Enum * yy5;
  ZCC_DeclFlags * yy9;
  ZCC_ExprFuncCall * yy13;
  ZCC_IfStmt * yy17;
  ZCC_AssignStmt * yy25;
  ZCC_LocalVarStmt * yy32;
  ZCC_FuncParamDecl * yy37;
  ZCC_PropertyStmt * yy39;
  ZCC_Identifier * yy41;
  ZCC_States * yy48;
  ZCC_FlagStmt * yy50;
  ZCC_CompoundStmt * yy51;
  ZCC_Default * yy65;
  ZCC_FuncParm * yy74;
  ZCC_Class * yy80;
  ZCC_VarInit * yy83;
  ZCC_BasicType * yy86;
  ZCC_VarName * yy134;
  ZCC_Expression * yy154;
  ZCC_TreeNode * yy168;
  StateOpts yy169;
  VarOrFun yy170;
  ZCC_ExprConstant * yy215;
  ZCC_Type * yy256;
  ZCC_CaseStmt * yy296;
  ZCC_ConstantDef * yy349;
  ZCC_FlagDef * yy357;
  ZCC_Declarator * yy365;
  ZCC_Statement * yy367;
  ZCC_ExpressionStmt * yy372;
  ZCC_StatePart * yy376;
  ZCC_MixinStmt * yy387;
  ZCC_Struct * yy411;
  ClassFlagsBlock yy435;
  ZCC_MixinDef * yy488;
  ZCC_Property * yy493;
  ZCC_StaticArrayStatement * yy502;
  int yy535;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 0
#endif
#define ZCCParseARG_SDECL  ZCCParseState *stat ;
#define ZCCParseARG_PDECL , ZCCParseState *stat 
#define ZCCParseARG_FETCH  ZCCParseState *stat  = yypParser->stat 
#define ZCCParseARG_STORE yypParser->stat  = stat 
#define YYERRORSYMBOL 159
#define YYERRSYMDT yy535
#define YYFALLBACK 1
#define YYNSTATE             396
#define YYNRULE              372
#define YY_MAX_SHIFT         395
#define YY_MIN_SHIFTREDUCE   663
#define YY_MAX_SHIFTREDUCE   1034
#define YY_MIN_REDUCE        1035
#define YY_MAX_REDUCE        1406
#define YY_ERROR_ACTION      1407
#define YY_ACCEPT_ACTION     1408
#define YY_NO_ACTION         1409
/************* End control #defines *******************************************/

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N <= YY_MAX_SHIFT             Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   N between YY_MIN_SHIFTREDUCE       Shift to an arbitrary state then
**     and YY_MAX_SHIFTREDUCE           reduce by rule N-YY_MIN_SHIFTREDUCE.
**
**   N between YY_MIN_REDUCE            Reduce by rule N-YY_MIN_REDUCE
**     and YY_MAX_REDUCE
**
**   N == YY_ERROR_ACTION               A syntax error has occurred.
**
**   N == YY_ACCEPT_ACTION              The parser accepts its input.
**
**   N == YY_NO_ACTION                  No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as either:
**
**    (A)   N = yy_action[ yy_shift_ofst[S] + X ]
**    (B)   N = yy_default[S]
**
** The (A) formula is preferred.  The B formula is used instead if:
**    (1)  The yy_shift_ofst[S]+X value is out of range, or
**    (2)  yy_lookahead[yy_shift_ofst[S]+X] is not equal to X, or
**    (3)  yy_shift_ofst[S] equal YY_SHIFT_USE_DFLT.
** (Implementation note: YY_SHIFT_USE_DFLT is chosen so that
** YY_SHIFT_USE_DFLT+X will be out of range for all possible lookaheads X.
** Hence only tests (1) and (2) need to be evaluated.)
**
** The formulas above are for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
**
*********** Begin parsing tables **********************************************/
#define YY_ACTTAB_COUNT (2947)
static const YYACTIONTYPE yy_action[] = {
 /*     0 */   116,  115,  665,  667,  668,  669,  670,  671,  672,  113,
 /*    10 */   114,  163,   34,   37,  139,  817,  956,  768,  174,  870,
 /*    20 */   162,  343,  755,  756,  757,  173,  817,  282,  947,   97,
 /*    30 */   316,  815,    2,  970,  163,    2,   64,  949,  734,  829,
 /*    40 */   817,  830,  128,  162,  818,  832,  137,  137, 1243,  882,
 /*    50 */   829,  817,  830,  128,  815,  281,  347,  800,  801,  802,
 /*    60 */   803,  804,  805,  806,  807,  808,  809,  810,  811,  812,
 /*    70 */   813,  814,  816,  361,  354,  307,  355,  351,  348,  124,
 /*    80 */   800,  801,  802,  803,  804,  805,  806,  807,  808,  809,
 /*    90 */   810,  811,  812,  813,  814,  816,  361,  354,  872,  355,
 /*   100 */   351,  348,  729,  112,  111,  110,  109,  950,  951,  952,
 /*   110 */   953,  954,  955,   45,  329,  328,   33,   10,  324,  321,
 /*   120 */   980,  981,  318,  362,  317,   53,   94,   93,   92,  102,
 /*   130 */   106,  100,   99,   98,   96,   95,   97,  116,  115,  666,
 /*   140 */   165,  363,  165,   64, 1408,   17,  113,  114,  163,   34,
 /*   150 */    37,  836,  695,  956,   47,  248,  248,  162,  343,  202,
 /*   160 */   374,  698,  697,  701,  240,  947,  688,  316,  815,    2,
 /*   170 */   971,  873,  694,   25,  949,  689,  690,  696,  702,  703,
 /*   180 */   699,  700,  704,  837,  837,  786,  882,  800,  801,  802,
 /*   190 */   803,  804,  805,   59,  800,  801,  802,  803,  804,  805,
 /*   200 */   806,  807,  808,  809,  810,  811,  812,  813,  814,  816,
 /*   210 */   361,  354,  828,  355,  351,  348,   81,   94,   93,   92,
 /*   220 */   102,  106,  100,   99,   98,   96,   95,   97,   16,  239,
 /*   230 */   202,  883,  943,  943,   64,  240,  322,  200,  374,  817,
 /*   240 */   112,  111,  110,  109,  950,  951,  952,  953,  954,  955,
 /*   250 */    45,  329,  328,   33,   10,  324,  321,  980,  981,  318,
 /*   260 */   315,  317,   53,  829,  817,  830,  128,  345,  119,  345,
 /*   270 */   344,  123,  837,   41,  116,  115,  310,  310,   44,  844,
 /*   280 */   371,  719,  721,  113,  114,  163,   34,   37,  719,  721,
 /*   290 */   956,  279,  722,  723,  162,  343,  122,  180,  374,  722,
 /*   300 */   723,  389,  947,  728,  316,  815,    2,  239,  200,  883,
 /*   310 */   765,  949,  724,  292,   43,   43,   32,  159,  159,  724,
 /*   320 */   378,   43,   43,  882,  681,  682,  683,  684,  141,  273,
 /*   330 */    46,  800,  801,  802,  803,  804,  805,  806,  807,  808,
 /*   340 */   809,  810,  811,  812,  813,  814,  816,  361,  354,  747,
 /*   350 */   355,  351,  348,  791,  794,  795,  198,  374,   16,  246,
 /*   360 */   759,  760,  761,  762,  136,   16,  763,  239,  180,  883,
 /*   370 */   892,  893,  372,   22,   61,   41,  764,  112,  111,  110,
 /*   380 */   109,  950,  951,  952,  953,  954,  955,   45,  329,  328,
 /*   390 */    33,   10,  324,  321,  980,  981,  318,  163,  317,   53,
 /*   400 */   177,  147,  118,   41,  243,  758,  162,  818,  862,  850,
 /*   410 */   864,  865,    1,  301,  874,   44,  172,  815,  202,  374,
 /*   420 */   390,  388,  312,  314,  314,  866,  239,  198,  883,  134,
 /*   430 */   770,  771,  772,  773,  774,  290,  289,  160,  160,  198,
 /*   440 */   374,  980,  981,  800,  801,  802,  803,  804,  805,  806,
 /*   450 */   807,  808,  809,  810,  811,  812,  813,  814,  816,  361,
 /*   460 */   360,  244,  355,  351,  348,  833, 1035,   50,  242,  754,
 /*   470 */   835,  154,  305,  851,  852,  853,  854,  855,  856,  857,
 /*   480 */   859,  860,  861,  863,  867,  116,  115,  249,  239,  202,
 /*   490 */   883,  244,  244,   32,  113,  114,  163,   34,  798,   59,
 /*   500 */  1090,  325,  675,  276,  334,  162,  343,  377,  365,  239,
 /*   510 */   198,  883,  339,  947, 1002,  117,  815,  169,  241,  275,
 /*   520 */   313,  294,  949,  100,   99,   98,   96,   95,   97,  108,
 /*   530 */   790,  794,  795,  269,  882,   64,  687,  379,  270,  377,
 /*   540 */   365,  140,  800,  801,  802,  803,  804,  805,  806,  807,
 /*   550 */   808,  809,  810,  811,  812,  813,  814,  816,  361,  354,
 /*   560 */   793,  355,  351,  348,  288,   41,  274,  102,  106,  100,
 /*   570 */    99,   98,   96,   95,   97,  306,  776,  789,  199,  374,
 /*   580 */    45,   64,  346,  200,  374,  838,  155,  749,  112,  111,
 /*   590 */   110,  109,  950,  951,  952,  953,  954,  955,   76,   73,
 /*   600 */    72,   71,   75,   74,   70,   69,   67,   66,   65,   68,
 /*   610 */    63,  171,   77,   78,   84,   83,   82,   90,   89,   88,
 /*   620 */    87,   86,   85,   91,   79,   80,   81,   94,   93,   92,
 /*   630 */   102,  106,  100,   99,   98,   96,   95,   97,  277,   35,
 /*   640 */    25,   59,   59,  962,   64,  195,  374,  103,  239,  199,
 /*   650 */   883,    9,  889,  239,  200,  883,  986,  998,  999,  885,
 /*   660 */   972,   76,   73,   72,   71,   75,   74,   70,   69,   67,
 /*   670 */    66,   65,   68,   63,  278,   77,   78,   84,   83,   82,
 /*   680 */    90,   89,   88,   87,   86,   85,   91,   79,   80,   81,
 /*   690 */    94,   93,   92,  102,  106,  100,   99,   98,   96,   95,
 /*   700 */    97,  969,   59,   20,  716,  713,  714,   64,  298,  374,
 /*   710 */   104,  193,  374,   19,  133,  239,  195,  883,  784,  359,
 /*   720 */   715,   59,  886,  135,   76,   73,   72,   71,   75,   74,
 /*   730 */    70,   69,   67,   66,   65,   68,   63,  330,   77,   78,
 /*   740 */    84,   83,   82,   90,   89,   88,   87,   86,   85,   91,
 /*   750 */    79,   80,   81,   94,   93,   92,  102,  106,  100,   99,
 /*   760 */    98,   96,   95,   97,   25,  962,  285,  777,   59,  245,
 /*   770 */    64,  358,  153,  105,  175, 1004,  888,  711,  239,  901,
 /*   780 */   883,  239,  193,  883,  335,  887,  303,   76,   73,   72,
 /*   790 */    71,   75,   74,   70,   69,   67,   66,   65,   68,   63,
 /*   800 */   302,   77,   78,   84,   83,   82,   90,   89,   88,   87,
 /*   810 */    86,   85,   91,   79,   80,   81,   94,   93,   92,  102,
 /*   820 */   106,  100,   99,   98,   96,   95,   97,  750,  247,  845,
 /*   830 */   127, 1001,  869,   64,  121,   76,   73,   72,   71,   75,
 /*   840 */    74,   70,   69,   67,   66,   65,   68,   63,  775,   77,
 /*   850 */    78,   84,   83,   82,   90,   89,   88,   87,   86,   85,
 /*   860 */    91,   79,   80,   81,   94,   93,   92,  102,  106,  100,
 /*   870 */    99,   98,   96,   95,   97,  842,  320,  258,  161,  826,
 /*   880 */   356,   64,  730,  164,   48, 1011,   76,   73,   72,   71,
 /*   890 */    75,   74,   70,   69,   67,   66,   65,   68,   63,  991,
 /*   900 */    77,   78,   84,   83,   82,   90,   89,   88,   87,   86,
 /*   910 */    85,   91,   79,   80,   81,   94,   93,   92,  102,  106,
 /*   920 */   100,   99,   98,   96,   95,   97,  726,  132,  364,  710,
 /*   930 */   947,  395,   64,   15,   76,   73,   72,   71,   75,   74,
 /*   940 */    70,   69,   67,   66,   65,   68,   63,  237,   77,   78,
 /*   950 */    84,   83,   82,   90,   89,   88,   87,   86,   85,   91,
 /*   960 */    79,   80,   81,   94,   93,   92,  102,  106,  100,   99,
 /*   970 */    98,   96,   95,   97,   13,  238,  283,  284,  291,   14,
 /*   980 */    64,  170,   76,   73,   72,   71,   75,   74,   70,   69,
 /*   990 */    67,   66,   65,   68,   63,    5,   77,   78,   84,   83,
 /*  1000 */    82,   90,   89,   88,   87,   86,   85,   91,   79,   80,
 /*  1010 */    81,   94,   93,   92,  102,  106,  100,   99,   98,   96,
 /*  1020 */    95,   97,  267,  268,  240,  686,  271,  272,   64,  142,
 /*  1030 */    76,   73,   72,   71,   75,   74,   70,   69,   67,   66,
 /*  1040 */    65,   68,   63,    6,   77,   78,   84,   83,   82,   90,
 /*  1050 */    89,   88,   87,   86,   85,   91,   79,   80,   81,   94,
 /*  1060 */    93,   92,  102,  106,  100,   99,   98,   96,   95,   97,
 /*  1070 */   676,  793,   42,  145,  146,  781,   64, 1012,   76,   73,
 /*  1080 */    72,   71,   75,   74,   70,   69,   67,   66,   65,   68,
 /*  1090 */    63,    8,   77,   78,   84,   83,   82,   90,   89,   88,
 /*  1100 */    87,   86,   85,   91,   79,   80,   81,   94,   93,   92,
 /*  1110 */   102,  106,  100,   99,   98,   96,   95,   97,  782,  780,
 /*  1120 */   779,  778,  130,  149,   64,   49,   76,   73,   72,   71,
 /*  1130 */    75,   74,   70,   69,   67,   66,   65,   68,   63,  977,
 /*  1140 */    77,   78,   84,   83,   82,   90,   89,   88,   87,   86,
 /*  1150 */    85,   91,   79,   80,   81,   94,   93,   92,  102,  106,
 /*  1160 */   100,   99,   98,   96,   95,   97,  151,   18,  293,  294,
 /*  1170 */   717,  296,   64,  297,   76,   73,   72,   71,   75,   74,
 /*  1180 */    70,   69,   67,   66,   65,   68,   63,   11,   77,   78,
 /*  1190 */    84,   83,   82,   90,   89,   88,   87,   86,   85,   91,
 /*  1200 */    79,   80,   81,   94,   93,   92,  102,  106,  100,   99,
 /*  1210 */    98,   96,   95,   97,  849,  299,  198,  374,  848,  300,
 /*  1220 */    64,  847,   76,   73,   72,   71,   75,   74,   70,   69,
 /*  1230 */    67,   66,   65,   68,   63,   62,   77,   78,   84,   83,
 /*  1240 */    82,   90,   89,   88,   87,   86,   85,   91,   79,   80,
 /*  1250 */    81,   94,   93,   92,  102,  106,  100,   99,   98,   96,
 /*  1260 */    95,   97,  202,  374,  253,  152,  751,  304,   64,  839,
 /*  1270 */    91,   79,   80,   81,   94,   93,   92,  102,  106,  100,
 /*  1280 */    99,   98,   96,   95,   97,   51,  239,  198,  883,  308,
 /*  1290 */   890,   64,   76,   73,   72,   71,   75,   74,   70,   69,
 /*  1300 */    67,   66,   65,   68,   63,  260,   77,   78,   84,   83,
 /*  1310 */    82,   90,   89,   88,   87,   86,   85,   91,   79,   80,
 /*  1320 */    81,   94,   93,   92,  102,  106,  100,   99,   98,   96,
 /*  1330 */    95,   97,  239,  202,  883,  260,  260,  125,   64,   35,
 /*  1340 */    76,   73,   72,   71,   75,   74,   70,   69,   67,   66,
 /*  1350 */    65,   68,   63,  884,   77,   78,   84,   83,   82,   90,
 /*  1360 */    89,   88,   87,   86,   85,   91,   79,   80,   81,   94,
 /*  1370 */    93,   92,  102,  106,  100,   99,   98,   96,   95,   97,
 /*  1380 */  1000,   36,  156,   52,  992,   54,   64,   55,  725,    7,
 /*  1390 */    76,   73,   72,   71,   75,   74,   70,   69,   67,   66,
 /*  1400 */    65,   68,   63,  157,   77,   78,   84,   83,   82,   90,
 /*  1410 */    89,   88,   87,   86,   85,   91,   79,   80,   81,   94,
 /*  1420 */    93,   92,  102,  106,  100,   99,   98,   96,   95,   97,
 /*  1430 */    56,  163,   24,   38,  967,   12,   64,   57,   58,   39,
 /*  1440 */   162,  818,  331,  817,  966,  965,  333,  964,  332,  336,
 /*  1450 */   337,  815,   79,   80,   81,   94,   93,   92,  102,  106,
 /*  1460 */   100,   99,   98,   96,   95,   97,   40,  829,  817,  830,
 /*  1470 */   128,  961,   64,  349,  338,  960,  959,  800,  801,  802,
 /*  1480 */   803,  804,  805,  806,  807,  808,  809,  810,  811,  812,
 /*  1490 */   813,  814,  816,  361,  354,  374,  355,  351,  348,   90,
 /*  1500 */    89,   88,   87,   86,   85,   91,   79,   80,   81,   94,
 /*  1510 */    93,   92,  102,  106,  100,   99,   98,   96,   95,   97,
 /*  1520 */   878,  879,  880, 1190,  202,  374,   64,  158,   26,  834,
 /*  1530 */   825,   27,   28,  824,   63,   29,   77,   78,   84,   83,
 /*  1540 */    82,   90,   89,   88,   87,   86,   85,   91,   79,   80,
 /*  1550 */    81,   94,   93,   92,  102,  106,  100,   99,   98,   96,
 /*  1560 */    95,   97,  148,  817,  239,  900,  883,  261,   64,  823,
 /*  1570 */   143,  144,   30,  259,   31,   60,  181,  374,  821,  199,
 /*  1580 */   374,  817,  357,  820,  819,  166,  792,  829,  817,  830,
 /*  1590 */   309,  687,  120,  963,  239,  202,  883,  261,  261,  167,
 /*  1600 */    21,  295,  366,  787,  817,  829,  817,  830,  128,  367,
 /*  1610 */   368,  350,  280,  369, 1022,  692,  394,  392,  385,  286,
 /*  1620 */   379,    3,  377,  365,  150,  891,   64,  948,  829,  817,
 /*  1630 */   830,  126,  374,   80,   81,   94,   93,   92,  102,  106,
 /*  1640 */   100,   99,   98,   96,   95,   97,  239,  181,  883,  239,
 /*  1650 */   199,  883,   64,  101,  958,  973,  957,  342,  973,  973,
 /*  1660 */   973,  973,  341,  340,  326,  107,  376,  319,   78,   84,
 /*  1670 */    83,   82,   90,   89,   88,   87,   86,   85,   91,   79,
 /*  1680 */    80,   81,   94,   93,   92,  102,  106,  100,   99,   98,
 /*  1690 */    96,   95,   97,  168,  257,   45,  176,  707,  380,   64,
 /*  1700 */   382,  239,  899,  883,  738,  143,  144,  116,  115,  381,
 /*  1710 */   383,  199,  374,  740,  739,  743,  113,  114,  240,   34,
 /*  1720 */   384,  792,  386,  799,  706,  963,  687, 1123,  881,  387,
 /*  1730 */   744,  745,  741,  742,  746,  947,  817, 1122,  788,  129,
 /*  1740 */   817,  705,  391,  393,  949, 1037, 1037,  817,  737, 1037,
 /*  1750 */  1037, 1037, 1037,    4, 1037, 1037,  882, 1037,  198,  374,
 /*  1760 */   829,  817,  830,  126,  829,  817,  830,  128,  374, 1037,
 /*  1770 */   352,  829,  817,  830,  128, 1037, 1037,  353,  182,  374,
 /*  1780 */    16,  239,  199,  883, 1037, 1037,  958,  973,  957,  342,
 /*  1790 */   973,  973,  973,  973,  341,  340,  326,  817, 1037,  319,
 /*  1800 */  1037, 1037, 1037, 1037, 1037, 1037,  250, 1037, 1037,  327,
 /*  1810 */   112,  111,  110,  109,  950,  951,  952,  953,  954,  955,
 /*  1820 */  1037,  829,  817,  830,  256, 1037,  199,  374,  239,  198,
 /*  1830 */   883, 1037, 1037, 1037, 1037, 1037, 1037,  239,  898,  883,
 /*  1840 */   963, 1037,  691,  116,  115, 1037,  295, 1037,  239,  182,
 /*  1850 */   883,  817,  113,  114, 1037,   34, 1037,  280, 1037,  968,
 /*  1860 */  1037,  394,  392,  385,  881,  379,  287,  377,  365,  150,
 /*  1870 */  1037,  947, 1037, 1037, 1037,  829,  817,  830,  126, 1037,
 /*  1880 */   949, 1037, 1037, 1037, 1037, 1037, 1037, 1037, 1037, 1037,
 /*  1890 */   131,  374,  882, 1037, 1037, 1037,  239,  199,  883, 1037,
 /*  1900 */  1037,  958,  974,  957,  342,  974,  974,  974,  974,  341,
 /*  1910 */   340,  326, 1037, 1037,  319,   84,   83,   82,   90,   89,
 /*  1920 */    88,   87,   86,   85,   91,   79,   80,   81,   94,   93,
 /*  1930 */    92,  102,  106,  100,   99,   98,   96,   95,   97, 1037,
 /*  1940 */    45,  327,  198,  374, 1037,   64,  112,  111,  110,  109,
 /*  1950 */   950,  951,  952,  953,  954,  955,  116,  115,  199,  374,
 /*  1960 */   239,  131,  883, 1037, 1037,  113,  114, 1037,   34, 1037,
 /*  1970 */  1037, 1037,  963, 1380, 1037, 1037,  370,  881,  295, 1037,
 /*  1980 */   377,  365, 1037,  817,  947, 1037, 1037, 1037, 1037,  280,
 /*  1990 */   251, 1037,  735,  949,  392,  385, 1037,  379, 1037,  377,
 /*  2000 */   365,  150, 1037, 1037, 1037,  882,  374,  829,  817,  830,
 /*  2010 */   126, 1037,  239,  198,  883,  374, 1037, 1037, 1037, 1037,
 /*  2020 */  1037,  199,  374, 1037, 1037, 1037, 1037, 1037,  239,  199,
 /*  2030 */   883, 1037, 1037,  958,  974,  957,  342,  974,  974,  974,
 /*  2040 */   974,  341,  340,  326, 1037, 1037,  319, 1037, 1037, 1037,
 /*  2050 */  1037, 1037, 1037,   45, 1037, 1037,  327, 1037, 1037,  112,
 /*  2060 */   111,  110,  109,  950,  951,  952,  953,  954,  955,  116,
 /*  2070 */   115, 1037,   45,  199,  374,  239,  897,  883,  113,  114,
 /*  2080 */  1037,   34, 1037, 1037,  239,  896,  883,  963, 1037, 1037,
 /*  2090 */   881,  239,  199,  883, 1037, 1037,  327,  947,  817,  985,
 /*  2100 */  1037,   23, 1037, 1037, 1037, 1037,  949, 1037,  252, 1037,
 /*  2110 */  1037, 1037, 1037,  199,  374, 1037, 1037, 1037,  882, 1037,
 /*  2120 */  1037, 1037,  829,  817,  830,  126, 1037,  963, 1037, 1037,
 /*  2130 */  1037, 1037, 1037, 1037,  198,  374, 1037, 1037,  817, 1037,
 /*  2140 */  1037, 1037, 1037,  239,  199,  883, 1037, 1037,  958,  990,
 /*  2150 */   957,  342,  990,  990,  990,  990,  341,  340,  326,  374,
 /*  2160 */  1037,  319,  829,  817,  830,  126, 1037, 1037, 1037,  327,
 /*  2170 */   198,  374,  112,  111,  110,  109,  950,  951,  952,  953,
 /*  2180 */   954,  955,  254,  239,  199,  883,  199,  374,  958,  989,
 /*  2190 */   957,  342,  989,  989,  989,  989,  341,  340,  326, 1037,
 /*  2200 */   963,  319, 1037, 1037,  239,  198,  883,  194,  374, 1037,
 /*  2210 */  1037,  817, 1037,  196,  374, 1037,  183,  374,  255, 1037,
 /*  2220 */  1037, 1037,  721,  184,  374, 1037, 1037, 1037,  239,  895,
 /*  2230 */   883, 1037,  722,  723, 1037,  829,  817,  830,  126, 1037,
 /*  2240 */   239,  198,  883, 1037, 1037, 1037,  185,  374, 1037, 1037,
 /*  2250 */   199,  374,  724, 1037, 1037,  720,  239,  199,  883, 1037,
 /*  2260 */  1037,  958,  988,  957,  342,  988,  988,  988,  988,  341,
 /*  2270 */   340,  326, 1037,  327,  319,  817,  374,  239,  194,  883,
 /*  2280 */  1037, 1037, 1037,  239,  196,  883,  239,  183,  883, 1037,
 /*  2290 */   199,  374, 1037,  239,  184,  883, 1037, 1037,   16,  829,
 /*  2300 */   817,  830,  126, 1037,  963, 1037, 1037, 1037, 1037, 1037,
 /*  2310 */  1037, 1037, 1037,  327, 1037,  817,  239,  185,  883, 1037,
 /*  2320 */   239,  199,  883, 1037, 1037,  186,  374, 1037,  985, 1037,
 /*  2330 */   199,  374, 1037, 1037,  982, 1037,  323,  311, 1037,  829,
 /*  2340 */   817,  830,  126, 1037,  963,  239,  894,  883, 1037,  187,
 /*  2350 */   374, 1037,  188,  374, 1037,  817,  197,  374, 1037, 1037,
 /*  2360 */   239,  199,  883, 1037, 1037,  958,  979,  957,  342,  979,
 /*  2370 */   979,  979,  979,  341,  340,  326, 1037, 1037,  319,  829,
 /*  2380 */   817,  830,  126, 1037, 1037,  201,  374, 1037, 1037, 1037,
 /*  2390 */   189,  374, 1037, 1037, 1037,  239,  186,  883, 1037, 1037,
 /*  2400 */   239,  199,  883, 1037, 1037,  958,  978,  957,  342,  978,
 /*  2410 */   978,  978,  978,  341,  340,  326,  327, 1037,  319,  239,
 /*  2420 */   187,  883,  239,  188,  883, 1037,  239,  197,  883,  216,
 /*  2430 */   374, 1037, 1037,  199,  374, 1037, 1037, 1037, 1037, 1037,
 /*  2440 */  1037, 1037,  116,  115, 1037, 1037, 1037,  963, 1037, 1037,
 /*  2450 */  1037,  113,  114, 1037,   34,  239,  201,  883,  817, 1037,
 /*  2460 */   239,  189,  883,  375,  190,  374, 1037,  940,  374, 1037,
 /*  2470 */   947,  203,  374, 1037, 1037, 1037, 1037, 1037, 1037,  949,
 /*  2480 */  1037, 1037,  829,  817,  830,  126, 1037, 1037, 1037, 1037,
 /*  2490 */  1037,  882,  204,  374, 1037,  205,  374, 1037, 1037,  239,
 /*  2500 */   216,  883,  327,  239,  199,  883, 1037, 1037,  958,  138,
 /*  2510 */   957,  342,  138,  138,  138,  138,  341,  340,  326,  199,
 /*  2520 */   374,  319, 1037, 1037, 1037, 1037, 1037, 1037, 1037, 1037,
 /*  2530 */  1037, 1037, 1037,  963,  239,  190,  883,  239,  940,  883,
 /*  2540 */  1037,  239,  203,  883,  817,  112,  111,  110,  109,  950,
 /*  2550 */   951,  952,  953,  954,  955, 1037, 1037,  206,  374, 1037,
 /*  2560 */  1037, 1037,  239,  204,  883,  239,  205,  883,  829,  817,
 /*  2570 */   830,  126, 1037,  207,  374, 1037, 1037,  208,  374, 1037,
 /*  2580 */  1037, 1037, 1037, 1037, 1037, 1037,  209,  374, 1037,  239,
 /*  2590 */   199,  883,  210,  374,  958,  976,  957,  342,  976,  976,
 /*  2600 */   976,  976,  341,  340,  326,  116,  115,  319, 1037, 1037,
 /*  2610 */   211,  374, 1037, 1037,  113,  114, 1037,   34, 1037,  212,
 /*  2620 */   374, 1037, 1037, 1037,  213,  374,  881,  239,  206,  883,
 /*  2630 */  1037,  214,  374,  947,  217,  374, 1037, 1037, 1037, 1037,
 /*  2640 */  1037, 1037,  949,  239,  207,  883, 1037,  239,  208,  883,
 /*  2650 */  1037, 1037, 1037, 1037,  882, 1037,  239,  209,  883,  218,
 /*  2660 */   374, 1037,  239,  210,  883, 1037, 1037, 1037,  229,  374,
 /*  2670 */  1037, 1037,  230,  374, 1037, 1037,  231,  374, 1037, 1037,
 /*  2680 */   239,  211,  883, 1037, 1037,  219,  374, 1037, 1037,  239,
 /*  2690 */   212,  883,  220,  374,  239,  213,  883,  221,  374, 1037,
 /*  2700 */  1037,  239,  214,  883,  239,  217,  883, 1037,  112,  111,
 /*  2710 */   110,  109,  950,  951,  952,  953,  954,  955,  222,  374,
 /*  2720 */  1037,  223,  374, 1037,  224,  374, 1037,  225,  374,  239,
 /*  2730 */   218,  883, 1037, 1037,  226,  374, 1037, 1037,  239,  229,
 /*  2740 */   883, 1037,  239,  230,  883, 1037,  239,  231,  883, 1037,
 /*  2750 */   227,  374, 1037, 1037, 1037,  239,  219,  883, 1037, 1037,
 /*  2760 */  1037, 1037,  239,  220,  883, 1037, 1037,  239,  221,  883,
 /*  2770 */   228,  374, 1037,  232,  374, 1037,  233,  374, 1037,  234,
 /*  2780 */   374, 1037,  262,  374, 1037, 1037, 1037, 1037,  239,  222,
 /*  2790 */   883,  239,  223,  883,  239,  224,  883,  239,  225,  883,
 /*  2800 */  1037,  263,  374, 1037,  239,  226,  883, 1037,  373,  374,
 /*  2810 */  1037,  264,  374, 1037, 1037, 1037, 1037, 1037, 1037, 1037,
 /*  2820 */   239,  227,  883, 1037,  265,  374, 1037, 1037, 1037, 1037,
 /*  2830 */  1037,  266,  374, 1037,  215,  374, 1037,  236,  374, 1037,
 /*  2840 */   239,  228,  883,  239,  232,  883,  239,  233,  883,  239,
 /*  2850 */   234,  883,  239,  262,  883, 1037, 1037,  191,  374, 1037,
 /*  2860 */  1037, 1037,  178,  374, 1037, 1037,  179,  374, 1037,  235,
 /*  2870 */   374,  239,  263,  883,  192,  374, 1037, 1037,  239,  373,
 /*  2880 */   883,  239,  264,  883, 1037, 1037, 1037, 1037, 1037, 1037,
 /*  2890 */  1037, 1037, 1037, 1037,  239,  265,  883, 1037, 1037, 1037,
 /*  2900 */  1037,  239,  266,  883,  239,  215,  883,  239,  236,  883,
 /*  2910 */  1037, 1037, 1037, 1037, 1037, 1037, 1037, 1037, 1037, 1037,
 /*  2920 */  1037, 1037, 1037, 1037, 1037, 1037, 1037,  239,  191,  883,
 /*  2930 */  1037, 1037,  239,  178,  883, 1037,  239,  179,  883,  239,
 /*  2940 */   235,  883, 1037, 1037,  239,  192,  883,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    33,   34,  166,  167,  168,  169,  170,  171,  172,   42,
 /*    10 */    43,   44,   45,   46,  178,  201,   49,  211,  212,   49,
 /*    20 */    53,   54,  206,  207,  208,  209,  201,   54,   61,   40,
 /*    30 */    63,   64,   65,   66,   44,   65,   47,   70,  202,  225,
 /*    40 */   226,  227,  228,   53,   54,  231,  233,  234,   62,   82,
 /*    50 */   225,  226,  227,  228,   64,   82,  231,   90,   91,   92,
 /*    60 */    93,   94,   95,   96,   97,   98,   99,  100,  101,  102,
 /*    70 */   103,  104,  105,  106,  107,  159,  109,  110,  111,   54,
 /*    80 */    90,   91,   92,   93,   94,   95,   96,   97,   98,   99,
 /*    90 */   100,  101,  102,  103,  104,  105,  106,  107,  112,  109,
 /*   100 */   110,  111,  159,  136,  137,  138,  139,  140,  141,  142,
 /*   110 */   143,  144,  145,  146,  147,  148,  149,  150,  151,  152,
 /*   120 */   153,  154,  155,   54,  157,  158,   30,   31,   32,   33,
 /*   130 */    34,   35,   36,   37,   38,   39,   40,   33,   34,  159,
 /*   140 */   197,  198,  199,   47,  164,  165,   42,   43,   44,   45,
 /*   150 */    46,  235,  160,   49,   34,  239,  240,   53,   54,  176,
 /*   160 */   177,  169,  170,  171,   44,   61,   54,   63,   64,   65,
 /*   170 */    66,  132,  180,   50,   70,   63,   64,  185,  186,  187,
 /*   180 */   188,  189,  190,  162,  163,   62,   82,   90,   91,   92,
 /*   190 */    93,   94,   95,   50,   90,   91,   92,   93,   94,   95,
 /*   200 */    96,   97,   98,   99,  100,  101,  102,  103,  104,  105,
 /*   210 */   106,  107,   21,  109,  110,  111,   29,   30,   31,   32,
 /*   220 */    33,   34,   35,   36,   37,   38,   39,   40,  236,  246,
 /*   230 */   247,  248,  249,  250,   47,   44,  175,  176,  177,  201,
 /*   240 */   136,  137,  138,  139,  140,  141,  142,  143,  144,  145,
 /*   250 */   146,  147,  148,  149,  150,  151,  152,  153,  154,  155,
 /*   260 */   117,  157,  158,  225,  226,  227,  228,  229,  230,  231,
 /*   270 */   237,   54,  251,   46,   33,   34,  243,  244,  245,  241,
 /*   280 */   159,  159,  160,   42,   43,   44,   45,   46,  159,  160,
 /*   290 */    49,  159,  170,  171,   53,   54,  214,  176,  177,  170,
 /*   300 */   171,   54,   61,  201,   63,   64,   65,  246,  247,  248,
 /*   310 */   210,   70,  190,  191,  192,  193,  184,  233,  234,  190,
 /*   320 */   191,  192,  193,   82,   55,   56,   57,   58,   59,   60,
 /*   330 */     1,   90,   91,   92,   93,   94,   95,   96,   97,   98,
 /*   340 */    99,  100,  101,  102,  103,  104,  105,  106,  107,   66,
 /*   350 */   109,  110,  111,  221,  222,  223,  176,  177,  236,   76,
 /*   360 */    77,   78,   79,   80,   81,  236,  210,  246,  247,  248,
 /*   370 */    42,   43,   44,   45,   46,   46,  210,  136,  137,  138,
 /*   380 */   139,  140,  141,  142,  143,  144,  145,  146,  147,  148,
 /*   390 */   149,  150,  151,  152,  153,  154,  155,   44,  157,  158,
 /*   400 */    49,   54,   45,   46,  224,   14,   53,   54,   55,   56,
 /*   410 */    57,   58,   65,   60,  244,  245,   65,   64,  176,  177,
 /*   420 */   194,  195,  159,  265,  266,   72,  246,  247,  248,  184,
 /*   430 */    83,   84,   85,   86,   87,   88,   89,  233,  234,  176,
 /*   440 */   177,  153,  154,   90,   91,   92,   93,   94,   95,   96,
 /*   450 */    97,   98,   99,  100,  101,  102,  103,  104,  105,  106,
 /*   460 */   107,  219,  109,  110,  111,  112,    0,   76,  159,  159,
 /*   470 */   234,  118,  119,  120,  121,  122,  123,  124,  125,  126,
 /*   480 */   127,  128,  129,  130,  131,   33,   34,  224,  246,  247,
 /*   490 */   248,  249,  250,  184,   42,   43,   44,   45,   49,   50,
 /*   500 */    66,  261,  181,   53,   46,   53,   54,   73,   74,  246,
 /*   510 */   247,  248,   54,   61,   48,  205,   64,   51,   52,   53,
 /*   520 */     1,   71,   70,   35,   36,   37,   38,   39,   40,  220,
 /*   530 */   221,  222,  223,   67,   82,   47,   54,   71,  184,   73,
 /*   540 */    74,   14,   90,   91,   92,   93,   94,   95,   96,   97,
 /*   550 */    98,   99,  100,  101,  102,  103,  104,  105,  106,  107,
 /*   560 */    49,  109,  110,  111,   50,   46,  184,   33,   34,   35,
 /*   570 */    36,   37,   38,   39,   40,   50,   62,   66,  176,  177,
 /*   580 */   146,   47,  175,  176,  177,   49,   50,   62,  136,  137,
 /*   590 */   138,  139,  140,  141,  142,  143,  144,  145,    1,    2,
 /*   600 */     3,    4,    5,    6,    7,    8,    9,   10,   11,   12,
 /*   610 */    13,  182,   15,   16,   17,   18,   19,   20,   21,   22,
 /*   620 */    23,   24,   25,   26,   27,   28,   29,   30,   31,   32,
 /*   630 */    33,   34,   35,   36,   37,   38,   39,   40,  184,   50,
 /*   640 */    50,   50,   50,   49,   47,  176,  177,   50,  246,  247,
 /*   650 */   248,   62,   62,  246,  247,  248,  254,   66,   66,   62,
 /*   660 */    66,    1,    2,    3,    4,    5,    6,    7,    8,    9,
 /*   670 */    10,   11,   12,   13,  184,   15,   16,   17,   18,   19,
 /*   680 */    20,   21,   22,   23,   24,   25,   26,   27,   28,   29,
 /*   690 */    30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
 /*   700 */    40,   49,   50,   45,   56,   57,   58,   47,   60,  177,
 /*   710 */    50,  176,  177,   65,  184,  246,  247,  248,  213,   54,
 /*   720 */    72,   50,   62,  184,    1,    2,    3,    4,    5,    6,
 /*   730 */     7,    8,    9,   10,   11,   12,   13,   66,   15,   16,
 /*   740 */    17,   18,   19,   20,   21,   22,   23,   24,   25,   26,
 /*   750 */    27,   28,   29,   30,   31,   32,   33,   34,   35,   36,
 /*   760 */    37,   38,   39,   40,   50,   49,  216,   61,   50,  217,
 /*   770 */    47,  106,   45,   50,  215,   49,   62,  173,  246,  247,
 /*   780 */   248,  246,  247,  248,   66,   62,   50,    1,    2,    3,
 /*   790 */     4,    5,    6,    7,    8,    9,   10,   11,   12,   13,
 /*   800 */   242,   15,   16,   17,   18,   19,   20,   21,   22,   23,
 /*   810 */    24,   25,   26,   27,   28,   29,   30,   31,   32,   33,
 /*   820 */    34,   35,   36,   37,   38,   39,   40,   54,  195,  215,
 /*   830 */    54,  266,   73,   47,  238,    1,    2,    3,    4,    5,
 /*   840 */     6,    7,    8,    9,   10,   11,   12,   13,   62,   15,
 /*   850 */    16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
 /*   860 */    26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
 /*   870 */    36,   37,   38,   39,   40,  240,  240,  184,   20,  232,
 /*   880 */   184,   47,  199,   50,   50,  174,    1,    2,    3,    4,
 /*   890 */     5,    6,    7,    8,    9,   10,   11,   12,   13,   14,
 /*   900 */    15,   16,   17,   18,   19,   20,   21,   22,   23,   24,
 /*   910 */    25,   26,   27,   28,   29,   30,   31,   32,   33,   34,
 /*   920 */    35,   36,   37,   38,   39,   40,  173,   14,  200,  173,
 /*   930 */    61,  177,   47,  203,    1,    2,    3,    4,    5,    6,
 /*   940 */     7,    8,    9,   10,   11,   12,   13,  183,   15,   16,
 /*   950 */    17,   18,   19,   20,   21,   22,   23,   24,   25,   26,
 /*   960 */    27,   28,   29,   30,   31,   32,   33,   34,   35,   36,
 /*   970 */    37,   38,   39,   40,  179,  196,  216,  216,  216,  179,
 /*   980 */    47,   65,    1,    2,    3,    4,    5,    6,    7,    8,
 /*   990 */     9,   10,   11,   12,   13,   62,   15,   16,   17,   18,
 /*  1000 */    19,   20,   21,   22,   23,   24,   25,   26,   27,   28,
 /*  1010 */    29,   30,   31,   32,   33,   34,   35,   36,   37,   38,
 /*  1020 */    39,   40,   54,   53,   44,   62,   61,   45,   47,   54,
 /*  1030 */     1,    2,    3,    4,    5,    6,    7,    8,    9,   10,
 /*  1040 */    11,   12,   13,   62,   15,   16,   17,   18,   19,   20,
 /*  1050 */    21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
 /*  1060 */    31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
 /*  1070 */    54,   49,   65,   47,   47,   66,   47,   49,    1,    2,
 /*  1080 */     3,    4,    5,    6,    7,    8,    9,   10,   11,   12,
 /*  1090 */    13,   62,   15,   16,   17,   18,   19,   20,   21,   22,
 /*  1100 */    23,   24,   25,   26,   27,   28,   29,   30,   31,   32,
 /*  1110 */    33,   34,   35,   36,   37,   38,   39,   40,   49,   66,
 /*  1120 */    66,   61,   65,   45,   47,   45,    1,    2,    3,    4,
 /*  1130 */     5,    6,    7,    8,    9,   10,   11,   12,   13,   62,
 /*  1140 */    15,   16,   17,   18,   19,   20,   21,   22,   23,   24,
 /*  1150 */    25,   26,   27,   28,   29,   30,   31,   32,   33,   34,
 /*  1160 */    35,   36,   37,   38,   39,   40,   66,   65,   54,   71,
 /*  1170 */    62,   61,   47,   45,    1,    2,    3,    4,    5,    6,
 /*  1180 */     7,    8,    9,   10,   11,   12,   13,   62,   15,   16,
 /*  1190 */    17,   18,   19,   20,   21,   22,   23,   24,   25,   26,
 /*  1200 */    27,   28,   29,   30,   31,   32,   33,   34,   35,   36,
 /*  1210 */    37,   38,   39,   40,   62,   61,  176,  177,   62,   45,
 /*  1220 */    47,   61,    1,    2,    3,    4,    5,    6,    7,    8,
 /*  1230 */     9,   10,   11,   12,   13,   14,   15,   16,   17,   18,
 /*  1240 */    19,   20,   21,   22,   23,   24,   25,   26,   27,   28,
 /*  1250 */    29,   30,   31,   32,   33,   34,   35,   36,   37,   38,
 /*  1260 */    39,   40,  176,  177,  224,   61,   54,   45,   47,   49,
 /*  1270 */    26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
 /*  1280 */    36,   37,   38,   39,   40,    1,  246,  247,  248,   54,
 /*  1290 */   117,   47,    1,    2,    3,    4,    5,    6,    7,    8,
 /*  1300 */     9,   10,   11,   12,   13,  219,   15,   16,   17,   18,
 /*  1310 */    19,   20,   21,   22,   23,   24,   25,   26,   27,   28,
 /*  1320 */    29,   30,   31,   32,   33,   34,   35,   36,   37,   38,
 /*  1330 */    39,   40,  246,  247,  248,  249,  250,   50,   47,   50,
 /*  1340 */     1,    2,    3,    4,    5,    6,    7,    8,    9,   10,
 /*  1350 */    11,   12,   13,   62,   15,   16,   17,   18,   19,   20,
 /*  1360 */    21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
 /*  1370 */    31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
 /*  1380 */    66,   65,   50,    1,   14,   45,   47,   45,   49,  156,
 /*  1390 */     1,    2,    3,    4,    5,    6,    7,    8,    9,   10,
 /*  1400 */    11,   12,   13,   45,   15,   16,   17,   18,   19,   20,
 /*  1410 */    21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
 /*  1420 */    31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
 /*  1430 */    14,   44,   49,   49,   49,   45,   47,   45,   45,   65,
 /*  1440 */    53,   54,    1,  201,   49,   49,  117,   49,   54,    1,
 /*  1450 */   117,   64,   27,   28,   29,   30,   31,   32,   33,   34,
 /*  1460 */    35,   36,   37,   38,   39,   40,   65,  225,  226,  227,
 /*  1470 */   228,   49,   47,  231,   46,   49,   49,   90,   91,   92,
 /*  1480 */    93,   94,   95,   96,   97,   98,   99,  100,  101,  102,
 /*  1490 */   103,  104,  105,  106,  107,  177,  109,  110,  111,   20,
 /*  1500 */    21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
 /*  1510 */    31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
 /*  1520 */   133,  134,  135,   54,  176,  177,   47,   62,   50,  117,
 /*  1530 */    21,   20,   50,   21,   13,   20,   15,   16,   17,   18,
 /*  1540 */    19,   20,   21,   22,   23,   24,   25,   26,   27,   28,
 /*  1550 */    29,   30,   31,   32,   33,   34,   35,   36,   37,   38,
 /*  1560 */    39,   40,  159,  201,  246,  247,  248,  219,   47,   21,
 /*  1570 */    33,   34,   50,   20,   20,    1,  176,  177,   21,  176,
 /*  1580 */   177,  201,   54,   21,   54,   66,   49,  225,  226,  227,
 /*  1590 */   228,   54,   65,  190,  246,  247,  248,  249,  250,   54,
 /*  1600 */    45,   52,   62,   66,  201,  225,  226,  227,  228,   21,
 /*  1610 */    54,  231,   63,   20,   62,   66,   67,   68,   69,  216,
 /*  1620 */    71,  218,   73,   74,   75,   54,   47,   61,  225,  226,
 /*  1630 */   227,  228,  177,   28,   29,   30,   31,   32,   33,   34,
 /*  1640 */    35,   36,   37,   38,   39,   40,  246,  247,  248,  246,
 /*  1650 */   247,  248,   47,   14,  251,  252,  253,  254,  255,  256,
 /*  1660 */   257,  258,  259,  260,  261,    1,   54,  264,   16,   17,
 /*  1670 */    18,   19,   20,   21,   22,   23,   24,   25,   26,   27,
 /*  1680 */    28,   29,   30,   31,   32,   33,   34,   35,   36,   37,
 /*  1690 */    38,   39,   40,   66,  159,  146,   54,   49,   70,   47,
 /*  1700 */    54,  246,  247,  248,  160,   33,   34,   33,   34,   50,
 /*  1710 */    14,  176,  177,  169,  170,  171,   42,   43,   44,   45,
 /*  1720 */    54,   49,   54,   49,   49,  190,   54,   50,   54,   50,
 /*  1730 */   186,  187,  188,  189,  190,   61,  201,   50,   66,   14,
 /*  1740 */   201,   49,   54,   54,   70,  267,  267,  201,  204,  267,
 /*  1750 */   267,  267,  267,  218,  267,  267,   82,  267,  176,  177,
 /*  1760 */   225,  226,  227,  228,  225,  226,  227,  228,  177,  267,
 /*  1770 */   231,  225,  226,  227,  228,  267,  267,  231,  176,  177,
 /*  1780 */   236,  246,  247,  248,  267,  267,  251,  252,  253,  254,
 /*  1790 */   255,  256,  257,  258,  259,  260,  261,  201,  267,  264,
 /*  1800 */   267,  267,  267,  267,  267,  267,  224,  267,  267,  159,
 /*  1810 */   136,  137,  138,  139,  140,  141,  142,  143,  144,  145,
 /*  1820 */   267,  225,  226,  227,  228,  267,  176,  177,  246,  247,
 /*  1830 */   248,  267,  267,  267,  267,  267,  267,  246,  247,  248,
 /*  1840 */   190,  267,   48,   33,   34,  267,   52,  267,  246,  247,
 /*  1850 */   248,  201,   42,   43,  267,   45,  267,   63,  267,   49,
 /*  1860 */   267,   67,   68,   69,   54,   71,  216,   73,   74,   75,
 /*  1870 */   267,   61,  267,  267,  267,  225,  226,  227,  228,  267,
 /*  1880 */    70,  267,  267,  267,  267,  267,  267,  267,  267,  267,
 /*  1890 */   176,  177,   82,  267,  267,  267,  246,  247,  248,  267,
 /*  1900 */   267,  251,  252,  253,  254,  255,  256,  257,  258,  259,
 /*  1910 */   260,  261,  267,  267,  264,   17,   18,   19,   20,   21,
 /*  1920 */    22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
 /*  1930 */    32,   33,   34,   35,   36,   37,   38,   39,   40,  267,
 /*  1940 */   146,  159,  176,  177,  267,   47,  136,  137,  138,  139,
 /*  1950 */   140,  141,  142,  143,  144,  145,   33,   34,  176,  177,
 /*  1960 */   246,  247,  248,  267,  267,   42,   43,  267,   45,  267,
 /*  1970 */   267,  267,  190,   66,  267,  267,   53,   54,   52,  267,
 /*  1980 */    73,   74,  267,  201,   61,  267,  267,  267,  267,   63,
 /*  1990 */   224,  267,   66,   70,   68,   69,  267,   71,  267,   73,
 /*  2000 */    74,   75,  267,  267,  267,   82,  177,  225,  226,  227,
 /*  2010 */   228,  267,  246,  247,  248,  177,  267,  267,  267,  267,
 /*  2020 */   267,  176,  177,  267,  267,  267,  267,  267,  246,  247,
 /*  2030 */   248,  267,  267,  251,  252,  253,  254,  255,  256,  257,
 /*  2040 */   258,  259,  260,  261,  267,  267,  264,  267,  267,  267,
 /*  2050 */   267,  267,  267,  146,  267,  267,  159,  267,  267,  136,
 /*  2060 */   137,  138,  139,  140,  141,  142,  143,  144,  145,   33,
 /*  2070 */    34,  267,  146,  176,  177,  246,  247,  248,   42,   43,
 /*  2080 */   267,   45,  267,  267,  246,  247,  248,  190,  267,  267,
 /*  2090 */    54,  246,  247,  248,  267,  267,  159,   61,  201,  254,
 /*  2100 */   267,   65,  267,  267,  267,  267,   70,  267,  263,  267,
 /*  2110 */   267,  267,  267,  176,  177,  267,  267,  267,   82,  267,
 /*  2120 */   267,  267,  225,  226,  227,  228,  267,  190,  267,  267,
 /*  2130 */   267,  267,  267,  267,  176,  177,  267,  267,  201,  267,
 /*  2140 */   267,  267,  267,  246,  247,  248,  267,  267,  251,  252,
 /*  2150 */   253,  254,  255,  256,  257,  258,  259,  260,  261,  177,
 /*  2160 */   267,  264,  225,  226,  227,  228,  267,  267,  267,  159,
 /*  2170 */   176,  177,  136,  137,  138,  139,  140,  141,  142,  143,
 /*  2180 */   144,  145,  224,  246,  247,  248,  176,  177,  251,  252,
 /*  2190 */   253,  254,  255,  256,  257,  258,  259,  260,  261,  267,
 /*  2200 */   190,  264,  267,  267,  246,  247,  248,  176,  177,  267,
 /*  2210 */   267,  201,  267,  176,  177,  267,  176,  177,  224,  267,
 /*  2220 */   267,  267,  160,  176,  177,  267,  267,  267,  246,  247,
 /*  2230 */   248,  267,  170,  171,  267,  225,  226,  227,  228,  267,
 /*  2240 */   246,  247,  248,  267,  267,  267,  176,  177,  267,  267,
 /*  2250 */   176,  177,  190,  267,  267,  193,  246,  247,  248,  267,
 /*  2260 */   267,  251,  252,  253,  254,  255,  256,  257,  258,  259,
 /*  2270 */   260,  261,  267,  159,  264,  201,  177,  246,  247,  248,
 /*  2280 */   267,  267,  267,  246,  247,  248,  246,  247,  248,  267,
 /*  2290 */   176,  177,  267,  246,  247,  248,  267,  267,  236,  225,
 /*  2300 */   226,  227,  228,  267,  190,  267,  267,  267,  267,  267,
 /*  2310 */   267,  267,  267,  159,  267,  201,  246,  247,  248,  267,
 /*  2320 */   246,  247,  248,  267,  267,  176,  177,  267,  254,  267,
 /*  2330 */   176,  177,  267,  267,  260,  267,  262,  263,  267,  225,
 /*  2340 */   226,  227,  228,  267,  190,  246,  247,  248,  267,  176,
 /*  2350 */   177,  267,  176,  177,  267,  201,  176,  177,  267,  267,
 /*  2360 */   246,  247,  248,  267,  267,  251,  252,  253,  254,  255,
 /*  2370 */   256,  257,  258,  259,  260,  261,  267,  267,  264,  225,
 /*  2380 */   226,  227,  228,  267,  267,  176,  177,  267,  267,  267,
 /*  2390 */   176,  177,  267,  267,  267,  246,  247,  248,  267,  267,
 /*  2400 */   246,  247,  248,  267,  267,  251,  252,  253,  254,  255,
 /*  2410 */   256,  257,  258,  259,  260,  261,  159,  267,  264,  246,
 /*  2420 */   247,  248,  246,  247,  248,  267,  246,  247,  248,  176,
 /*  2430 */   177,  267,  267,  176,  177,  267,  267,  267,  267,  267,
 /*  2440 */   267,  267,   33,   34,  267,  267,  267,  190,  267,  267,
 /*  2450 */   267,   42,   43,  267,   45,  246,  247,  248,  201,  267,
 /*  2460 */   246,  247,  248,   54,  176,  177,  267,  176,  177,  267,
 /*  2470 */    61,  176,  177,  267,  267,  267,  267,  267,  267,   70,
 /*  2480 */   267,  267,  225,  226,  227,  228,  267,  267,  267,  267,
 /*  2490 */   267,   82,  176,  177,  267,  176,  177,  267,  267,  246,
 /*  2500 */   247,  248,  159,  246,  247,  248,  267,  267,  251,  252,
 /*  2510 */   253,  254,  255,  256,  257,  258,  259,  260,  261,  176,
 /*  2520 */   177,  264,  267,  267,  267,  267,  267,  267,  267,  267,
 /*  2530 */   267,  267,  267,  190,  246,  247,  248,  246,  247,  248,
 /*  2540 */   267,  246,  247,  248,  201,  136,  137,  138,  139,  140,
 /*  2550 */   141,  142,  143,  144,  145,  267,  267,  176,  177,  267,
 /*  2560 */   267,  267,  246,  247,  248,  246,  247,  248,  225,  226,
 /*  2570 */   227,  228,  267,  176,  177,  267,  267,  176,  177,  267,
 /*  2580 */   267,  267,  267,  267,  267,  267,  176,  177,  267,  246,
 /*  2590 */   247,  248,  176,  177,  251,  252,  253,  254,  255,  256,
 /*  2600 */   257,  258,  259,  260,  261,   33,   34,  264,  267,  267,
 /*  2610 */   176,  177,  267,  267,   42,   43,  267,   45,  267,  176,
 /*  2620 */   177,  267,  267,  267,  176,  177,   54,  246,  247,  248,
 /*  2630 */   267,  176,  177,   61,  176,  177,  267,  267,  267,  267,
 /*  2640 */   267,  267,   70,  246,  247,  248,  267,  246,  247,  248,
 /*  2650 */   267,  267,  267,  267,   82,  267,  246,  247,  248,  176,
 /*  2660 */   177,  267,  246,  247,  248,  267,  267,  267,  176,  177,
 /*  2670 */   267,  267,  176,  177,  267,  267,  176,  177,  267,  267,
 /*  2680 */   246,  247,  248,  267,  267,  176,  177,  267,  267,  246,
 /*  2690 */   247,  248,  176,  177,  246,  247,  248,  176,  177,  267,
 /*  2700 */   267,  246,  247,  248,  246,  247,  248,  267,  136,  137,
 /*  2710 */   138,  139,  140,  141,  142,  143,  144,  145,  176,  177,
 /*  2720 */   267,  176,  177,  267,  176,  177,  267,  176,  177,  246,
 /*  2730 */   247,  248,  267,  267,  176,  177,  267,  267,  246,  247,
 /*  2740 */   248,  267,  246,  247,  248,  267,  246,  247,  248,  267,
 /*  2750 */   176,  177,  267,  267,  267,  246,  247,  248,  267,  267,
 /*  2760 */   267,  267,  246,  247,  248,  267,  267,  246,  247,  248,
 /*  2770 */   176,  177,  267,  176,  177,  267,  176,  177,  267,  176,
 /*  2780 */   177,  267,  176,  177,  267,  267,  267,  267,  246,  247,
 /*  2790 */   248,  246,  247,  248,  246,  247,  248,  246,  247,  248,
 /*  2800 */   267,  176,  177,  267,  246,  247,  248,  267,  176,  177,
 /*  2810 */   267,  176,  177,  267,  267,  267,  267,  267,  267,  267,
 /*  2820 */   246,  247,  248,  267,  176,  177,  267,  267,  267,  267,
 /*  2830 */   267,  176,  177,  267,  176,  177,  267,  176,  177,  267,
 /*  2840 */   246,  247,  248,  246,  247,  248,  246,  247,  248,  246,
 /*  2850 */   247,  248,  246,  247,  248,  267,  267,  176,  177,  267,
 /*  2860 */   267,  267,  176,  177,  267,  267,  176,  177,  267,  176,
 /*  2870 */   177,  246,  247,  248,  176,  177,  267,  267,  246,  247,
 /*  2880 */   248,  246,  247,  248,  267,  267,  267,  267,  267,  267,
 /*  2890 */   267,  267,  267,  267,  246,  247,  248,  267,  267,  267,
 /*  2900 */   267,  246,  247,  248,  246,  247,  248,  246,  247,  248,
 /*  2910 */   267,  267,  267,  267,  267,  267,  267,  267,  267,  267,
 /*  2920 */   267,  267,  267,  267,  267,  267,  267,  246,  247,  248,
 /*  2930 */   267,  267,  246,  247,  248,  267,  246,  247,  248,  246,
 /*  2940 */   247,  248,  267,  267,  246,  247,  248,
};
#define YY_SHIFT_USE_DFLT (2947)
#define YY_SHIFT_COUNT    (395)
#define YY_SHIFT_MIN      (-33)
#define YY_SHIFT_MAX      (2572)
static const short yy_shift_ofst[] = {
 /*     0 */  2947,  241,  -33,  241,  104,  241,  241,  241,  241,  241,
 /*    10 */   241,  241,  452, 1549, 1794, 1926,  353,  466,  434,  434,
 /*    20 */  2409, 2409, 2409, 2572, 2572, 2409,  -10,  -10,  -10,  -10,
 /*    30 */   -10,  -10, 1674, 1810, 1923, 2572, 2572, 2572, 2572, 2572,
 /*    40 */  2572, 2572, 1537, 1907, 1387,  -10, 2036, 2572, 2572, 2572,
 /*    50 */  2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572,
 /*    60 */  2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572,
 /*    70 */  2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572,
 /*    80 */  2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572,
 /*    90 */  2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572,
 /*   100 */  2572, 2572, 2572, 2572, 2572, 2572, 2572, 2572, 1672, 2572,
 /*   110 */  2572, 2572, 2572, 2572, 2572, 2572, 2572,  283,  -14,   25,
 /*   120 */    69,  -30,  347,  329,  357,   39,  217,  227,  227,  247,
 /*   130 */  2947, 1389,   97,  120,  120,  120,  -27,  519,  288,  351,
 /*   140 */   482,  482,  527,  482,  482,  482,  482,  658,  716,  706,
 /*   150 */   727,  726,  736,  773,  727,  776,  217,  776,  759,  227,
 /*   160 */   227,  482,  858,  482,   69,  833,  726,  913,  726,  869,
 /*   170 */  2947, 2947, 2947, 2947, 2947, 2947, 2947, 2947,  597,  660,
 /*   180 */   723,  786,  834,  885,  933,  981, 1029, 1077, 1125, 1173,
 /*   190 */  1221, 1291, 1339, 1389, 1389, 1389, 1389, 1389, 1389, 1389,
 /*   200 */  1389, 1389, 1389, 1389, 1389, 1389, 1389, 1389, 1389, 1389,
 /*   210 */  1389, 1389, 1389, 1389, 1389, 1389, 1521, 1652, 1898, 1479,
 /*   220 */  1479, 1479, 1244, 1244, 1244, 1244, 1244, 1244, 1425, 1605,
 /*   230 */   187,   96,  534,  534,  534,  488,  488,  269,  648,  328,
 /*   240 */   112,  450,  511,  449,  123,  514,  391,  525,  536,  591,
 /*   250 */   592,  143,  589,  652,  671,  718,  458,  594,  191,  665,
 /*   260 */   590,  714,  -11,  -11,  -11,  -11,  -11,  916,  968,  970,
 /*   270 */   980,  963,  965,  982,  980,  975, 1016,  980,  980, 1022,
 /*   280 */  1007, 1026, 1027, 1028, 1069, 1009, 1053, 1054, 1060, 1078,
 /*   290 */  1080, 1057, 1100, 1102, 1114, 1098, 1108, 1110, 1128, 1152,
 /*   300 */  1154, 1174, 1156, 1160, 1204, 1222, 1212, 1220, 1284, 1235,
 /*   310 */  1287, 1289, 1314, 1316, 1332, 1382, 1370, 1340, 1342, 1233,
 /*   320 */  1416, 1358, 1383, 1384, 1390, 1392, 1393,  716, 1385, 1395,
 /*   330 */  1396, 1374, 1441, 1394, 1329, 1398, 1401, 1448, 1333, 1428,
 /*   340 */  1422, 1426, 1427, 1469, 1465, 1478, 1412, 1509, 1511, 1512,
 /*   350 */  1482, 1515, 1548, 1522, 1553, 1554,  980, 1557, 1528, 1562,
 /*   360 */  1553, 1530, 1574, 1519, 1527, 1545, 1555, 1540, 1588, 1556,
 /*   370 */  1593, 1552, 1571, 1579, 1566, 1639, 1664, 1612, 1627, 1642,
 /*   380 */  1648, 1628, 1659, 1646, 1696, 1666, 1677, 1668, 1679, 1687,
 /*   390 */  1675, 1725, 1688, 1692, 1689, 1566,
};
#define YY_REDUCE_USE_DFLT (-195)
#define YY_REDUCE_COUNT (177)
#define YY_REDUCE_MIN   (-194)
#define YY_REDUCE_MAX   (2698)
static const short yy_reduce_ofst[] = {
 /*     0 */   -20, 1403, 1535, 1650, 1782, 1897, 1937, 2010, 2114, 2154,
 /*    10 */  2257, 2343, 2074,   -8,   -8, 1544,   38, -164,  122,  129,
 /*    20 */   242, 1086, 1348,  263, 1845,  -17, -186, -175, 1242, 1380,
 /*    30 */  1539, 1546,  180, 1040,  121,  402, 1582, 1766,   61, 1958,
 /*    40 */  1994,  407,  309, 2062, 1362, 1596,  469,  535, 1400, 1602,
 /*    50 */  1714, 2031, 2037, 2040, 2047, 2070, 2149, 2173, 2176, 2180,
 /*    60 */  2209, 2214, 2253, 2288, 2291, 2295, 2316, 2319, 2381, 2397,
 /*    70 */  2401, 2410, 2416, 2434, 2443, 2448, 2455, 2458, 2483, 2492,
 /*    80 */  2496, 2500, 2509, 2516, 2521, 2542, 2545, 2548, 2551, 2558,
 /*    90 */  2574, 2594, 2597, 2600, 2603, 2606, 2625, 2632, 2635, 2648,
 /*   100 */  2655, 2658, 2661, 2681, 2686, 2690, 2693, 2698,  132,  532,
 /*   110 */  1318, 1455, 1591, 1829, 1838, 1982, 2099, -184,   33,  -84,
 /*   120 */   -57,   21, -194, -187,   84,  170,  158,   84,  204,  226,
 /*   130 */   310,   82,  102,  100,  156,  166,  245,  236,  240,  321,
 /*   140 */   354,  382,  429,  454,  490,  530,  539,  505,  550,  552,
 /*   150 */   559,  604,  558,  633,  614,  635,  565,  636,  596,  236,
 /*   160 */   236,  693,  647,  696,  683,  711,  753,  728,  756,  754,
 /*   170 */   730,  764,  795,  760,  761,  762,  779,  800,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */  1036, 1124, 1407, 1124, 1407, 1407, 1407, 1407, 1407, 1407,
 /*    10 */  1407, 1407, 1356, 1215, 1215, 1215, 1407, 1407, 1215, 1215,
 /*    20 */  1316, 1316, 1316, 1407, 1356, 1316, 1407, 1407, 1407, 1407,
 /*    30 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1045, 1407,
 /*    40 */  1407, 1045, 1407, 1215, 1407, 1407, 1407, 1407, 1407, 1407,
 /*    50 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*    60 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*    70 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*    80 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*    90 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   100 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   110 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1249, 1407,
 /*   120 */  1103, 1407, 1155, 1367, 1212, 1249, 1407, 1212, 1386, 1407,
 /*   130 */  1125, 1141, 1407, 1138, 1138, 1138, 1407, 1368, 1407, 1407,
 /*   140 */  1407, 1407, 1050, 1407, 1407, 1407, 1407, 1157, 1124, 1407,
 /*   150 */  1120, 1375, 1218, 1407, 1120, 1407, 1407, 1407, 1240, 1213,
 /*   160 */  1203, 1407, 1199, 1407, 1378, 1377, 1375, 1099, 1375, 1407,
 /*   170 */  1108, 1052, 1065, 1124, 1124, 1124, 1084, 1065, 1407, 1407,
 /*   180 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   190 */  1407, 1407, 1407, 1139, 1248, 1369, 1365, 1314, 1397, 1347,
 /*   200 */  1379, 1105, 1318, 1311, 1310, 1309, 1308, 1307, 1306, 1305,
 /*   210 */  1304, 1303, 1302, 1301, 1300, 1317, 1313, 1299, 1298, 1294,
 /*   220 */  1293, 1292, 1291, 1290, 1289, 1288, 1287, 1286, 1285, 1297,
 /*   230 */  1296, 1295, 1284, 1283, 1282, 1274, 1275, 1049, 1407, 1395,
 /*   240 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   250 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   260 */  1407, 1407, 1281, 1280, 1278, 1277, 1276, 1407, 1407, 1407,
 /*   270 */  1051, 1407, 1407, 1407, 1057, 1407, 1407, 1169, 1168, 1407,
 /*   280 */  1407, 1407, 1059, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   290 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   300 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1247, 1407,
 /*   310 */  1392, 1355, 1407, 1407, 1366, 1407, 1407, 1407, 1407, 1359,
 /*   320 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   330 */  1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   340 */  1407, 1407, 1407, 1253, 1407, 1388, 1407, 1407, 1407, 1407,
 /*   350 */  1407, 1407, 1407, 1407, 1407, 1407, 1194, 1407, 1407, 1407,
 /*   360 */  1230, 1407, 1104, 1407, 1407, 1407, 1407, 1407, 1407, 1407,
 /*   370 */  1407, 1407, 1407, 1279, 1400, 1253, 1407, 1407, 1407, 1407,
 /*   380 */  1407, 1407, 1407, 1407, 1407, 1407, 1081, 1407, 1407, 1080,
 /*   390 */  1407, 1407, 1407, 1407, 1407, 1046,
};
/********** End of lemon-generated parsing tables *****************************/

/* The next table maps tokens (terminal symbols) into fallback tokens.  
** If a construct like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
**
** This feature can be used, for example, to cause some keywords in a language
** to revert to identifiers if they keyword does not apply in the context where
** it appears.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
    0,  /*          $ => nothing */
    0,  /*         EQ => nothing */
    0,  /*      MULEQ => nothing */
    0,  /*      DIVEQ => nothing */
    0,  /*      MODEQ => nothing */
    0,  /*      ADDEQ => nothing */
    0,  /*      SUBEQ => nothing */
    0,  /*      LSHEQ => nothing */
    0,  /*      RSHEQ => nothing */
    0,  /*      ANDEQ => nothing */
    0,  /*       OREQ => nothing */
    0,  /*      XOREQ => nothing */
    0,  /*     URSHEQ => nothing */
    0,  /*   QUESTION => nothing */
    0,  /*      COLON => nothing */
    0,  /*       OROR => nothing */
    0,  /*     ANDAND => nothing */
    0,  /*       EQEQ => nothing */
    0,  /*        NEQ => nothing */
    0,  /*   APPROXEQ => nothing */
    0,  /*         LT => nothing */
    0,  /*         GT => nothing */
    0,  /*       LTEQ => nothing */
    0,  /*       GTEQ => nothing */
    0,  /*     LTGTEQ => nothing */
    0,  /*         IS => nothing */
    0,  /*     DOTDOT => nothing */
    0,  /*         OR => nothing */
    0,  /*        XOR => nothing */
    0,  /*        AND => nothing */
    0,  /*        LSH => nothing */
    0,  /*        RSH => nothing */
    0,  /*       URSH => nothing */
    0,  /*        SUB => nothing */
    0,  /*        ADD => nothing */
    0,  /*        MUL => nothing */
    0,  /*        DIV => nothing */
    0,  /*        MOD => nothing */
    0,  /*  CROSSPROD => nothing */
    0,  /*    DOTPROD => nothing */
    0,  /*        POW => nothing */
    0,  /*      UNARY => nothing */
    0,  /*     ADDADD => nothing */
    0,  /*     SUBSUB => nothing */
    0,  /*        DOT => nothing */
    0,  /*     LPAREN => nothing */
    0,  /*   LBRACKET => nothing */
    0,  /*      SCOPE => nothing */
    0,  /*        EOF => nothing */
    0,  /*  SEMICOLON => nothing */
    0,  /*      COMMA => nothing */
    0,  /*    INCLUDE => nothing */
    0,  /*     EXTEND => nothing */
    0,  /*      CLASS => nothing */
    0,  /* IDENTIFIER => nothing */
    0,  /*   ABSTRACT => nothing */
    0,  /*     NATIVE => nothing */
    0,  /*         UI => nothing */
    0,  /*       PLAY => nothing */
    0,  /*   REPLACES => nothing */
    0,  /*    VERSION => nothing */
    0,  /*   STRCONST => nothing */
    0,  /*     RPAREN => nothing */
   54,  /*    DEFAULT => IDENTIFIER */
   54,  /*      COLOR => IDENTIFIER */
    0,  /*     LBRACE => nothing */
    0,  /*     RBRACE => nothing */
    0,  /*      MIXIN => nothing */
   54,  /*   PROPERTY => IDENTIFIER */
    0,  /*    FLAGDEF => nothing */
    0,  /*   INTCONST => nothing */
    0,  /*     STRUCT => nothing */
    0,  /* CLEARSCOPE => nothing */
    0,  /*      CONST => nothing */
    0,  /*       ENUM => nothing */
    0,  /*     STATES => nothing */
    0,  /*        NWS => nothing */
    0,  /*       STOP => nothing */
    0,  /*       WAIT => nothing */
    0,  /*       FAIL => nothing */
    0,  /*       LOOP => nothing */
    0,  /*       GOTO => nothing */
    0,  /*      SUPER => nothing */
    0,  /*     BRIGHT => nothing */
    0,  /*       FAST => nothing */
    0,  /*       SLOW => nothing */
    0,  /*    NODELAY => nothing */
    0,  /*   CANRAISE => nothing */
    0,  /*     OFFSET => nothing */
    0,  /*      LIGHT => nothing */
   54,  /*      SBYTE => IDENTIFIER */
   54,  /*       BYTE => IDENTIFIER */
   54,  /*      SHORT => IDENTIFIER */
   54,  /*     USHORT => IDENTIFIER */
   54,  /*        INT => IDENTIFIER */
   54,  /*       UINT => IDENTIFIER */
   54,  /*       BOOL => IDENTIFIER */
   54,  /*      FLOAT => IDENTIFIER */
   54,  /*     DOUBLE => IDENTIFIER */
   54,  /*    VECTOR2 => IDENTIFIER */
   54,  /*    VECTOR3 => IDENTIFIER */
   54,  /*    VECTOR4 => IDENTIFIER */
   54,  /*       NAME => IDENTIFIER */
   54,  /*      SOUND => IDENTIFIER */
   54,  /*      STATE => IDENTIFIER */
    0,  /*        LET => nothing */
    0,  /*     ATSIGN => nothing */
    0,  /*   READONLY => nothing */
   54,  /*     STRING => IDENTIFIER */
   54,  /*        MAP => IDENTIFIER */
   54,  /* MAPITERATOR => IDENTIFIER */
   54,  /*      ARRAY => IDENTIFIER */
   54,  /*       VOID => IDENTIFIER */
   54,  /*      UINT8 => IDENTIFIER */
   54,  /*       INT8 => IDENTIFIER */
   54,  /*     UINT16 => IDENTIFIER */
   54,  /*      INT16 => IDENTIFIER */
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
**
** After the "shift" half of a SHIFTREDUCE action, the stateno field
** actually contains the reduce action for the second half of the
** SHIFTREDUCE.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number, or reduce action in SHIFTREDUCE */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  yyStackEntry *yytos;          /* Pointer to top element of the stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyhwm;                    /* High-water mark of the stack */
#endif
#ifndef YYNOERRORRECOVERY
  int yyerrcnt;                 /* Shifts left before out of the error */
#endif
  ZCCParseARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
  yyStackEntry yystk0;          /* First stack entry */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
#include <stdio.h>
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void ZCCParseTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "EQ",            "MULEQ",         "DIVEQ",       
  "MODEQ",         "ADDEQ",         "SUBEQ",         "LSHEQ",       
  "RSHEQ",         "ANDEQ",         "OREQ",          "XOREQ",       
  "URSHEQ",        "QUESTION",      "COLON",         "OROR",        
  "ANDAND",        "EQEQ",          "NEQ",           "APPROXEQ",    
  "LT",            "GT",            "LTEQ",          "GTEQ",        
  "LTGTEQ",        "IS",            "DOTDOT",        "OR",          
  "XOR",           "AND",           "LSH",           "RSH",         
  "URSH",          "SUB",           "ADD",           "MUL",         
  "DIV",           "MOD",           "CROSSPROD",     "DOTPROD",     
  "POW",           "UNARY",         "ADDADD",        "SUBSUB",      
  "DOT",           "LPAREN",        "LBRACKET",      "SCOPE",       
  "EOF",           "SEMICOLON",     "COMMA",         "INCLUDE",     
  "EXTEND",        "CLASS",         "IDENTIFIER",    "ABSTRACT",    
  "NATIVE",        "UI",            "PLAY",          "REPLACES",    
  "VERSION",       "STRCONST",      "RPAREN",        "DEFAULT",     
  "COLOR",         "LBRACE",        "RBRACE",        "MIXIN",       
  "PROPERTY",      "FLAGDEF",       "INTCONST",      "STRUCT",      
  "CLEARSCOPE",    "CONST",         "ENUM",          "STATES",      
  "NWS",           "STOP",          "WAIT",          "FAIL",        
  "LOOP",          "GOTO",          "SUPER",         "BRIGHT",      
  "FAST",          "SLOW",          "NODELAY",       "CANRAISE",    
  "OFFSET",        "LIGHT",         "SBYTE",         "BYTE",        
  "SHORT",         "USHORT",        "INT",           "UINT",        
  "BOOL",          "FLOAT",         "DOUBLE",        "VECTOR2",     
  "VECTOR3",       "VECTOR4",       "NAME",          "SOUND",       
  "STATE",         "LET",           "ATSIGN",        "READONLY",    
  "STRING",        "MAP",           "MAPITERATOR",   "ARRAY",       
  "VOID",          "UINT8",         "INT8",          "UINT16",      
  "INT16",         "RBRACKET",      "ACTION",        "DEPRECATED",  
  "STATIC",        "PRIVATE",       "PROTECTED",     "LATENT",      
  "FINAL",         "META",          "TRANSIENT",     "INTERNAL",    
  "VIRTUAL",       "OVERRIDE",      "VARARG",        "VIRTUALSCOPE",
  "ELLIPSIS",      "IN",            "OUT",           "OPTIONAL",    
  "TILDE",         "BANG",          "SIZEOF",        "ALIGNOF",     
  "UINTCONST",     "FLOATCONST",    "NAMECONST",     "FALSE",       
  "TRUE",          "NULLPTR",       "STATICCONST",   "CONTINUE",    
  "BREAK",         "RETURN",        "DO",            "FOR",         
  "FOREACH",       "WHILE",         "UNTIL",         "IF",          
  "ELSE",          "SWITCH",        "CASE",          "error",       
  "declarator",    "declarator_no_fun",  "opt_func_body",  "function_body",
  "main",          "translation_unit",  "external_declaration",  "mixin_definition",
  "class_definition",  "struct_def",    "enum_def",      "const_def",   
  "include_def",   "opt_semicolon",  "opt_comma",     "opt_expr",    
  "expr",          "string_constant",  "class_head",    "class_innards",
  "class_member",  "class_body",    "class_ancestry",  "class_flags", 
  "dottable_id",   "mixin_statement",  "property_def",  "flag_def",    
  "states_def",    "default_def",   "staticarray_statement",  "opt_struct_body",
  "struct_body",   "struct_member",  "identifier_list",  "states_opt",  
  "struct_flags",  "enum_list",     "opt_enum_list",  "enumerator",  
  "enum_type",     "int_type",      "mixin_class_definition",  "mixin_class_body",
  "mixin_class_member",  "states_body",   "state_line",    "state_label", 
  "state_flow",    "state_flow_type",  "state_goto_offset",  "state_action",
  "state_call",    "state_call_params",  "state_opts",    "states_opts", 
  "scanner_mode",  "light_list",    "statement_list",  "func_expr_list",
  "default_statement_list",  "default_statement",  "property_statement",  "flag_statement",
  "expr_list",     "type_name",     "type_name1",    "aggregate_type",
  "type",          "type_list",     "type_list_or_void",  "type_or_array",
  "class_restrictor",  "array_size",    "array_size_expr",  "variables_or_function",
  "decl_flags",    "func_params",   "func_const",    "variable_list",
  "variable_name",  "decl_flag",     "opt_deprecation_message",  "func_param_list",
  "func_param",    "func_param_flags",  "primary",       "unary_expr",  
  "constant",      "func_expr_item",  "named_expr",    "compound_statement",
  "statement",     "labeled_statement",  "expression_statement",  "selection_statement",
  "iteration_statement",  "array_iteration_statement",  "jump_statement",  "assign_statement",
  "local_var",     "while_or_until",  "for_init",      "for_bump",    
  "if_front",      "variable_list_with_init",  "var_init",    
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "main ::= translation_unit",
 /*   1 */ "translation_unit ::=",
 /*   2 */ "translation_unit ::= translation_unit external_declaration",
 /*   3 */ "translation_unit ::= error",
 /*   4 */ "external_declaration ::= mixin_definition",
 /*   5 */ "external_declaration ::= class_definition",
 /*   6 */ "external_declaration ::= struct_def",
 /*   7 */ "external_declaration ::= enum_def",
 /*   8 */ "external_declaration ::= const_def",
 /*   9 */ "external_declaration ::= include_def",
 /*  10 */ "opt_expr ::=",
 /*  11 */ "include_def ::= INCLUDE string_constant",
 /*  12 */ "class_definition ::= class_head class_body",
 /*  13 */ "class_head ::= EXTEND CLASS IDENTIFIER",
 /*  14 */ "class_head ::= CLASS IDENTIFIER class_ancestry class_flags",
 /*  15 */ "class_ancestry ::=",
 /*  16 */ "class_ancestry ::= COLON dottable_id",
 /*  17 */ "class_flags ::=",
 /*  18 */ "class_flags ::= class_flags ABSTRACT",
 /*  19 */ "class_flags ::= class_flags NATIVE",
 /*  20 */ "class_flags ::= class_flags UI",
 /*  21 */ "class_flags ::= class_flags PLAY",
 /*  22 */ "class_flags ::= class_flags REPLACES dottable_id",
 /*  23 */ "class_flags ::= class_flags VERSION LPAREN STRCONST RPAREN",
 /*  24 */ "dottable_id ::= IDENTIFIER",
 /*  25 */ "dottable_id ::= dottable_id DOT IDENTIFIER",
 /*  26 */ "dottable_id ::= dottable_id DOT DEFAULT",
 /*  27 */ "dottable_id ::= dottable_id DOT COLOR",
 /*  28 */ "class_body ::= SEMICOLON class_innards EOF",
 /*  29 */ "class_body ::= LBRACE class_innards RBRACE",
 /*  30 */ "class_innards ::=",
 /*  31 */ "class_innards ::= class_innards class_member",
 /*  32 */ "class_member ::= declarator",
 /*  33 */ "class_member ::= mixin_statement",
 /*  34 */ "class_member ::= enum_def",
 /*  35 */ "class_member ::= struct_def",
 /*  36 */ "class_member ::= states_def",
 /*  37 */ "class_member ::= default_def",
 /*  38 */ "class_member ::= const_def",
 /*  39 */ "class_member ::= property_def",
 /*  40 */ "class_member ::= flag_def",
 /*  41 */ "class_member ::= staticarray_statement",
 /*  42 */ "mixin_statement ::= MIXIN IDENTIFIER SEMICOLON",
 /*  43 */ "property_def ::= PROPERTY IDENTIFIER COLON identifier_list SEMICOLON",
 /*  44 */ "flag_def ::= FLAGDEF IDENTIFIER COLON IDENTIFIER COMMA INTCONST SEMICOLON",
 /*  45 */ "identifier_list ::= IDENTIFIER",
 /*  46 */ "identifier_list ::= states_opt COMMA IDENTIFIER",
 /*  47 */ "struct_def ::= STRUCT IDENTIFIER struct_flags LBRACE opt_struct_body RBRACE opt_semicolon",
 /*  48 */ "struct_def ::= EXTEND STRUCT IDENTIFIER LBRACE opt_struct_body RBRACE opt_semicolon",
 /*  49 */ "struct_flags ::=",
 /*  50 */ "struct_flags ::= struct_flags UI",
 /*  51 */ "struct_flags ::= struct_flags PLAY",
 /*  52 */ "struct_flags ::= struct_flags CLEARSCOPE",
 /*  53 */ "struct_flags ::= struct_flags NATIVE",
 /*  54 */ "struct_flags ::= struct_flags VERSION LPAREN STRCONST RPAREN",
 /*  55 */ "opt_struct_body ::=",
 /*  56 */ "opt_struct_body ::= error",
 /*  57 */ "struct_body ::= struct_body struct_member",
 /*  58 */ "struct_member ::= declarator",
 /*  59 */ "struct_member ::= enum_def",
 /*  60 */ "struct_member ::= const_def",
 /*  61 */ "struct_member ::= staticarray_statement",
 /*  62 */ "const_def ::= CONST IDENTIFIER EQ expr SEMICOLON",
 /*  63 */ "enum_def ::= ENUM IDENTIFIER enum_type LBRACE opt_enum_list RBRACE opt_semicolon",
 /*  64 */ "enum_type ::=",
 /*  65 */ "enum_type ::= COLON int_type",
 /*  66 */ "enum_list ::= error",
 /*  67 */ "enum_list ::= enum_list COMMA enumerator",
 /*  68 */ "opt_enum_list ::=",
 /*  69 */ "enumerator ::= IDENTIFIER",
 /*  70 */ "enumerator ::= IDENTIFIER EQ expr",
 /*  71 */ "mixin_definition ::= mixin_class_definition",
 /*  72 */ "mixin_class_definition ::= MIXIN CLASS IDENTIFIER LBRACE mixin_class_body RBRACE",
 /*  73 */ "mixin_class_body ::=",
 /*  74 */ "mixin_class_body ::= mixin_class_body mixin_class_member",
 /*  75 */ "mixin_class_member ::= declarator",
 /*  76 */ "mixin_class_member ::= enum_def",
 /*  77 */ "mixin_class_member ::= struct_def",
 /*  78 */ "mixin_class_member ::= states_def",
 /*  79 */ "mixin_class_member ::= default_def",
 /*  80 */ "mixin_class_member ::= const_def",
 /*  81 */ "mixin_class_member ::= property_def",
 /*  82 */ "mixin_class_member ::= flag_def",
 /*  83 */ "mixin_class_member ::= staticarray_statement",
 /*  84 */ "states_def ::= STATES states_opts scanner_mode LBRACE states_body RBRACE",
 /*  85 */ "states_opts ::=",
 /*  86 */ "states_opts ::= LPAREN states_opt RPAREN",
 /*  87 */ "states_opt ::= IDENTIFIER",
 /*  88 */ "states_opt ::= states_opt COMMA IDENTIFIER",
 /*  89 */ "scanner_mode ::=",
 /*  90 */ "states_body ::=",
 /*  91 */ "states_body ::= error",
 /*  92 */ "states_body ::= states_body state_line",
 /*  93 */ "states_body ::= states_body state_label",
 /*  94 */ "states_body ::= states_body state_flow",
 /*  95 */ "state_label ::= NWS COLON",
 /*  96 */ "state_flow_type ::= STOP",
 /*  97 */ "state_flow_type ::= WAIT",
 /*  98 */ "state_flow_type ::= FAIL",
 /*  99 */ "state_flow_type ::= LOOP",
 /* 100 */ "state_flow_type ::= GOTO dottable_id state_goto_offset",
 /* 101 */ "state_flow_type ::= GOTO IDENTIFIER SCOPE dottable_id state_goto_offset",
 /* 102 */ "state_flow_type ::= GOTO SUPER SCOPE dottable_id state_goto_offset",
 /* 103 */ "state_goto_offset ::=",
 /* 104 */ "state_goto_offset ::= ADD expr",
 /* 105 */ "state_line ::= NWS NWS expr state_opts state_action",
 /* 106 */ "state_opts ::=",
 /* 107 */ "state_opts ::= state_opts BRIGHT",
 /* 108 */ "state_opts ::= state_opts FAST",
 /* 109 */ "state_opts ::= state_opts SLOW",
 /* 110 */ "state_opts ::= state_opts NODELAY",
 /* 111 */ "state_opts ::= state_opts CANRAISE",
 /* 112 */ "state_opts ::= state_opts OFFSET LPAREN expr COMMA expr RPAREN",
 /* 113 */ "state_opts ::= state_opts LIGHT LPAREN light_list RPAREN",
 /* 114 */ "light_list ::= STRCONST",
 /* 115 */ "light_list ::= light_list COMMA STRCONST",
 /* 116 */ "state_action ::= LBRACE statement_list scanner_mode RBRACE",
 /* 117 */ "state_action ::= LBRACE scanner_mode RBRACE",
 /* 118 */ "state_action ::= LBRACE error scanner_mode RBRACE",
 /* 119 */ "state_action ::= state_call scanner_mode SEMICOLON",
 /* 120 */ "state_call ::=",
 /* 121 */ "state_call ::= IDENTIFIER state_call_params",
 /* 122 */ "state_call_params ::=",
 /* 123 */ "state_call_params ::= LPAREN func_expr_list RPAREN",
 /* 124 */ "default_def ::= DEFAULT LBRACE RBRACE",
 /* 125 */ "default_def ::= DEFAULT LBRACE default_statement_list RBRACE",
 /* 126 */ "default_def ::= DEFAULT LBRACE error RBRACE",
 /* 127 */ "default_statement_list ::= default_statement",
 /* 128 */ "default_statement_list ::= default_statement_list default_statement",
 /* 129 */ "default_statement ::= SEMICOLON",
 /* 130 */ "default_statement ::= error SEMICOLON",
 /* 131 */ "default_statement ::= property_statement",
 /* 132 */ "default_statement ::= flag_statement",
 /* 133 */ "flag_statement ::= ADD dottable_id",
 /* 134 */ "flag_statement ::= SUB dottable_id",
 /* 135 */ "property_statement ::= dottable_id expr_list SEMICOLON",
 /* 136 */ "property_statement ::= dottable_id SEMICOLON",
 /* 137 */ "int_type ::= SBYTE",
 /* 138 */ "int_type ::= BYTE",
 /* 139 */ "int_type ::= SHORT",
 /* 140 */ "int_type ::= USHORT",
 /* 141 */ "int_type ::= INT",
 /* 142 */ "int_type ::= UINT",
 /* 143 */ "type_name1 ::= BOOL",
 /* 144 */ "type_name1 ::= FLOAT",
 /* 145 */ "type_name1 ::= DOUBLE",
 /* 146 */ "type_name1 ::= VECTOR2",
 /* 147 */ "type_name1 ::= VECTOR3",
 /* 148 */ "type_name1 ::= VECTOR4",
 /* 149 */ "type_name1 ::= NAME",
 /* 150 */ "type_name1 ::= SOUND",
 /* 151 */ "type_name1 ::= STATE",
 /* 152 */ "type_name1 ::= COLOR",
 /* 153 */ "type_name1 ::= LET",
 /* 154 */ "type_name ::= type_name1",
 /* 155 */ "type_name ::= IDENTIFIER",
 /* 156 */ "type_name ::= ATSIGN IDENTIFIER",
 /* 157 */ "type_name ::= READONLY LT IDENTIFIER GT",
 /* 158 */ "type_name ::= READONLY LT ATSIGN IDENTIFIER GT",
 /* 159 */ "type_name ::= DOT dottable_id",
 /* 160 */ "aggregate_type ::= MAP LT type_or_array COMMA type_or_array GT",
 /* 161 */ "aggregate_type ::= MAPITERATOR LT type_or_array COMMA type_or_array GT",
 /* 162 */ "aggregate_type ::= ARRAY LT type_or_array GT",
 /* 163 */ "aggregate_type ::= CLASS class_restrictor",
 /* 164 */ "class_restrictor ::=",
 /* 165 */ "class_restrictor ::= LT dottable_id GT",
 /* 166 */ "type ::= type_name",
 /* 167 */ "type ::= aggregate_type",
 /* 168 */ "type_or_array ::= type array_size",
 /* 169 */ "type_list ::= type_list COMMA type_or_array",
 /* 170 */ "type_list_or_void ::= VOID",
 /* 171 */ "array_size_expr ::= LBRACKET opt_expr RBRACKET",
 /* 172 */ "array_size ::= array_size array_size_expr",
 /* 173 */ "declarator ::= decl_flags type_list_or_void variables_or_function",
 /* 174 */ "variables_or_function ::= IDENTIFIER LPAREN func_params RPAREN func_const opt_func_body",
 /* 175 */ "variables_or_function ::= variable_list SEMICOLON",
 /* 176 */ "variables_or_function ::= error SEMICOLON",
 /* 177 */ "variable_name ::= IDENTIFIER",
 /* 178 */ "variable_name ::= IDENTIFIER array_size",
 /* 179 */ "variable_list ::= variable_list COMMA variable_name",
 /* 180 */ "decl_flags ::=",
 /* 181 */ "decl_flags ::= decl_flags decl_flag",
 /* 182 */ "decl_flags ::= decl_flags ACTION states_opts",
 /* 183 */ "opt_deprecation_message ::=",
 /* 184 */ "opt_deprecation_message ::= COMMA STRCONST",
 /* 185 */ "decl_flags ::= decl_flags DEPRECATED LPAREN STRCONST opt_deprecation_message RPAREN",
 /* 186 */ "decl_flags ::= decl_flags VERSION LPAREN STRCONST RPAREN",
 /* 187 */ "decl_flag ::= NATIVE",
 /* 188 */ "decl_flag ::= STATIC",
 /* 189 */ "decl_flag ::= PRIVATE",
 /* 190 */ "decl_flag ::= PROTECTED",
 /* 191 */ "decl_flag ::= LATENT",
 /* 192 */ "decl_flag ::= FINAL",
 /* 193 */ "decl_flag ::= META",
 /* 194 */ "decl_flag ::= TRANSIENT",
 /* 195 */ "decl_flag ::= READONLY",
 /* 196 */ "decl_flag ::= INTERNAL",
 /* 197 */ "decl_flag ::= VIRTUAL",
 /* 198 */ "decl_flag ::= OVERRIDE",
 /* 199 */ "decl_flag ::= ABSTRACT",
 /* 200 */ "decl_flag ::= VARARG",
 /* 201 */ "decl_flag ::= UI",
 /* 202 */ "decl_flag ::= PLAY",
 /* 203 */ "decl_flag ::= CLEARSCOPE",
 /* 204 */ "decl_flag ::= VIRTUALSCOPE",
 /* 205 */ "func_const ::=",
 /* 206 */ "func_const ::= CONST",
 /* 207 */ "opt_func_body ::= SEMICOLON",
 /* 208 */ "func_params ::=",
 /* 209 */ "func_params ::= VOID",
 /* 210 */ "func_params ::= func_param_list COMMA ELLIPSIS",
 /* 211 */ "func_param_list ::= func_param_list COMMA func_param",
 /* 212 */ "func_param ::= func_param_flags type IDENTIFIER",
 /* 213 */ "func_param ::= func_param_flags type IDENTIFIER EQ expr",
 /* 214 */ "func_param_flags ::=",
 /* 215 */ "func_param_flags ::= func_param_flags IN",
 /* 216 */ "func_param_flags ::= func_param_flags OUT",
 /* 217 */ "func_param_flags ::= func_param_flags OPTIONAL",
 /* 218 */ "primary ::= IDENTIFIER",
 /* 219 */ "primary ::= SUPER",
 /* 220 */ "primary ::= constant",
 /* 221 */ "primary ::= LPAREN expr COMMA expr COMMA expr COMMA expr RPAREN",
 /* 222 */ "primary ::= LPAREN expr COMMA expr COMMA expr RPAREN",
 /* 223 */ "primary ::= LPAREN expr COMMA expr RPAREN",
 /* 224 */ "primary ::= LPAREN expr RPAREN",
 /* 225 */ "primary ::= primary LPAREN func_expr_list RPAREN",
 /* 226 */ "primary ::= LPAREN CLASS LT IDENTIFIER GT RPAREN LPAREN func_expr_list RPAREN",
 /* 227 */ "primary ::= primary LBRACKET expr RBRACKET",
 /* 228 */ "primary ::= primary DOT IDENTIFIER",
 /* 229 */ "primary ::= primary ADDADD",
 /* 230 */ "primary ::= primary SUBSUB",
 /* 231 */ "unary_expr ::= SUB unary_expr",
 /* 232 */ "unary_expr ::= ADD unary_expr",
 /* 233 */ "unary_expr ::= SUBSUB unary_expr",
 /* 234 */ "unary_expr ::= ADDADD unary_expr",
 /* 235 */ "unary_expr ::= TILDE unary_expr",
 /* 236 */ "unary_expr ::= BANG unary_expr",
 /* 237 */ "unary_expr ::= SIZEOF unary_expr",
 /* 238 */ "unary_expr ::= ALIGNOF unary_expr",
 /* 239 */ "expr ::= expr ADD expr",
 /* 240 */ "expr ::= expr SUB expr",
 /* 241 */ "expr ::= expr MUL expr",
 /* 242 */ "expr ::= expr DIV expr",
 /* 243 */ "expr ::= expr MOD expr",
 /* 244 */ "expr ::= expr POW expr",
 /* 245 */ "expr ::= expr CROSSPROD expr",
 /* 246 */ "expr ::= expr DOTPROD expr",
 /* 247 */ "expr ::= expr LSH expr",
 /* 248 */ "expr ::= expr RSH expr",
 /* 249 */ "expr ::= expr URSH expr",
 /* 250 */ "expr ::= expr DOTDOT expr",
 /* 251 */ "expr ::= expr LT expr",
 /* 252 */ "expr ::= expr GT expr",
 /* 253 */ "expr ::= expr LTEQ expr",
 /* 254 */ "expr ::= expr GTEQ expr",
 /* 255 */ "expr ::= expr LTGTEQ expr",
 /* 256 */ "expr ::= expr IS expr",
 /* 257 */ "expr ::= expr EQEQ expr",
 /* 258 */ "expr ::= expr NEQ expr",
 /* 259 */ "expr ::= expr APPROXEQ expr",
 /* 260 */ "expr ::= expr AND expr",
 /* 261 */ "expr ::= expr XOR expr",
 /* 262 */ "expr ::= expr OR expr",
 /* 263 */ "expr ::= expr ANDAND expr",
 /* 264 */ "expr ::= expr OROR expr",
 /* 265 */ "expr ::= expr EQ expr",
 /* 266 */ "expr ::= expr ADDEQ expr",
 /* 267 */ "expr ::= expr SUBEQ expr",
 /* 268 */ "expr ::= expr MULEQ expr",
 /* 269 */ "expr ::= expr DIVEQ expr",
 /* 270 */ "expr ::= expr MODEQ expr",
 /* 271 */ "expr ::= expr LSHEQ expr",
 /* 272 */ "expr ::= expr RSHEQ expr",
 /* 273 */ "expr ::= expr URSHEQ expr",
 /* 274 */ "expr ::= expr ANDEQ expr",
 /* 275 */ "expr ::= expr OREQ expr",
 /* 276 */ "expr ::= expr XOREQ expr",
 /* 277 */ "expr ::= expr SCOPE expr",
 /* 278 */ "expr ::= expr QUESTION expr COLON expr",
 /* 279 */ "expr_list ::= expr_list COMMA expr",
 /* 280 */ "func_expr_list ::= func_expr_list COMMA func_expr_item",
 /* 281 */ "func_expr_item ::=",
 /* 282 */ "named_expr ::= IDENTIFIER COLON expr",
 /* 283 */ "named_expr ::= expr",
 /* 284 */ "string_constant ::= STRCONST",
 /* 285 */ "string_constant ::= string_constant STRCONST",
 /* 286 */ "constant ::= INTCONST",
 /* 287 */ "constant ::= UINTCONST",
 /* 288 */ "constant ::= FLOATCONST",
 /* 289 */ "constant ::= NAMECONST",
 /* 290 */ "constant ::= FALSE",
 /* 291 */ "constant ::= TRUE",
 /* 292 */ "constant ::= NULLPTR",
 /* 293 */ "statement ::= SEMICOLON",
 /* 294 */ "statement ::= labeled_statement",
 /* 295 */ "statement ::= compound_statement",
 /* 296 */ "statement ::= expression_statement SEMICOLON",
 /* 297 */ "statement ::= assign_statement SEMICOLON",
 /* 298 */ "statement ::= local_var SEMICOLON",
 /* 299 */ "statement ::= error SEMICOLON",
 /* 300 */ "statement ::= staticarray_statement",
 /* 301 */ "staticarray_statement ::= STATICCONST type IDENTIFIER LBRACKET RBRACKET EQ LBRACE expr_list RBRACE SEMICOLON",
 /* 302 */ "staticarray_statement ::= STATICCONST type LBRACKET RBRACKET IDENTIFIER EQ LBRACE expr_list RBRACE SEMICOLON",
 /* 303 */ "jump_statement ::= CONTINUE SEMICOLON",
 /* 304 */ "jump_statement ::= BREAK SEMICOLON",
 /* 305 */ "jump_statement ::= RETURN SEMICOLON",
 /* 306 */ "jump_statement ::= RETURN expr_list SEMICOLON",
 /* 307 */ "compound_statement ::= LBRACE RBRACE",
 /* 308 */ "compound_statement ::= LBRACE statement_list RBRACE",
 /* 309 */ "compound_statement ::= LBRACE error RBRACE",
 /* 310 */ "statement_list ::= statement",
 /* 311 */ "statement_list ::= statement_list statement",
 /* 312 */ "expression_statement ::= expr",
 /* 313 */ "iteration_statement ::= while_or_until LPAREN expr RPAREN statement",
 /* 314 */ "iteration_statement ::= DO statement while_or_until LPAREN expr RPAREN",
 /* 315 */ "iteration_statement ::= FOR LPAREN for_init SEMICOLON opt_expr SEMICOLON for_bump RPAREN statement",
 /* 316 */ "array_iteration_statement ::= FOREACH LPAREN variable_name COLON expr RPAREN statement",
 /* 317 */ "while_or_until ::= WHILE",
 /* 318 */ "while_or_until ::= UNTIL",
 /* 319 */ "for_init ::= local_var",
 /* 320 */ "for_init ::= for_bump",
 /* 321 */ "for_bump ::=",
 /* 322 */ "for_bump ::= expression_statement",
 /* 323 */ "for_bump ::= for_bump COMMA expression_statement",
 /* 324 */ "selection_statement ::= if_front",
 /* 325 */ "selection_statement ::= if_front ELSE statement",
 /* 326 */ "if_front ::= IF LPAREN expr RPAREN statement",
 /* 327 */ "selection_statement ::= SWITCH LPAREN expr RPAREN statement",
 /* 328 */ "labeled_statement ::= CASE expr COLON",
 /* 329 */ "labeled_statement ::= DEFAULT COLON",
 /* 330 */ "assign_statement ::= LBRACKET expr_list RBRACKET EQ expr",
 /* 331 */ "local_var ::= type variable_list_with_init",
 /* 332 */ "var_init ::= IDENTIFIER",
 /* 333 */ "var_init ::= IDENTIFIER array_size",
 /* 334 */ "var_init ::= IDENTIFIER EQ expr",
 /* 335 */ "var_init ::= IDENTIFIER EQ LBRACE expr_list RBRACE",
 /* 336 */ "var_init ::= IDENTIFIER array_size EQ LBRACE expr_list RBRACE",
 /* 337 */ "var_init ::= IDENTIFIER EQ LBRACE error RBRACE",
 /* 338 */ "variable_list_with_init ::= variable_list_with_init COMMA var_init",
 /* 339 */ "translation_unit ::= translation_unit EOF",
 /* 340 */ "opt_semicolon ::=",
 /* 341 */ "opt_semicolon ::= SEMICOLON",
 /* 342 */ "opt_comma ::=",
 /* 343 */ "opt_comma ::= COMMA",
 /* 344 */ "opt_expr ::= expr",
 /* 345 */ "opt_struct_body ::= struct_body",
 /* 346 */ "struct_body ::= struct_member",
 /* 347 */ "enum_list ::= enumerator",
 /* 348 */ "opt_enum_list ::= enum_list opt_comma",
 /* 349 */ "state_flow ::= state_flow_type scanner_mode SEMICOLON",
 /* 350 */ "type_name1 ::= int_type",
 /* 351 */ "type_or_array ::= type",
 /* 352 */ "type_list ::= type_or_array",
 /* 353 */ "type_list_or_void ::= type_list",
 /* 354 */ "array_size ::= array_size_expr",
 /* 355 */ "variable_list ::= variable_name",
 /* 356 */ "opt_func_body ::= function_body",
 /* 357 */ "func_params ::= func_param_list",
 /* 358 */ "func_param_list ::= func_param",
 /* 359 */ "primary ::= LPAREN error RPAREN",
 /* 360 */ "unary_expr ::= primary",
 /* 361 */ "expr ::= unary_expr",
 /* 362 */ "expr_list ::= expr",
 /* 363 */ "func_expr_list ::= func_expr_item",
 /* 364 */ "func_expr_item ::= named_expr",
 /* 365 */ "constant ::= string_constant",
 /* 366 */ "function_body ::= compound_statement",
 /* 367 */ "statement ::= selection_statement",
 /* 368 */ "statement ::= iteration_statement",
 /* 369 */ "statement ::= array_iteration_statement",
 /* 370 */ "statement ::= jump_statement",
 /* 371 */ "variable_list_with_init ::= var_init",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.  Return the number
** of errors.  Return 0 on success.
*/
static int yyGrowStack(yyParser *p){
  int newSize;
  int idx;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  idx = p->yytos ? (int)(p->yytos - p->yystack) : 0;
  if( p->yystack==&p->yystk0 ){
    pNew = (yyStackEntry *)malloc(newSize*sizeof(pNew[0]));
    if( pNew ) pNew[0] = p->yystk0;
  }else{
    pNew = (yyStackEntry *)realloc(p->yystack, newSize*sizeof(pNew[0]));
  }
  if( pNew ){
    p->yystack = pNew;
    p->yytos = &p->yystack[idx];
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows from %d to %d entries.\n",
              yyTracePrompt, p->yystksz, newSize);
      fflush(yyTraceFILE);
    }
#endif
    p->yystksz = newSize;
  }
  return pNew==0; 
}
#endif

/* Datatype of the argument to the memory allocated passed as the
** second argument to ZCCParseAlloc() below.  This can be changed by
** putting an appropriate #define in the %include section of the input
** grammar.
*/
#ifndef YYMALLOCARGTYPE
# define YYMALLOCARGTYPE size_t
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to ZCCParse and ZCCParseFree.
*/
void *ZCCParseAlloc(void *(CDECL *mallocProc)(YYMALLOCARGTYPE)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (YYMALLOCARGTYPE)sizeof(yyParser) );
  if( pParser ){
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyhwm = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yytos = NULL;
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    if( yyGrowStack(pParser) ){
      pParser->yystack = &pParser->yystk0;
      pParser->yystksz = 1;
    }
#endif
#ifndef YYNOERRORRECOVERY
    pParser->yyerrcnt = -1;
#endif
    pParser->yytos = pParser->yystack;
    pParser->yystack[0].stateno = 0;
    pParser->yystack[0].major = 0;
  }
  return pParser;
}

/* The following function deletes the "minor type" or semantic value
** associated with a symbol.  The symbol can be either a terminal
** or nonterminal. "yymajor" is the symbol code, and "yypminor" is
** a pointer to the value to be deleted.  The code used to do the 
** deletions is derived from the %destructor and/or %token_destructor
** directives of the input grammar.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  ZCCParseARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are *not* used
    ** inside the C code.
    */
/********* Begin destructor definitions ***************************************/
      /* TERMINAL Destructor */
    case 1: /* EQ */
    case 2: /* MULEQ */
    case 3: /* DIVEQ */
    case 4: /* MODEQ */
    case 5: /* ADDEQ */
    case 6: /* SUBEQ */
    case 7: /* LSHEQ */
    case 8: /* RSHEQ */
    case 9: /* ANDEQ */
    case 10: /* OREQ */
    case 11: /* XOREQ */
    case 12: /* URSHEQ */
    case 13: /* QUESTION */
    case 14: /* COLON */
    case 15: /* OROR */
    case 16: /* ANDAND */
    case 17: /* EQEQ */
    case 18: /* NEQ */
    case 19: /* APPROXEQ */
    case 20: /* LT */
    case 21: /* GT */
    case 22: /* LTEQ */
    case 23: /* GTEQ */
    case 24: /* LTGTEQ */
    case 25: /* IS */
    case 26: /* DOTDOT */
    case 27: /* OR */
    case 28: /* XOR */
    case 29: /* AND */
    case 30: /* LSH */
    case 31: /* RSH */
    case 32: /* URSH */
    case 33: /* SUB */
    case 34: /* ADD */
    case 35: /* MUL */
    case 36: /* DIV */
    case 37: /* MOD */
    case 38: /* CROSSPROD */
    case 39: /* DOTPROD */
    case 40: /* POW */
    case 41: /* UNARY */
    case 42: /* ADDADD */
    case 43: /* SUBSUB */
    case 44: /* DOT */
    case 45: /* LPAREN */
    case 46: /* LBRACKET */
    case 47: /* SCOPE */
    case 48: /* EOF */
    case 49: /* SEMICOLON */
    case 50: /* COMMA */
    case 51: /* INCLUDE */
    case 52: /* EXTEND */
    case 53: /* CLASS */
    case 54: /* IDENTIFIER */
    case 55: /* ABSTRACT */
    case 56: /* NATIVE */
    case 57: /* UI */
    case 58: /* PLAY */
    case 59: /* REPLACES */
    case 60: /* VERSION */
    case 61: /* STRCONST */
    case 62: /* RPAREN */
    case 63: /* DEFAULT */
    case 64: /* COLOR */
    case 65: /* LBRACE */
    case 66: /* RBRACE */
    case 67: /* MIXIN */
    case 68: /* PROPERTY */
    case 69: /* FLAGDEF */
    case 70: /* INTCONST */
    case 71: /* STRUCT */
    case 72: /* CLEARSCOPE */
    case 73: /* CONST */
    case 74: /* ENUM */
    case 75: /* STATES */
    case 76: /* NWS */
    case 77: /* STOP */
    case 78: /* WAIT */
    case 79: /* FAIL */
    case 80: /* LOOP */
    case 81: /* GOTO */
    case 82: /* SUPER */
    case 83: /* BRIGHT */
    case 84: /* FAST */
    case 85: /* SLOW */
    case 86: /* NODELAY */
    case 87: /* CANRAISE */
    case 88: /* OFFSET */
    case 89: /* LIGHT */
    case 90: /* SBYTE */
    case 91: /* BYTE */
    case 92: /* SHORT */
    case 93: /* USHORT */
    case 94: /* INT */
    case 95: /* UINT */
    case 96: /* BOOL */
    case 97: /* FLOAT */
    case 98: /* DOUBLE */
    case 99: /* VECTOR2 */
    case 100: /* VECTOR3 */
    case 101: /* VECTOR4 */
    case 102: /* NAME */
    case 103: /* SOUND */
    case 104: /* STATE */
    case 105: /* LET */
    case 106: /* ATSIGN */
    case 107: /* READONLY */
    case 108: /* STRING */
    case 109: /* MAP */
    case 110: /* MAPITERATOR */
    case 111: /* ARRAY */
    case 112: /* VOID */
    case 113: /* UINT8 */
    case 114: /* INT8 */
    case 115: /* UINT16 */
    case 116: /* INT16 */
    case 117: /* RBRACKET */
    case 118: /* ACTION */
    case 119: /* DEPRECATED */
    case 120: /* STATIC */
    case 121: /* PRIVATE */
    case 122: /* PROTECTED */
    case 123: /* LATENT */
    case 124: /* FINAL */
    case 125: /* META */
    case 126: /* TRANSIENT */
    case 127: /* INTERNAL */
    case 128: /* VIRTUAL */
    case 129: /* OVERRIDE */
    case 130: /* VARARG */
    case 131: /* VIRTUALSCOPE */
    case 132: /* ELLIPSIS */
    case 133: /* IN */
    case 134: /* OUT */
    case 135: /* OPTIONAL */
    case 136: /* TILDE */
    case 137: /* BANG */
    case 138: /* SIZEOF */
    case 139: /* ALIGNOF */
    case 140: /* UINTCONST */
    case 141: /* FLOATCONST */
    case 142: /* NAMECONST */
    case 143: /* FALSE */
    case 144: /* TRUE */
    case 145: /* NULLPTR */
    case 146: /* STATICCONST */
    case 147: /* CONTINUE */
    case 148: /* BREAK */
    case 149: /* RETURN */
    case 150: /* DO */
    case 151: /* FOR */
    case 152: /* FOREACH */
    case 153: /* WHILE */
    case 154: /* UNTIL */
    case 155: /* IF */
    case 156: /* ELSE */
    case 157: /* SWITCH */
    case 158: /* CASE */
{
#line 117 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"

#line 1952 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
      break;
/********* End destructor definitions *****************************************/
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
*/
static void yy_pop_parser_stack(yyParser *pParser){
  yyStackEntry *yytos;
  assert( pParser->yytos!=0 );
  assert( pParser->yytos > pParser->yystack );
  yytos = pParser->yytos--;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
    fflush(yyTraceFILE);
  }
#endif
  yy_destructor(pParser, yytos->major, &yytos->minor);
}

/* 
** Deallocate and destroy a parser.  Destructors are called for
** all stack elements before shutting the parser down.
**
** If the YYPARSEFREENEVERNULL macro exists (for example because it
** is defined in a %include section of the input grammar) then it is
** assumed that the input pointer is never NULL.
*/
void ZCCParseFree(
  void *p,                    /* The parser to be deleted */
  void (CDECL *freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
#ifndef YYPARSEFREENEVERNULL
  if( pParser==0 ) return;
#endif
  while( pParser->yytos>pParser->yystack ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  if( pParser->yystack!=&pParser->yystk0 ) free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int ZCCParseStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyhwm;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
*/
static unsigned int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yytos->stateno;
 
  if( stateno>=YY_MIN_REDUCE ) return stateno;
  assert( stateno <= YY_SHIFT_COUNT );
  do{
    i = yy_shift_ofst[stateno];
    assert( iLookAhead!=YYNOCODE );
    i += iLookAhead;
    if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
          fflush(yyTraceFILE);
        }
#endif
        assert( yyFallback[iFallback]==0 ); /* Fallback loop must terminate */
        iLookAhead = iFallback;
        continue;
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( 
#if YY_SHIFT_MIN+YYWILDCARD<0
          j>=0 &&
#endif
#if YY_SHIFT_MAX+YYWILDCARD>=YY_ACTTAB_COUNT
          j<YY_ACTTAB_COUNT &&
#endif
          yy_lookahead[j]==YYWILDCARD && iLookAhead>0
        ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead],
               yyTokenName[YYWILDCARD]);
            fflush(yyTraceFILE);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
      return yy_default[stateno];
    }else{
      return yy_action[i];
    }
  }while(1);
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_COUNT ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_COUNT );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_ACTTAB_COUNT );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser){
   ZCCParseARG_FETCH;
   yypParser->yytos--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
     fflush(yyTraceFILE);
   }
#endif
   while( yypParser->yytos>yypParser->yystack ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */
/******** Begin %stack_overflow code ******************************************/
/******** End %stack_overflow code ********************************************/
   ZCCParseARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Print tracing information for a SHIFT action
*/
#ifndef NDEBUG
static void yyTraceShift(yyParser *yypParser, int yyNewState){
  if( yyTraceFILE ){
    if( yyNewState<YYNSTATE ){
      fprintf(yyTraceFILE,"%sShift '%s', go to state %d\n",
         yyTracePrompt,yyTokenName[yypParser->yytos->major],
         yyNewState);
    }else{
      fprintf(yyTraceFILE,"%sShift '%s'\n",
         yyTracePrompt,yyTokenName[yypParser->yytos->major]);
    }
    fflush(yyTraceFILE);
  }
}
#else
# define yyTraceShift(X,Y)
#endif

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  ZCCParseTOKENTYPE yyMinor        /* The minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yytos++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( (int)(yypParser->yytos - yypParser->yystack)>yypParser->yyhwm ){
    yypParser->yyhwm++;
    assert( yypParser->yyhwm == (int)(yypParser->yytos - yypParser->yystack) );
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yytos>=&yypParser->yystack[YYSTACKDEPTH] ){
    yyStackOverflow(yypParser);
    return;
  }
#else
  if( yypParser->yytos>=&yypParser->yystack[yypParser->yystksz] ){
    if( yyGrowStack(yypParser) ){
      yyStackOverflow(yypParser);
      return;
    }
  }
#endif
  if( yyNewState > YY_MAX_SHIFT ){
    yyNewState += YY_MIN_REDUCE - YY_MIN_SHIFTREDUCE;
  }
  yytos = yypParser->yytos;
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor.yy0 = yyMinor;
  yyTraceShift(yypParser, yyNewState);
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 164, 1 },
  { 165, 0 },
  { 165, 2 },
  { 165, 1 },
  { 166, 1 },
  { 166, 1 },
  { 166, 1 },
  { 166, 1 },
  { 166, 1 },
  { 166, 1 },
  { 175, 0 },
  { 172, 2 },
  { 168, 2 },
  { 178, 3 },
  { 178, 4 },
  { 182, 0 },
  { 182, 2 },
  { 183, 0 },
  { 183, 2 },
  { 183, 2 },
  { 183, 2 },
  { 183, 2 },
  { 183, 3 },
  { 183, 5 },
  { 184, 1 },
  { 184, 3 },
  { 184, 3 },
  { 184, 3 },
  { 181, 3 },
  { 181, 3 },
  { 179, 0 },
  { 179, 2 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 180, 1 },
  { 185, 3 },
  { 186, 5 },
  { 187, 7 },
  { 194, 1 },
  { 194, 3 },
  { 169, 7 },
  { 169, 7 },
  { 196, 0 },
  { 196, 2 },
  { 196, 2 },
  { 196, 2 },
  { 196, 2 },
  { 196, 5 },
  { 191, 0 },
  { 191, 1 },
  { 192, 2 },
  { 193, 1 },
  { 193, 1 },
  { 193, 1 },
  { 193, 1 },
  { 171, 5 },
  { 170, 7 },
  { 200, 0 },
  { 200, 2 },
  { 197, 1 },
  { 197, 3 },
  { 198, 0 },
  { 199, 1 },
  { 199, 3 },
  { 167, 1 },
  { 202, 6 },
  { 203, 0 },
  { 203, 2 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 204, 1 },
  { 188, 6 },
  { 215, 0 },
  { 215, 3 },
  { 195, 1 },
  { 195, 3 },
  { 216, 0 },
  { 205, 0 },
  { 205, 1 },
  { 205, 2 },
  { 205, 2 },
  { 205, 2 },
  { 207, 2 },
  { 209, 1 },
  { 209, 1 },
  { 209, 1 },
  { 209, 1 },
  { 209, 3 },
  { 209, 5 },
  { 209, 5 },
  { 210, 0 },
  { 210, 2 },
  { 206, 5 },
  { 214, 0 },
  { 214, 2 },
  { 214, 2 },
  { 214, 2 },
  { 214, 2 },
  { 214, 2 },
  { 214, 7 },
  { 214, 5 },
  { 217, 1 },
  { 217, 3 },
  { 211, 4 },
  { 211, 3 },
  { 211, 4 },
  { 211, 3 },
  { 212, 0 },
  { 212, 2 },
  { 213, 0 },
  { 213, 3 },
  { 189, 3 },
  { 189, 4 },
  { 189, 4 },
  { 220, 1 },
  { 220, 2 },
  { 221, 1 },
  { 221, 2 },
  { 221, 1 },
  { 221, 1 },
  { 223, 2 },
  { 223, 2 },
  { 222, 3 },
  { 222, 2 },
  { 201, 1 },
  { 201, 1 },
  { 201, 1 },
  { 201, 1 },
  { 201, 1 },
  { 201, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 226, 1 },
  { 225, 1 },
  { 225, 1 },
  { 225, 2 },
  { 225, 4 },
  { 225, 5 },
  { 225, 2 },
  { 227, 6 },
  { 227, 6 },
  { 227, 4 },
  { 227, 2 },
  { 232, 0 },
  { 232, 3 },
  { 228, 1 },
  { 228, 1 },
  { 231, 2 },
  { 229, 3 },
  { 230, 1 },
  { 234, 3 },
  { 233, 2 },
  { 160, 3 },
  { 235, 6 },
  { 235, 2 },
  { 235, 2 },
  { 240, 1 },
  { 240, 2 },
  { 239, 3 },
  { 236, 0 },
  { 236, 2 },
  { 236, 3 },
  { 242, 0 },
  { 242, 2 },
  { 236, 6 },
  { 236, 5 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 241, 1 },
  { 238, 0 },
  { 238, 1 },
  { 162, 1 },
  { 237, 0 },
  { 237, 1 },
  { 237, 3 },
  { 243, 3 },
  { 244, 3 },
  { 244, 5 },
  { 245, 0 },
  { 245, 2 },
  { 245, 2 },
  { 245, 2 },
  { 246, 1 },
  { 246, 1 },
  { 246, 1 },
  { 246, 9 },
  { 246, 7 },
  { 246, 5 },
  { 246, 3 },
  { 246, 4 },
  { 246, 9 },
  { 246, 4 },
  { 246, 3 },
  { 246, 2 },
  { 246, 2 },
  { 247, 2 },
  { 247, 2 },
  { 247, 2 },
  { 247, 2 },
  { 247, 2 },
  { 247, 2 },
  { 247, 2 },
  { 247, 2 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 3 },
  { 176, 5 },
  { 224, 3 },
  { 219, 3 },
  { 249, 0 },
  { 250, 3 },
  { 250, 1 },
  { 177, 1 },
  { 177, 2 },
  { 248, 1 },
  { 248, 1 },
  { 248, 1 },
  { 248, 1 },
  { 248, 1 },
  { 248, 1 },
  { 248, 1 },
  { 252, 1 },
  { 252, 1 },
  { 252, 1 },
  { 252, 2 },
  { 252, 2 },
  { 252, 2 },
  { 252, 2 },
  { 252, 1 },
  { 190, 10 },
  { 190, 10 },
  { 258, 2 },
  { 258, 2 },
  { 258, 2 },
  { 258, 3 },
  { 251, 2 },
  { 251, 3 },
  { 251, 3 },
  { 218, 1 },
  { 218, 2 },
  { 254, 1 },
  { 256, 5 },
  { 256, 6 },
  { 256, 9 },
  { 257, 7 },
  { 261, 1 },
  { 261, 1 },
  { 262, 1 },
  { 262, 1 },
  { 263, 0 },
  { 263, 1 },
  { 263, 3 },
  { 255, 1 },
  { 255, 3 },
  { 264, 5 },
  { 255, 5 },
  { 253, 3 },
  { 253, 2 },
  { 259, 5 },
  { 260, 2 },
  { 266, 1 },
  { 266, 2 },
  { 266, 3 },
  { 266, 5 },
  { 266, 6 },
  { 266, 5 },
  { 265, 3 },
  { 165, 2 },
  { 173, 0 },
  { 173, 1 },
  { 174, 0 },
  { 174, 1 },
  { 175, 1 },
  { 191, 1 },
  { 192, 1 },
  { 197, 1 },
  { 198, 2 },
  { 208, 3 },
  { 226, 1 },
  { 231, 1 },
  { 229, 1 },
  { 230, 1 },
  { 233, 1 },
  { 239, 1 },
  { 162, 1 },
  { 237, 1 },
  { 243, 1 },
  { 246, 3 },
  { 247, 1 },
  { 176, 1 },
  { 224, 1 },
  { 219, 1 },
  { 249, 1 },
  { 248, 1 },
  { 163, 1 },
  { 252, 1 },
  { 252, 1 },
  { 252, 1 },
  { 252, 1 },
  { 265, 1 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  unsigned int yyruleno        /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  ZCCParseARG_FETCH;
  yymsp = yypParser->yytos;
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    yysize = yyRuleInfo[yyruleno].nrhs;
    fprintf(yyTraceFILE, "%sReduce [%s], go to state %d.\n", yyTracePrompt,
      yyRuleName[yyruleno], yymsp[-yysize].stateno);
    fflush(yyTraceFILE);
  }
#endif /* NDEBUG */

  /* Check that the stack is large enough to grow by a single entry
  ** if the RHS of the rule is empty.  This ensures that there is room
  ** enough on the stack to push the LHS value */
  if( yyRuleInfo[yyruleno].nrhs==0 ){
#ifdef YYTRACKMAXSTACKDEPTH
    if( (int)(yypParser->yytos - yypParser->yystack)>yypParser->yyhwm ){
      yypParser->yyhwm++;
      assert( yypParser->yyhwm == (int)(yypParser->yytos - yypParser->yystack));
    }
#endif
#if YYSTACKDEPTH>0 
    if( yypParser->yytos>=&yypParser->yystack[YYSTACKDEPTH-1] ){
      yyStackOverflow(yypParser);
      return;
    }
#else
    if( yypParser->yytos>=&yypParser->yystack[yypParser->yystksz-1] ){
      if( yyGrowStack(yypParser) ){
        yyStackOverflow(yypParser);
        return;
      }
      yymsp = yypParser->yytos;
    }
#endif
  }

  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
/********** Begin reduce actions **********************************************/
        YYMINORTYPE yylhsminor;
      case 0: /* main ::= translation_unit */
#line 173 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ stat->TopNode = yymsp[0].minor.yy168; DPrintf(DMSG_SPAMMY, "Parse complete\n"); }
#line 2638 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 1: /* translation_unit ::= */
      case 30: /*class_innards ::= */ yytestcase(yyruleno==30);
      case 55: /*opt_struct_body ::= */ yytestcase(yyruleno==55);
      case 73: /*mixin_class_body ::= */ yytestcase(yyruleno==73);
#line 176 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy168 = NULL; }
#line 2646 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 2: /* translation_unit ::= translation_unit external_declaration */
      case 31: /*class_innards ::= class_innards class_member */ yytestcase(yyruleno==31);
      case 74: /*mixin_class_body ::= mixin_class_body mixin_class_member */ yytestcase(yyruleno==74);
#line 177 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ SAFE_APPEND(yymsp[-1].minor.yy168,yymsp[0].minor.yy168); }
#line 2653 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 3: /* translation_unit ::= error */
      case 56: /*opt_struct_body ::= error */ yytestcase(yyruleno==56);
#line 179 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = NULL; }
#line 2659 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 4: /* external_declaration ::= mixin_definition */
#line 182 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy488; /*X-overwrites-A*/ }
#line 2664 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 5: /* external_declaration ::= class_definition */
#line 183 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy80; /*X-overwrites-A*/ }
#line 2669 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 6: /* external_declaration ::= struct_def */
      case 35: /*class_member ::= struct_def */ yytestcase(yyruleno==35);
      case 77: /*mixin_class_member ::= struct_def */ yytestcase(yyruleno==77);
#line 184 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy411; /*X-overwrites-A*/ }
#line 2676 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 7: /* external_declaration ::= enum_def */
      case 34: /*class_member ::= enum_def */ yytestcase(yyruleno==34);
      case 59: /*struct_member ::= enum_def */ yytestcase(yyruleno==59);
      case 76: /*mixin_class_member ::= enum_def */ yytestcase(yyruleno==76);
#line 185 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy5; /*X-overwrites-A*/ }
#line 2684 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 8: /* external_declaration ::= const_def */
      case 38: /*class_member ::= const_def */ yytestcase(yyruleno==38);
      case 60: /*struct_member ::= const_def */ yytestcase(yyruleno==60);
      case 80: /*mixin_class_member ::= const_def */ yytestcase(yyruleno==80);
#line 186 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy349; /*X-overwrites-A*/ }
#line 2692 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 9: /* external_declaration ::= include_def */
#line 187 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = nullptr; }
#line 2697 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 10: /* opt_expr ::= */
#line 198 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[1].minor.yy154 = NULL;
}
#line 2704 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 11: /* include_def ::= INCLUDE string_constant */
{  yy_destructor(yypParser,51,&yymsp[-1].minor);
#line 205 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	AddInclude(yymsp[0].minor.yy215);
}
#line 2712 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 12: /* class_definition ::= class_head class_body */
#line 219 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[-1].minor.yy80->Body = yymsp[0].minor.yy168;
	yymsp[-1].minor.yy80 = yymsp[-1].minor.yy80; /*X-overwrites-A*/
}
#line 2721 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 13: /* class_head ::= EXTEND CLASS IDENTIFIER */
{  yy_destructor(yypParser,52,&yymsp[-2].minor);
#line 225 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Class,head,yymsp[-1].minor.yy0);
	head->NodeName = yymsp[0].minor.yy0.Name();
	head->ParentName = nullptr;
	head->Flags =  ZCC_Extension;
	head->Replaces = nullptr;
	head->Version = {0, 0};
	head->Type = nullptr;
	head->Symbol = nullptr;
	yymsp[-2].minor.yy80 = head;
}
#line 2737 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 14: /* class_head ::= CLASS IDENTIFIER class_ancestry class_flags */
#line 239 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Class,head,yymsp[-3].minor.yy0);
	head->NodeName = yymsp[-2].minor.yy0.Name();
	head->ParentName = yymsp[-1].minor.yy41;
	head->Flags = yymsp[0].minor.yy435.Flags;
	head->Replaces = yymsp[0].minor.yy435.Replaces;
	head->Version = yymsp[0].minor.yy435.Version;
	head->Type = nullptr;
	head->Symbol = nullptr;
	yylhsminor.yy80 = head;
}
#line 2753 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-3].minor.yy80 = yylhsminor.yy80;
        break;
      case 15: /* class_ancestry ::= */
      case 164: /*class_restrictor ::= */ yytestcase(yyruleno==164);
#line 252 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy41 = NULL; }
#line 2760 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 16: /* class_ancestry ::= COLON dottable_id */
{  yy_destructor(yypParser,14,&yymsp[-1].minor);
#line 253 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy41 = yymsp[0].minor.yy41; /*yymsp[-1].minor.yy41-overwrites-yymsp[0].minor.yy41*/ }
#line 2766 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 17: /* class_flags ::= */
#line 256 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy435.Flags = 0; yymsp[1].minor.yy435.Replaces = NULL; yymsp[1].minor.yy435.Version = {0,0}; }
#line 2772 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 18: /* class_flags ::= class_flags ABSTRACT */
#line 257 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_Abstract; yylhsminor.yy435.Replaces = yymsp[-1].minor.yy435.Replaces; }
#line 2777 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,55,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 19: /* class_flags ::= class_flags NATIVE */
#line 258 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_Native; yylhsminor.yy435.Replaces = yymsp[-1].minor.yy435.Replaces; }
#line 2784 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,56,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 20: /* class_flags ::= class_flags UI */
#line 259 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_UIFlag; yylhsminor.yy435.Replaces = yymsp[-1].minor.yy435.Replaces; }
#line 2791 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,57,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 21: /* class_flags ::= class_flags PLAY */
#line 260 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_Play; yylhsminor.yy435.Replaces = yymsp[-1].minor.yy435.Replaces; }
#line 2798 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,58,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 22: /* class_flags ::= class_flags REPLACES dottable_id */
#line 261 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-2].minor.yy435.Flags; yylhsminor.yy435.Replaces = yymsp[0].minor.yy41; }
#line 2805 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,59,&yymsp[-1].minor);
  yymsp[-2].minor.yy435 = yylhsminor.yy435;
        break;
      case 23: /* class_flags ::= class_flags VERSION LPAREN STRCONST RPAREN */
#line 262 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-4].minor.yy435.Flags | ZCC_Version; yylhsminor.yy435.Replaces = yymsp[-4].minor.yy435.Replaces; yylhsminor.yy435.Version = yymsp[-1].minor.yy0.String->GetChars(); }
#line 2812 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,60,&yymsp[-3].minor);
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
  yymsp[-4].minor.yy435 = yylhsminor.yy435;
        break;
      case 24: /* dottable_id ::= IDENTIFIER */
      case 45: /*identifier_list ::= IDENTIFIER */ yytestcase(yyruleno==45);
      case 87: /*states_opt ::= IDENTIFIER */ yytestcase(yyruleno==87);
#line 270 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Identifier,id,yymsp[0].minor.yy0);
	id->Id = yymsp[0].minor.yy0.Name();
	yylhsminor.yy41 = id;
}
#line 2827 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy41 = yylhsminor.yy41;
        break;
      case 25: /* dottable_id ::= dottable_id DOT IDENTIFIER */
#line 276 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Identifier,id2,yymsp[-2].minor.yy41);
	id2->Id = yymsp[0].minor.yy0.Name();
	AppendTreeNodeSibling(yymsp[-2].minor.yy41, id2);
	yymsp[-2].minor.yy41 = yymsp[-2].minor.yy41; /*X-overwrites-A*/
}
#line 2838 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,44,&yymsp[-1].minor);
        break;
      case 26: /* dottable_id ::= dottable_id DOT DEFAULT */
#line 283 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Identifier,id2,yymsp[-2].minor.yy41);
	id2->Id = NAME_Default;
	AppendTreeNodeSibling(yymsp[-2].minor.yy41, id2);
	yymsp[-2].minor.yy41 = yymsp[-2].minor.yy41; /*X-overwrites-A*/
}
#line 2849 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,44,&yymsp[-1].minor);
  yy_destructor(yypParser,63,&yymsp[0].minor);
        break;
      case 27: /* dottable_id ::= dottable_id DOT COLOR */
#line 294 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Identifier,id2,yymsp[-2].minor.yy41);
	id2->Id = NAME_Color;
	AppendTreeNodeSibling(yymsp[-2].minor.yy41, id2);
	yymsp[-2].minor.yy41 = yymsp[-2].minor.yy41; /*X-overwrites-A*/
}
#line 2861 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,44,&yymsp[-1].minor);
  yy_destructor(yypParser,64,&yymsp[0].minor);
        break;
      case 28: /* class_body ::= SEMICOLON class_innards EOF */
{  yy_destructor(yypParser,49,&yymsp[-2].minor);
#line 311 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy168 = yymsp[-1].minor.yy168; /*yymsp[-2].minor.yy168-overwrites-yymsp[-1].minor.yy168*/ }
#line 2869 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,48,&yymsp[0].minor);
}
        break;
      case 29: /* class_body ::= LBRACE class_innards RBRACE */
{  yy_destructor(yypParser,65,&yymsp[-2].minor);
#line 312 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy168 = yymsp[-1].minor.yy168; /*yymsp[-2].minor.yy168-overwrites-yymsp[-1].minor.yy168*/ }
#line 2877 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 32: /* class_member ::= declarator */
      case 58: /*struct_member ::= declarator */ yytestcase(yyruleno==58);
      case 75: /*mixin_class_member ::= declarator */ yytestcase(yyruleno==75);
#line 325 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy365; /*X-overwrites-A*/ }
#line 2886 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 33: /* class_member ::= mixin_statement */
#line 326 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy387; /*X-overwrites-A*/ }
#line 2891 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 36: /* class_member ::= states_def */
      case 78: /*mixin_class_member ::= states_def */ yytestcase(yyruleno==78);
#line 329 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy48; /*X-overwrites-A*/ }
#line 2897 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 37: /* class_member ::= default_def */
      case 79: /*mixin_class_member ::= default_def */ yytestcase(yyruleno==79);
#line 330 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy65; /*X-overwrites-A*/ }
#line 2903 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 39: /* class_member ::= property_def */
      case 81: /*mixin_class_member ::= property_def */ yytestcase(yyruleno==81);
#line 332 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy493; /*X-overwrites-A*/ }
#line 2909 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 40: /* class_member ::= flag_def */
      case 82: /*mixin_class_member ::= flag_def */ yytestcase(yyruleno==82);
#line 333 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy357; /*X-overwrites-A*/ }
#line 2915 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 41: /* class_member ::= staticarray_statement */
      case 61: /*struct_member ::= staticarray_statement */ yytestcase(yyruleno==61);
      case 83: /*mixin_class_member ::= staticarray_statement */ yytestcase(yyruleno==83);
#line 334 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy168 = yymsp[0].minor.yy502; /*X-overwrites-A*/ }
#line 2922 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 42: /* mixin_statement ::= MIXIN IDENTIFIER SEMICOLON */
#line 339 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(MixinStmt,stmt,yymsp[-2].minor.yy0);
	stmt->MixinName = yymsp[-1].minor.yy0.Name();
	yylhsminor.yy387 = stmt;
}
#line 2931 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-2].minor.yy387 = yylhsminor.yy387;
        break;
      case 43: /* property_def ::= PROPERTY IDENTIFIER COLON identifier_list SEMICOLON */
#line 354 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Property,def,yymsp[-4].minor.yy0);
	def->NodeName = yymsp[-3].minor.yy0.Name();
	def->Body = yymsp[-1].minor.yy41;
	yylhsminor.yy493 = def;
}
#line 2943 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,14,&yymsp[-2].minor);
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-4].minor.yy493 = yylhsminor.yy493;
        break;
      case 44: /* flag_def ::= FLAGDEF IDENTIFIER COLON IDENTIFIER COMMA INTCONST SEMICOLON */
#line 362 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FlagDef,def,yymsp[-6].minor.yy0);
	def->NodeName = yymsp[-5].minor.yy0.Name();
	def->RefName = yymsp[-3].minor.yy0.Name();
	def->BitValue = yymsp[-1].minor.yy0.Int;
	yylhsminor.yy357 = def;
}
#line 2957 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,14,&yymsp[-4].minor);
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-6].minor.yy357 = yylhsminor.yy357;
        break;
      case 46: /* identifier_list ::= states_opt COMMA IDENTIFIER */
      case 88: /*states_opt ::= states_opt COMMA IDENTIFIER */ yytestcase(yyruleno==88);
#line 379 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Identifier,id,yymsp[0].minor.yy0);
	id->Id = yymsp[0].minor.yy0.Name();
	yymsp[-2].minor.yy41 = yymsp[-2].minor.yy41; /*X-overwrites-A*/
	AppendTreeNodeSibling(yymsp[-2].minor.yy41, id);
}
#line 2972 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 47: /* struct_def ::= STRUCT IDENTIFIER struct_flags LBRACE opt_struct_body RBRACE opt_semicolon */
#line 387 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Struct,def,yymsp[-6].minor.yy0);
	def->NodeName = yymsp[-5].minor.yy0.Name();
	def->Body = yymsp[-2].minor.yy168;
	def->Type = nullptr;
	def->Symbol = nullptr;
	def->Version = yymsp[-4].minor.yy435.Version;
	def->Flags = yymsp[-4].minor.yy435.Flags;
	yylhsminor.yy411 = def;
}
#line 2987 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,65,&yymsp[-3].minor);
  yy_destructor(yypParser,66,&yymsp[-1].minor);
  yymsp[-6].minor.yy411 = yylhsminor.yy411;
        break;
      case 48: /* struct_def ::= EXTEND STRUCT IDENTIFIER LBRACE opt_struct_body RBRACE opt_semicolon */
{  yy_destructor(yypParser,52,&yymsp[-6].minor);
#line 399 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Struct,def,yymsp[-5].minor.yy0);
	def->NodeName = yymsp[-4].minor.yy0.Name();
	def->Body = yymsp[-2].minor.yy168;
	def->Type = nullptr;
	def->Symbol = nullptr;
	def->Flags = ZCC_Extension;
	yymsp[-6].minor.yy411 = def;
}
#line 3004 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,65,&yymsp[-3].minor);
  yy_destructor(yypParser,66,&yymsp[-1].minor);
}
        break;
      case 49: /* struct_flags ::= */
#line 410 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy435.Flags = 0; yymsp[1].minor.yy435.Version = {0, 0}; }
#line 3012 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 50: /* struct_flags ::= struct_flags UI */
#line 411 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_UIFlag; }
#line 3017 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,57,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 51: /* struct_flags ::= struct_flags PLAY */
#line 412 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_Play; }
#line 3024 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,58,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 52: /* struct_flags ::= struct_flags CLEARSCOPE */
#line 413 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_ClearScope; }
#line 3031 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,72,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 53: /* struct_flags ::= struct_flags NATIVE */
#line 414 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-1].minor.yy435.Flags | ZCC_Native; }
#line 3038 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,56,&yymsp[0].minor);
  yymsp[-1].minor.yy435 = yylhsminor.yy435;
        break;
      case 54: /* struct_flags ::= struct_flags VERSION LPAREN STRCONST RPAREN */
#line 415 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy435.Flags = yymsp[-4].minor.yy435.Flags | ZCC_Version; yylhsminor.yy435.Version = yymsp[-1].minor.yy0.String->GetChars(); }
#line 3045 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,60,&yymsp[-3].minor);
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
  yymsp[-4].minor.yy435 = yylhsminor.yy435;
        break;
      case 57: /* struct_body ::= struct_body struct_member */
#line 423 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy168 = yymsp[-1].minor.yy168; /*X-overwrites-A*/ AppendTreeNodeSibling(yymsp[-1].minor.yy168, yymsp[0].minor.yy168); }
#line 3054 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 62: /* const_def ::= CONST IDENTIFIER EQ expr SEMICOLON */
#line 433 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ConstantDef,def,yymsp[-4].minor.yy0);
	def->NodeName = yymsp[-3].minor.yy0.Name();
	def->Value = yymsp[-1].minor.yy154;
	def->Symbol = NULL;
	yylhsminor.yy349 = def;
}
#line 3065 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-2].minor);
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-4].minor.yy349 = yylhsminor.yy349;
        break;
      case 63: /* enum_def ::= ENUM IDENTIFIER enum_type LBRACE opt_enum_list RBRACE opt_semicolon */
#line 450 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Enum,def,yymsp[-6].minor.yy0);
	def->NodeName = yymsp[-5].minor.yy0.Name();
	def->EnumType = (EZCCBuiltinType)yymsp[-4].minor.yy0.Int;
	def->Elements = yymsp[-2].minor.yy349;
	def->Symbol = nullptr;

	// If the first element does not have an explicit value, make it 0.
	if (yymsp[-2].minor.yy349 != NULL)
	{
		ZCC_ConstantDef *node = yymsp[-2].minor.yy349, *prev = node;

		if (node->Value == NULL)
		{
			NEW_INTCONST_NODE(zero, TypeSInt32, 0, yymsp[-2].minor.yy349);
			node->Value = zero;
		}
		for (node = static_cast<ZCC_ConstantDef *>(node->SiblingNext);
			node != yymsp[-2].minor.yy349;
			prev = node, node = static_cast<ZCC_ConstantDef *>(node->SiblingNext))
		{
			assert(node->NodeType == AST_ConstantDef);
			// Leave explicit values alone.
			if (node->Value != NULL)
			{
				continue;
			}
			// Compute implicit values by adding one to the preceding value.
			assert(prev->Value != NULL);
			// If the preceding node is a constant, then we can do this now.
			if (prev->Value->Operation == PEX_ConstValue && prev->Value->Type->isInt())
			{
				NEW_INTCONST_NODE(cval, prev->Value->Type, static_cast<ZCC_ExprConstant *>(prev->Value)->IntVal + 1, node);
				node->Value = cval;
			}
			// Otherwise, create a new addition expression to add 1.
			else
			{
				NEW_INTCONST_NODE(one, TypeSInt32, 1, yymsp[-6].minor.yy0);
				NEW_AST_NODE(ExprID, label, node);
				label->Operation = PEX_ID;
				label->Identifier = prev->NodeName;
				label->Type = NULL;

				BINARY_EXPR(label, one, PEX_Add);
				node->Value = expr2;
			}
		}
		// Add a new terminating node, to indicate that the ConstantDefs for this enum are done.
		NEW_AST_NODE(EnumTerminator,term,yymsp[-1].minor.yy0);
		AppendTreeNodeSibling(yymsp[-2].minor.yy349, term);
	}
	if (yymsp[-2].minor.yy349 != NULL)
	{
		AppendTreeNodeSibling(def, yymsp[-2].minor.yy349);
	}
	yylhsminor.yy5 = def;
}
#line 3130 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,65,&yymsp[-3].minor);
  yymsp[-6].minor.yy5 = yylhsminor.yy5;
        break;
      case 64: /* enum_type ::= */
#line 509 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy0.Int = ZCC_IntAuto; yymsp[1].minor.yy0.SourceLoc = stat->sc->GetMessageLine(); }
#line 3137 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 65: /* enum_type ::= COLON int_type */
{  yy_destructor(yypParser,14,&yymsp[-1].minor);
#line 510 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy0 = yymsp[0].minor.yy0; /*yymsp[-1].minor.yy0-overwrites-yymsp[0].minor.yy0*/ }
#line 3143 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 66: /* enum_list ::= error */
#line 512 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy349 = NULL; }
#line 3149 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 67: /* enum_list ::= enum_list COMMA enumerator */
#line 514 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy349 = yymsp[-2].minor.yy349; /*X-overwrites-A*/ AppendTreeNodeSibling(yymsp[-2].minor.yy349, yymsp[0].minor.yy349); }
#line 3154 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 68: /* opt_enum_list ::= */
#line 516 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy349 = NULL; }
#line 3160 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 69: /* enumerator ::= IDENTIFIER */
#line 520 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ConstantDef,node,yymsp[0].minor.yy0);
	node->NodeName = yymsp[0].minor.yy0.Name();
	node->Value = NULL;
	node->Symbol = NULL;
	yylhsminor.yy349 = node;
}
#line 3171 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy349 = yylhsminor.yy349;
        break;
      case 70: /* enumerator ::= IDENTIFIER EQ expr */
#line 528 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ConstantDef,node,yymsp[-2].minor.yy0);
	node->NodeName = yymsp[-2].minor.yy0.Name();
	node->Value = yymsp[0].minor.yy154;
	node->Symbol = NULL;
	yylhsminor.yy349 = node;
}
#line 3183 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-1].minor);
  yymsp[-2].minor.yy349 = yylhsminor.yy349;
        break;
      case 71: /* mixin_definition ::= mixin_class_definition */
#line 542 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy488 = yymsp[0].minor.yy488; /*X-overwrites-A*/ }
#line 3190 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 72: /* mixin_class_definition ::= MIXIN CLASS IDENTIFIER LBRACE mixin_class_body RBRACE */
#line 549 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(MixinDef,def,yymsp[-5].minor.yy0);
	def->Body = yymsp[-1].minor.yy168;
	def->NodeName = yymsp[-3].minor.yy0.Name();
	def->MixinType = ZCC_Mixin_Class;
	def->Symbol = nullptr;
	yylhsminor.yy488 = def;
}
#line 3202 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,53,&yymsp[-4].minor);
  yy_destructor(yypParser,65,&yymsp[-2].minor);
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-5].minor.yy488 = yylhsminor.yy488;
        break;
      case 84: /* states_def ::= STATES states_opts scanner_mode LBRACE states_body RBRACE */
#line 598 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(States,def,yymsp[-5].minor.yy0);
	def->Flags = yymsp[-4].minor.yy41;
	def->Body = yymsp[-1].minor.yy376;
	yylhsminor.yy48 = def;
}
#line 3216 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,65,&yymsp[-2].minor);
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-5].minor.yy48 = yylhsminor.yy48;
        break;
      case 85: /* states_opts ::= */
#line 605 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy41 = nullptr; }
#line 3224 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 86: /* states_opts ::= LPAREN states_opt RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-2].minor);
#line 606 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy41 = yymsp[-1].minor.yy41; /*yymsp[-2].minor.yy41-overwrites-yymsp[-1].minor.yy41*/ }
#line 3230 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 89: /* scanner_mode ::= */
#line 635 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ stat->sc->SetStateMode(true); }
#line 3237 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 90: /* states_body ::= */
#line 637 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy376 = NULL; }
#line 3242 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 91: /* states_body ::= error */
#line 638 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy376 = NULL; }
#line 3247 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 92: /* states_body ::= states_body state_line */
      case 93: /*states_body ::= states_body state_label */ yytestcase(yyruleno==93);
      case 94: /*states_body ::= states_body state_flow */ yytestcase(yyruleno==94);
#line 639 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ SAFE_APPEND(yymsp[-1].minor.yy376,yymsp[0].minor.yy376); }
#line 3254 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 95: /* state_label ::= NWS COLON */
#line 644 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StateLabel, label, yymsp[-1].minor.yy0);
	label->Label = yymsp[-1].minor.yy0.Name();
	yylhsminor.yy376 = label;
}
#line 3263 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,14,&yymsp[0].minor);
  yymsp[-1].minor.yy376 = yylhsminor.yy376;
        break;
      case 96: /* state_flow_type ::= STOP */
#line 652 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ NEW_AST_NODE(StateStop, flow, yymsp[0].minor.yy0); yylhsminor.yy376 = flow; }
#line 3270 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy376 = yylhsminor.yy376;
        break;
      case 97: /* state_flow_type ::= WAIT */
#line 653 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ NEW_AST_NODE(StateWait, flow, yymsp[0].minor.yy0); yylhsminor.yy376 = flow; }
#line 3276 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy376 = yylhsminor.yy376;
        break;
      case 98: /* state_flow_type ::= FAIL */
#line 654 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ NEW_AST_NODE(StateFail, flow, yymsp[0].minor.yy0); yylhsminor.yy376 = flow; }
#line 3282 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy376 = yylhsminor.yy376;
        break;
      case 99: /* state_flow_type ::= LOOP */
#line 655 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ NEW_AST_NODE(StateLoop, flow, yymsp[0].minor.yy0); yylhsminor.yy376 = flow; }
#line 3288 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy376 = yylhsminor.yy376;
        break;
      case 100: /* state_flow_type ::= GOTO dottable_id state_goto_offset */
#line 657 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StateGoto, flow, yymsp[-2].minor.yy0);
	flow->Label = yymsp[-1].minor.yy41;
	flow->Offset = yymsp[0].minor.yy154;
	flow->Qualifier = nullptr;
	yylhsminor.yy376 = flow;
}
#line 3300 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-2].minor.yy376 = yylhsminor.yy376;
        break;
      case 101: /* state_flow_type ::= GOTO IDENTIFIER SCOPE dottable_id state_goto_offset */
#line 666 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StateGoto, flow, yymsp[-4].minor.yy0);
	flow->Label = yymsp[-1].minor.yy41;
	flow->Offset = yymsp[0].minor.yy154;

	NEW_AST_NODE(Identifier,id,yymsp[-3].minor.yy0);
	id->Id = yymsp[-3].minor.yy0.Name();
	flow->Qualifier =id;
	yylhsminor.yy376 = flow;
}
#line 3315 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,47,&yymsp[-2].minor);
  yymsp[-4].minor.yy376 = yylhsminor.yy376;
        break;
      case 102: /* state_flow_type ::= GOTO SUPER SCOPE dottable_id state_goto_offset */
#line 678 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StateGoto, flow, yymsp[-4].minor.yy0);
	flow->Label = yymsp[-1].minor.yy41;
	flow->Offset = yymsp[0].minor.yy154;

	NEW_AST_NODE(Identifier,id,yymsp[-3].minor.yy0);
	id->Id = NAME_Super;
	flow->Qualifier =id;
	yylhsminor.yy376 = flow;
}
#line 3331 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,47,&yymsp[-2].minor);
  yymsp[-4].minor.yy376 = yylhsminor.yy376;
        break;
      case 103: /* state_goto_offset ::= */
#line 689 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy154 = NULL; }
#line 3338 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 104: /* state_goto_offset ::= ADD expr */
{  yy_destructor(yypParser,34,&yymsp[-1].minor);
#line 690 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy154 = yymsp[0].minor.yy154; /*yymsp[-1].minor.yy154-overwrites-yymsp[0].minor.yy154*/ }
#line 3344 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 105: /* state_line ::= NWS NWS expr state_opts state_action */
#line 693 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StateLine, line, yymsp[-4].minor.yy0);
	line->Sprite = stat->Strings.Alloc(FName(yymsp[-4].minor.yy0.Name()).GetChars());
	line->Frames = stat->Strings.Alloc(FName(yymsp[-3].minor.yy0.Name()).GetChars());
	line->Duration = yymsp[-2].minor.yy154;
	line->bBright = yymsp[-1].minor.yy169.Bright;
	line->bFast = yymsp[-1].minor.yy169.Fast;
	line->bSlow = yymsp[-1].minor.yy169.Slow;
	line->bNoDelay = yymsp[-1].minor.yy169.NoDelay;
	line->bCanRaise = yymsp[-1].minor.yy169.CanRaise;
	line->Offset = yymsp[-1].minor.yy169.Offset;
	line->Lights = yymsp[-1].minor.yy169.Lights;
	line->Action = yymsp[0].minor.yy168;
	yylhsminor.yy376 = line;
}
#line 3364 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-4].minor.yy376 = yylhsminor.yy376;
        break;
      case 106: /* state_opts ::= */
#line 709 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ StateOpts opts; opts.Zero(); yymsp[1].minor.yy169 = opts; }
#line 3370 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 107: /* state_opts ::= state_opts BRIGHT */
#line 710 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy169.Bright = true; yymsp[-1].minor.yy169 = yymsp[-1].minor.yy169; /*X-overwrites-A*/ }
#line 3375 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,83,&yymsp[0].minor);
        break;
      case 108: /* state_opts ::= state_opts FAST */
#line 711 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy169.Fast = true; yymsp[-1].minor.yy169 = yymsp[-1].minor.yy169; /*X-overwrites-A*/ }
#line 3381 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,84,&yymsp[0].minor);
        break;
      case 109: /* state_opts ::= state_opts SLOW */
#line 712 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy169.Slow = true; yymsp[-1].minor.yy169 = yymsp[-1].minor.yy169; /*X-overwrites-A*/ }
#line 3387 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,85,&yymsp[0].minor);
        break;
      case 110: /* state_opts ::= state_opts NODELAY */
#line 713 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy169.NoDelay = true; yymsp[-1].minor.yy169 = yymsp[-1].minor.yy169; /*X-overwrites-A*/ }
#line 3393 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,86,&yymsp[0].minor);
        break;
      case 111: /* state_opts ::= state_opts CANRAISE */
#line 714 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy169.CanRaise = true; yymsp[-1].minor.yy169 = yymsp[-1].minor.yy169; /*X-overwrites-A*/ }
#line 3399 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,87,&yymsp[0].minor);
        break;
      case 112: /* state_opts ::= state_opts OFFSET LPAREN expr COMMA expr RPAREN */
#line 715 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-6].minor.yy169.Offset = yymsp[-3].minor.yy154; AppendTreeNodeSibling(yymsp[-3].minor.yy154, yymsp[-1].minor.yy154); yymsp[-6].minor.yy169 = yymsp[-6].minor.yy169; /*X-overwrites-A*/ }
#line 3405 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,88,&yymsp[-5].minor);
  yy_destructor(yypParser,45,&yymsp[-4].minor);
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
        break;
      case 113: /* state_opts ::= state_opts LIGHT LPAREN light_list RPAREN */
#line 716 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-4].minor.yy169 = yymsp[-4].minor.yy169; /*X-overwrites-A*/ yymsp[-4].minor.yy169.Lights = yymsp[-1].minor.yy215; }
#line 3414 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,89,&yymsp[-3].minor);
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
        break;
      case 114: /* light_list ::= STRCONST */
      case 284: /*string_constant ::= STRCONST */ yytestcase(yyruleno==284);
#line 721 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprConstant, strconst, yymsp[0].minor.yy0);
	strconst->Operation = PEX_ConstValue;
	strconst->Type = TypeString;
	strconst->StringVal = yymsp[0].minor.yy0.String;
	yylhsminor.yy215 = strconst;
}
#line 3429 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 115: /* light_list ::= light_list COMMA STRCONST */
#line 730 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprConstant, strconst, yymsp[0].minor.yy0);
	strconst->Operation = PEX_ConstValue;
	strconst->Type = TypeString;
	strconst->StringVal = yymsp[0].minor.yy0.String;
	AppendTreeNodeSibling(yymsp[-2].minor.yy215, strconst); 
	yymsp[-2].minor.yy215 = yymsp[-2].minor.yy215; /*X-overwrites-A*/
}
#line 3442 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 116: /* state_action ::= LBRACE statement_list scanner_mode RBRACE */
#line 741 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(CompoundStmt,stmt,yymsp[-3].minor.yy0);
	stmt->Content = yymsp[-2].minor.yy367;
	yylhsminor.yy168 = stmt;
}
#line 3452 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-3].minor.yy168 = yylhsminor.yy168;
        break;
      case 117: /* state_action ::= LBRACE scanner_mode RBRACE */
{  yy_destructor(yypParser,65,&yymsp[-2].minor);
#line 747 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[-2].minor.yy168 = NULL;
}
#line 3462 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 118: /* state_action ::= LBRACE error scanner_mode RBRACE */
{  yy_destructor(yypParser,65,&yymsp[-3].minor);
#line 750 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-3].minor.yy168 = NULL; }
#line 3470 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 119: /* state_action ::= state_call scanner_mode SEMICOLON */
#line 751 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy168 = yymsp[-2].minor.yy13; /*X-overwrites-A*/ }
#line 3477 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
        break;
      case 120: /* state_call ::= */
#line 753 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy13 = NULL; }
#line 3483 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 121: /* state_call ::= IDENTIFIER state_call_params */
#line 755 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprFuncCall, expr, yymsp[-1].minor.yy0);
	NEW_AST_NODE(ExprID, func, yymsp[-1].minor.yy0);

	func->Operation = PEX_ID;
	func->Identifier = yymsp[-1].minor.yy0.Name();
	expr->Operation = PEX_FuncCall;
	expr->Function = func;
	expr->Parameters = yymsp[0].minor.yy74;
	yylhsminor.yy13 = expr;
}
#line 3498 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy13 = yylhsminor.yy13;
        break;
      case 122: /* state_call_params ::= */
#line 767 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy74 = NULL; }
#line 3504 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 123: /* state_call_params ::= LPAREN func_expr_list RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-2].minor);
#line 768 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy74 = yymsp[-1].minor.yy74; /*yymsp[-2].minor.yy74-overwrites-yymsp[-1].minor.yy74*/ }
#line 3510 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 124: /* default_def ::= DEFAULT LBRACE RBRACE */
{  yy_destructor(yypParser,63,&yymsp[-2].minor);
#line 776 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Default,stmt,yymsp[-1].minor.yy0);
	stmt->Content = NULL;
	yymsp[-2].minor.yy65 = stmt;
}
#line 3522 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 125: /* default_def ::= DEFAULT LBRACE default_statement_list RBRACE */
{  yy_destructor(yypParser,63,&yymsp[-3].minor);
#line 782 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Default,stmt,yymsp[-2].minor.yy0);
	stmt->Content = yymsp[-1].minor.yy367;
	yymsp[-3].minor.yy65 = stmt;
}
#line 3534 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 126: /* default_def ::= DEFAULT LBRACE error RBRACE */
{  yy_destructor(yypParser,63,&yymsp[-3].minor);
#line 788 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Default,stmt,yymsp[-2].minor.yy0);
	stmt->Content = NULL;
	yymsp[-3].minor.yy65 = stmt;
}
#line 3546 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 127: /* default_statement_list ::= default_statement */
      case 310: /*statement_list ::= statement */ yytestcase(yyruleno==310);
#line 795 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[0].minor.yy367 = yymsp[0].minor.yy367; /*X-overwrites-A*/
}
#line 3556 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 128: /* default_statement_list ::= default_statement_list default_statement */
      case 311: /*statement_list ::= statement_list statement */ yytestcase(yyruleno==311);
#line 799 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	SAFE_APPEND(yymsp[-1].minor.yy367,yymsp[0].minor.yy367);
}
#line 3564 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 129: /* default_statement ::= SEMICOLON */
      case 293: /*statement ::= SEMICOLON */ yytestcase(yyruleno==293);
{  yy_destructor(yypParser,49,&yymsp[0].minor);
#line 804 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = NULL; }
#line 3571 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 130: /* default_statement ::= error SEMICOLON */
      case 299: /*statement ::= error SEMICOLON */ yytestcase(yyruleno==299);
#line 805 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy367 = NULL; }
#line 3578 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
        break;
      case 131: /* default_statement ::= property_statement */
#line 807 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy39; /*X-overwrites-A*/ }
#line 3584 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 132: /* default_statement ::= flag_statement */
#line 808 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy50; /*X-overwrites-A*/ }
#line 3589 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 133: /* flag_statement ::= ADD dottable_id */
{  yy_destructor(yypParser,34,&yymsp[-1].minor);
#line 813 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FlagStmt, type, yymsp[0].minor.yy41);
	type->set = true;
	type->name = yymsp[0].minor.yy41;
	yymsp[-1].minor.yy50 = type;
}
#line 3600 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 134: /* flag_statement ::= SUB dottable_id */
{  yy_destructor(yypParser,33,&yymsp[-1].minor);
#line 820 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FlagStmt, type, yymsp[0].minor.yy41);
	type->set = false;
	type->name = yymsp[0].minor.yy41;
	yymsp[-1].minor.yy50 = type;
}
#line 3612 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 135: /* property_statement ::= dottable_id expr_list SEMICOLON */
#line 830 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(PropertyStmt,stmt,yymsp[-2].minor.yy41);
	stmt->Prop = yymsp[-2].minor.yy41;
	stmt->Values = yymsp[-1].minor.yy154;
	yylhsminor.yy39 = stmt;
}
#line 3623 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-2].minor.yy39 = yylhsminor.yy39;
        break;
      case 136: /* property_statement ::= dottable_id SEMICOLON */
#line 838 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(PropertyStmt,stmt,yymsp[-1].minor.yy41);
	stmt->Prop = yymsp[-1].minor.yy41;
	stmt->Values = nullptr;
	yylhsminor.yy39 = stmt;
}
#line 3635 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-1].minor.yy39 = yylhsminor.yy39;
        break;
      case 137: /* int_type ::= SBYTE */
#line 850 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_SInt8; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3642 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 138: /* int_type ::= BYTE */
#line 851 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_UInt8; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3648 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 139: /* int_type ::= SHORT */
#line 852 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_SInt16; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3654 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 140: /* int_type ::= USHORT */
#line 853 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_UInt16; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3660 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 141: /* int_type ::= INT */
#line 854 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_SInt32; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3666 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 142: /* int_type ::= UINT */
#line 855 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_UInt32; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3672 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 143: /* type_name1 ::= BOOL */
#line 857 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Bool; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3678 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 144: /* type_name1 ::= FLOAT */
#line 859 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_FloatAuto; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3684 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 145: /* type_name1 ::= DOUBLE */
#line 860 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Float64; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3690 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 146: /* type_name1 ::= VECTOR2 */
#line 862 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Vector2; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3696 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 147: /* type_name1 ::= VECTOR3 */
#line 863 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Vector3; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3702 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 148: /* type_name1 ::= VECTOR4 */
#line 864 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Vector4; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3708 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 149: /* type_name1 ::= NAME */
#line 865 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Name; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3714 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 150: /* type_name1 ::= SOUND */
#line 866 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Sound; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3720 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 151: /* type_name1 ::= STATE */
#line 867 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_State; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3726 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 152: /* type_name1 ::= COLOR */
#line 868 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Color; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3732 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 153: /* type_name1 ::= LET */
#line 869 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Let; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 3738 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 154: /* type_name ::= type_name1 */
#line 872 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BasicType, type, yymsp[0].minor.yy0);
	type->Type = (EZCCBuiltinType)yymsp[0].minor.yy0.Int;
	type->UserType = NULL;
	type->isconst = false;
	yylhsminor.yy86 = type;
}
#line 3750 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy86 = yylhsminor.yy86;
        break;
      case 155: /* type_name ::= IDENTIFIER */
#line 880 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BasicType, type, yymsp[0].minor.yy0);
	NEW_AST_NODE(Identifier, id, yymsp[0].minor.yy0);
	type->Type = ZCC_UserType;
	type->UserType = id;
	type->isconst = false;
	id->Id = yymsp[0].minor.yy0.Name();
	yylhsminor.yy86 = type;
}
#line 3764 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy86 = yylhsminor.yy86;
        break;
      case 156: /* type_name ::= ATSIGN IDENTIFIER */
{  yy_destructor(yypParser,106,&yymsp[-1].minor);
#line 891 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BasicType, type, yymsp[0].minor.yy0);
	NEW_AST_NODE(Identifier, id, yymsp[0].minor.yy0);
	type->Type = ZCC_NativeType;
	type->UserType = id;
	type->isconst = false;
	id->Id = yymsp[0].minor.yy0.Name();
	yymsp[-1].minor.yy86 = type;
}
#line 3779 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 157: /* type_name ::= READONLY LT IDENTIFIER GT */
{  yy_destructor(yypParser,107,&yymsp[-3].minor);
#line 902 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BasicType, type, yymsp[-1].minor.yy0);
	NEW_AST_NODE(Identifier, id, yymsp[-1].minor.yy0);
	type->Type = ZCC_UserType;
	type->UserType = id;
	type->isconst = true;
	id->Id = yymsp[-1].minor.yy0.Name();
	yymsp[-3].minor.yy86 = type;
}
#line 3794 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,20,&yymsp[-2].minor);
  yy_destructor(yypParser,21,&yymsp[0].minor);
}
        break;
      case 158: /* type_name ::= READONLY LT ATSIGN IDENTIFIER GT */
{  yy_destructor(yypParser,107,&yymsp[-4].minor);
#line 913 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BasicType, type, yymsp[-1].minor.yy0);
	NEW_AST_NODE(Identifier, id, yymsp[-1].minor.yy0);
	type->Type = ZCC_NativeType;
	type->UserType = id;
	type->isconst = true;
	id->Id = yymsp[-1].minor.yy0.Name();
	yymsp[-4].minor.yy86 = type;
}
#line 3811 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,20,&yymsp[-3].minor);
  yy_destructor(yypParser,106,&yymsp[-2].minor);
  yy_destructor(yypParser,21,&yymsp[0].minor);
}
        break;
      case 159: /* type_name ::= DOT dottable_id */
{  yy_destructor(yypParser,44,&yymsp[-1].minor);
#line 924 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BasicType, type, yymsp[0].minor.yy41);
	type->Type = ZCC_UserType;
	type->UserType = yymsp[0].minor.yy41;
	type->isconst = false;
	yymsp[-1].minor.yy86 = type;
}
#line 3827 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 160: /* aggregate_type ::= MAP LT type_or_array COMMA type_or_array GT */
#line 948 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(MapType,map,yymsp[-5].minor.yy0);
	map->KeyType = yymsp[-3].minor.yy256;
	map->ValueType = yymsp[-1].minor.yy256;
	yylhsminor.yy256 = map;
}
#line 3838 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,20,&yymsp[-4].minor);
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,21,&yymsp[0].minor);
  yymsp[-5].minor.yy256 = yylhsminor.yy256;
        break;
      case 161: /* aggregate_type ::= MAPITERATOR LT type_or_array COMMA type_or_array GT */
#line 956 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(MapIteratorType,map_it,yymsp[-5].minor.yy0);
	map_it->KeyType = yymsp[-3].minor.yy256;
	map_it->ValueType = yymsp[-1].minor.yy256;
	yylhsminor.yy256 = map_it;
}
#line 3852 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,20,&yymsp[-4].minor);
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,21,&yymsp[0].minor);
  yymsp[-5].minor.yy256 = yylhsminor.yy256;
        break;
      case 162: /* aggregate_type ::= ARRAY LT type_or_array GT */
#line 964 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(DynArrayType,arr,yymsp[-3].minor.yy0);
	arr->ElementType = yymsp[-1].minor.yy256;
	yylhsminor.yy256 = arr;
}
#line 3865 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,20,&yymsp[-2].minor);
  yy_destructor(yypParser,21,&yymsp[0].minor);
  yymsp[-3].minor.yy256 = yylhsminor.yy256;
        break;
      case 163: /* aggregate_type ::= CLASS class_restrictor */
#line 971 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ClassType,cls,yymsp[-1].minor.yy0);
	cls->Restriction = yymsp[0].minor.yy41;
	yylhsminor.yy256 = cls;
}
#line 3877 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy256 = yylhsminor.yy256;
        break;
      case 165: /* class_restrictor ::= LT dottable_id GT */
{  yy_destructor(yypParser,20,&yymsp[-2].minor);
#line 977 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy41 = yymsp[-1].minor.yy41; /*yymsp[-2].minor.yy41-overwrites-yymsp[-1].minor.yy41*/ }
#line 3884 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,21,&yymsp[0].minor);
}
        break;
      case 166: /* type ::= type_name */
#line 979 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy256 = yymsp[0].minor.yy86; /*X-overwrites-A*/ yymsp[0].minor.yy256->ArraySize = NULL; }
#line 3891 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 167: /* type ::= aggregate_type */
#line 980 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy256 = yymsp[0].minor.yy256; /*X-overwrites-A*/ yymsp[0].minor.yy256->ArraySize = NULL; }
#line 3896 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 168: /* type_or_array ::= type array_size */
#line 983 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy256 = yymsp[-1].minor.yy256; /*X-overwrites-A*/ yymsp[-1].minor.yy256->ArraySize = yymsp[0].minor.yy154; }
#line 3901 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 169: /* type_list ::= type_list COMMA type_or_array */
#line 986 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy256 = yymsp[-2].minor.yy256; /*X-overwrites-A*/ AppendTreeNodeSibling(yymsp[-2].minor.yy256, yymsp[0].minor.yy256); }
#line 3906 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 170: /* type_list_or_void ::= VOID */
{  yy_destructor(yypParser,112,&yymsp[0].minor);
#line 988 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy256 = NULL; }
#line 3913 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 171: /* array_size_expr ::= LBRACKET opt_expr RBRACKET */
#line 992 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	if (yymsp[-1].minor.yy154 == NULL)
	{
		NEW_AST_NODE(Expression,nil,yymsp[-2].minor.yy0.SourceLoc);
		nil->Operation = PEX_Nil;
		nil->Type = NULL;
		yylhsminor.yy154 = nil;
	}
	else
	{
		yylhsminor.yy154 = yymsp[-1].minor.yy154;
	}
}
#line 3931 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,117,&yymsp[0].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 172: /* array_size ::= array_size array_size_expr */
#line 1007 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	AppendTreeNodeSibling(yymsp[-1].minor.yy154, yymsp[0].minor.yy154);
	yymsp[-1].minor.yy154 = yymsp[-1].minor.yy154; /*X-overwrites-A*/
}
#line 3941 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 173: /* declarator ::= decl_flags type_list_or_void variables_or_function */
#line 1016 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	if (yymsp[0].minor.yy170.FuncName == NAME_None && yymsp[0].minor.yy170.VarNames == NULL)
	{ // An error. yymsp[-2].minor.yy9 message was already printed.
		yylhsminor.yy365 = NULL;
	}
	else if (yymsp[0].minor.yy170.FuncName != NAME_None)
	{ // yymsp[-2].minor.yy9 function
		NEW_AST_NODE(FuncDeclarator, decl, yymsp[-2].minor.yy9 == nullptr? yymsp[0].minor.yy170.SourceLoc : yymsp[-2].minor.yy9->SourceLoc);
		decl->Type = yymsp[-1].minor.yy256;
		decl->Params = yymsp[0].minor.yy170.FuncParams;
		decl->Name = yymsp[0].minor.yy170.FuncName;
		decl->UseFlags = yymsp[-2].minor.yy9 == nullptr? nullptr : yymsp[-2].minor.yy9->Id;
		decl->Flags = (yymsp[-2].minor.yy9 == nullptr? 0 : yymsp[-2].minor.yy9->Flags) | yymsp[0].minor.yy170.FuncFlags;
		decl->DeprecationMessage = yymsp[-2].minor.yy9 == nullptr ? nullptr : yymsp[-2].minor.yy9->DeprecationMessage;
		if (yymsp[-2].minor.yy9 == nullptr) decl->Version = {0,0,0};
		else decl->Version = yymsp[-2].minor.yy9->Version;

		decl->Body = yymsp[0].minor.yy170.FuncBody;
		yylhsminor.yy365 = decl;
	}
	else if (yymsp[-1].minor.yy256 != NULL && yymsp[-1].minor.yy256->SiblingNext == yymsp[-1].minor.yy256)
	{ // yymsp[-2].minor.yy9 variable
		NEW_AST_NODE(VarDeclarator, decl, yymsp[-2].minor.yy9 == nullptr? yymsp[-1].minor.yy256->SourceLoc : yymsp[-2].minor.yy9->SourceLoc);
		decl->Type = yymsp[-1].minor.yy256;
		decl->Names = yymsp[0].minor.yy170.VarNames;
		if (yymsp[-2].minor.yy9 == nullptr)
		{
			decl->Flags = 0;
			decl->Version = {0,0,0};
			decl->DeprecationMessage = nullptr;
		}
		else
		{
			decl->Flags = yymsp[-2].minor.yy9->Flags;
			decl->Version = yymsp[-2].minor.yy9->Version;
			decl->DeprecationMessage = yymsp[-2].minor.yy9->DeprecationMessage;
		}
		yylhsminor.yy365 = decl;
	}
	else
	{ // An invalid
		if (yymsp[-1].minor.yy256 == NULL)
		{
			stat->sc->ScriptMessage("Variables may not be of type void.\n");
		}
		else
		{
			stat->sc->ScriptMessage("Variables may be of only one type.\n");
		}
		yylhsminor.yy365 = NULL;
	}
}
#line 3997 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-2].minor.yy365 = yylhsminor.yy365;
        break;
      case 174: /* variables_or_function ::= IDENTIFIER LPAREN func_params RPAREN func_const opt_func_body */
#line 1071 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	VarOrFun fun;

	fun.VarNames = NULL;
	fun.FuncParams = yymsp[-3].minor.yy37;
	fun.FuncFlags = yymsp[-1].minor.yy0.Int;
	fun.FuncName = yymsp[-5].minor.yy0.Name();
	fun.FuncBody = yymsp[0].minor.yy51;
	fun.SourceLoc = yymsp[-5].minor.yy0.SourceLoc;
	yylhsminor.yy170 = fun;
}
#line 4013 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-4].minor);
  yy_destructor(yypParser,62,&yymsp[-2].minor);
  yymsp[-5].minor.yy170 = yylhsminor.yy170;
        break;
      case 175: /* variables_or_function ::= variable_list SEMICOLON */
#line 1083 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	VarOrFun var;

	var.VarNames = yymsp[-1].minor.yy134;
	var.FuncParams = NULL;
	var.FuncFlags = 0;
	var.FuncName = NAME_None;
	var.FuncBody = NULL;
	var.SourceLoc = yymsp[-1].minor.yy134->SourceLoc;
	yylhsminor.yy170 = var;
}
#line 4031 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-1].minor.yy170 = yylhsminor.yy170;
        break;
      case 176: /* variables_or_function ::= error SEMICOLON */
#line 1095 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	VarOrFun bad;
	bad.VarNames = NULL;
	bad.FuncParams = NULL;
	bad.FuncFlags = 0;
	bad.FuncName = NAME_None;
	bad.FuncBody = NULL;
	bad.SourceLoc = yymsp[0].minor.yy0.SourceLoc;
	yymsp[-1].minor.yy170 = bad;
}
#line 4047 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 177: /* variable_name ::= IDENTIFIER */
#line 1112 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(VarName,var,yymsp[0].minor.yy0);
	var->Name = ENamedName(yymsp[0].minor.yy0.Int);
	var->ArraySize = NULL;
	yylhsminor.yy134 = var;
}
#line 4057 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy134 = yylhsminor.yy134;
        break;
      case 178: /* variable_name ::= IDENTIFIER array_size */
#line 1119 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(VarName,var,yymsp[-1].minor.yy0);
	var->Name = ENamedName(yymsp[-1].minor.yy0.Int);
	var->ArraySize = yymsp[0].minor.yy154;
	yylhsminor.yy134 = var;
}
#line 4068 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy134 = yylhsminor.yy134;
        break;
      case 179: /* variable_list ::= variable_list COMMA variable_name */
#line 1128 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	AppendTreeNodeSibling(yymsp[-2].minor.yy134, yymsp[0].minor.yy134);
	yymsp[-2].minor.yy134 = yymsp[-2].minor.yy134; /*X-overwrites-A*/
}
#line 4077 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 180: /* decl_flags ::= */
#line 1134 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy9 = NULL; }
#line 4083 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 181: /* decl_flags ::= decl_flags decl_flag */
#line 1136 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	if (yymsp[-1].minor.yy9 == nullptr)
	{
		NEW_AST_NODE(DeclFlags,nil_f,yymsp[0].minor.yy0);
		yylhsminor.yy9 = nil_f;
		yylhsminor.yy9->Id = nullptr;
		yylhsminor.yy9->Flags = yymsp[0].minor.yy0.Int;
		yylhsminor.yy9->Version = { 0, 0 };
		yylhsminor.yy9->DeprecationMessage = nullptr;
	}
	else
	{
		yylhsminor.yy9 = yymsp[-1].minor.yy9;
		yylhsminor.yy9->Flags |= yymsp[0].minor.yy0.Int;
	}
}
#line 4103 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy9 = yylhsminor.yy9;
        break;
      case 182: /* decl_flags ::= decl_flags ACTION states_opts */
#line 1155 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	if (yymsp[-2].minor.yy9 == nullptr)
	{
		NEW_AST_NODE(DeclFlags,nil_f,yymsp[-1].minor.yy0.SourceLoc);
		yylhsminor.yy9 = nil_f;
		yylhsminor.yy9->Flags = ZCC_Action;
		yylhsminor.yy9->Id = nullptr;
		yylhsminor.yy9->Version = { 0, 0 };
		yylhsminor.yy9->DeprecationMessage = nullptr;
	}
	else
	{
		yylhsminor.yy9 = yymsp[-2].minor.yy9;
		yylhsminor.yy9->Flags |= ZCC_Action;
	}
	yylhsminor.yy9->Id = yymsp[0].minor.yy41;
}
#line 4125 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-2].minor.yy9 = yylhsminor.yy9;
        break;
      case 183: /* opt_deprecation_message ::= */
#line 1173 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy0.String = nullptr; yymsp[1].minor.yy0.SourceLoc = stat->sc->GetMessageLine(); }
#line 4131 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 184: /* opt_deprecation_message ::= COMMA STRCONST */
{  yy_destructor(yypParser,50,&yymsp[-1].minor);
#line 1174 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy0 = yymsp[0].minor.yy0; }
#line 4137 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 185: /* decl_flags ::= decl_flags DEPRECATED LPAREN STRCONST opt_deprecation_message RPAREN */
#line 1177 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	if (yymsp[-5].minor.yy9 == nullptr)
	{
		NEW_AST_NODE(DeclFlags,nil_f,yymsp[-4].minor.yy0.SourceLoc);
		yylhsminor.yy9 = nil_f;
		yylhsminor.yy9->Flags = ZCC_Deprecated;
		yylhsminor.yy9->Id = nullptr;
		yylhsminor.yy9->Version = { 0, 0 };
	}
	else
	{
		yylhsminor.yy9 = yymsp[-5].minor.yy9;
		yylhsminor.yy9->Flags |= ZCC_Deprecated;
	}
	yylhsminor.yy9->Version = yymsp[-2].minor.yy0.String->GetChars();
	yylhsminor.yy9->DeprecationMessage = yymsp[-1].minor.yy0.String;
}
#line 4159 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-3].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
  yymsp[-5].minor.yy9 = yylhsminor.yy9;
        break;
      case 186: /* decl_flags ::= decl_flags VERSION LPAREN STRCONST RPAREN */
#line 1196 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ 
	if (yymsp[-4].minor.yy9 == nullptr)
	{
		NEW_AST_NODE(DeclFlags,nil_f,yymsp[-3].minor.yy0.SourceLoc);
		yylhsminor.yy9 = nil_f;
		yylhsminor.yy9->Flags = ZCC_Version;
		yylhsminor.yy9->Id = nullptr;
		yylhsminor.yy9->Version = { 0, 0 };
		yylhsminor.yy9->DeprecationMessage = nullptr;
	}
	else
	{
		yylhsminor.yy9 = yymsp[-4].minor.yy9;
		yylhsminor.yy9->Flags |= ZCC_Version;
	}
	yylhsminor.yy9->Version = yymsp[-1].minor.yy0.String->GetChars();
}
#line 4183 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
  yymsp[-4].minor.yy9 = yylhsminor.yy9;
        break;
      case 187: /* decl_flag ::= NATIVE */
#line 1214 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Native; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4191 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 188: /* decl_flag ::= STATIC */
#line 1215 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Static; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4197 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 189: /* decl_flag ::= PRIVATE */
#line 1216 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Private; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4203 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 190: /* decl_flag ::= PROTECTED */
#line 1217 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Protected; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4209 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 191: /* decl_flag ::= LATENT */
#line 1218 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Latent; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4215 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 192: /* decl_flag ::= FINAL */
#line 1219 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Final; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4221 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 193: /* decl_flag ::= META */
#line 1220 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Meta; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4227 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 194: /* decl_flag ::= TRANSIENT */
#line 1221 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Transient; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4233 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 195: /* decl_flag ::= READONLY */
#line 1222 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_ReadOnly; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4239 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 196: /* decl_flag ::= INTERNAL */
#line 1223 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Internal; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4245 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 197: /* decl_flag ::= VIRTUAL */
#line 1224 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Virtual; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4251 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 198: /* decl_flag ::= OVERRIDE */
#line 1225 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Override; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4257 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 199: /* decl_flag ::= ABSTRACT */
#line 1226 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Abstract; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4263 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 200: /* decl_flag ::= VARARG */
#line 1227 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_VarArg; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 4269 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 201: /* decl_flag ::= UI */
#line 1228 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_UIFlag; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 4275 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 202: /* decl_flag ::= PLAY */
#line 1229 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_Play; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 4281 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 203: /* decl_flag ::= CLEARSCOPE */
#line 1230 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_ClearScope; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 4287 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 204: /* decl_flag ::= VIRTUALSCOPE */
#line 1231 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_VirtualScope; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 4293 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 205: /* func_const ::= */
#line 1233 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy0.Int = 0; yymsp[1].minor.yy0.SourceLoc = stat->sc->GetMessageLine(); }
#line 4299 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 206: /* func_const ::= CONST */
#line 1234 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = ZCC_FuncConst; yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc; }
#line 4304 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 207: /* opt_func_body ::= SEMICOLON */
{  yy_destructor(yypParser,49,&yymsp[0].minor);
#line 1236 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy51 = NULL; }
#line 4311 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 208: /* func_params ::= */
#line 1243 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy37 = NULL; }
#line 4317 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 209: /* func_params ::= VOID */
{  yy_destructor(yypParser,112,&yymsp[0].minor);
#line 1244 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy37 = NULL; }
#line 4323 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 210: /* func_params ::= func_param_list COMMA ELLIPSIS */
#line 1248 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FuncParamDecl,parm,stat->sc->GetMessageLine());
	parm->Type = nullptr;
	parm->Name = NAME_None;
	parm->Flags = 0;
	parm->Default = nullptr;
	yymsp[-2].minor.yy37 = yymsp[-2].minor.yy37; /*X-overwrites-A*/ 
	AppendTreeNodeSibling(yymsp[-2].minor.yy37, parm);
}
#line 4337 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
  yy_destructor(yypParser,132,&yymsp[0].minor);
        break;
      case 211: /* func_param_list ::= func_param_list COMMA func_param */
#line 1259 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy37 = yymsp[-2].minor.yy37; /*X-overwrites-A*/ AppendTreeNodeSibling(yymsp[-2].minor.yy37, yymsp[0].minor.yy37); }
#line 4344 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 212: /* func_param ::= func_param_flags type IDENTIFIER */
#line 1262 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FuncParamDecl,parm,yymsp[-2].minor.yy0.SourceLoc ? yymsp[-2].minor.yy0.SourceLoc : yymsp[-1].minor.yy256->SourceLoc);
	parm->Type = yymsp[-1].minor.yy256;
	parm->Name = yymsp[0].minor.yy0.Name();
	parm->Flags = yymsp[-2].minor.yy0.Int;
	parm->Default = nullptr;
	yylhsminor.yy37 = parm;
}
#line 4357 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-2].minor.yy37 = yylhsminor.yy37;
        break;
      case 213: /* func_param ::= func_param_flags type IDENTIFIER EQ expr */
#line 1272 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FuncParamDecl,parm,yymsp[-4].minor.yy0.SourceLoc ? yymsp[-4].minor.yy0.SourceLoc : yymsp[-3].minor.yy256->SourceLoc);
	parm->Type = yymsp[-3].minor.yy256;
	parm->Name = yymsp[-2].minor.yy0.Name();
	parm->Flags = yymsp[-4].minor.yy0.Int;
	parm->Default = yymsp[0].minor.yy154;
	yylhsminor.yy37 = parm;
}
#line 4370 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-1].minor);
  yymsp[-4].minor.yy37 = yylhsminor.yy37;
        break;
      case 214: /* func_param_flags ::= */
#line 1281 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy0.Int = 0; yymsp[1].minor.yy0.SourceLoc = 0; }
#line 4377 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 215: /* func_param_flags ::= func_param_flags IN */
#line 1282 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = yymsp[-1].minor.yy0.Int | ZCC_In; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4382 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy0 = yylhsminor.yy0;
        break;
      case 216: /* func_param_flags ::= func_param_flags OUT */
#line 1283 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = yymsp[-1].minor.yy0.Int | ZCC_Out; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4388 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy0 = yylhsminor.yy0;
        break;
      case 217: /* func_param_flags ::= func_param_flags OPTIONAL */
#line 1284 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yylhsminor.yy0.Int = yymsp[-1].minor.yy0.Int | ZCC_Optional; yylhsminor.yy0.SourceLoc =  yymsp[0].minor.yy0.SourceLoc; }
#line 4394 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy0 = yylhsminor.yy0;
        break;
      case 218: /* primary ::= IDENTIFIER */
#line 1300 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprID, expr, yymsp[0].minor.yy0);
	expr->Operation = PEX_ID;
	expr->Identifier = yymsp[0].minor.yy0.Name();
	expr->Type = NULL;
	yylhsminor.yy154 = expr;
}
#line 4406 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy154 = yylhsminor.yy154;
        break;
      case 219: /* primary ::= SUPER */
#line 1308 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(Expression, expr, yymsp[0].minor.yy0);
	expr->Operation = PEX_Super;
	expr->Type = NULL;
	yylhsminor.yy154 = expr;
}
#line 4417 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy154 = yylhsminor.yy154;
        break;
      case 220: /* primary ::= constant */
#line 1314 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy154 = yymsp[0].minor.yy215; /*X-overwrites-A*/ }
#line 4423 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 221: /* primary ::= LPAREN expr COMMA expr COMMA expr COMMA expr RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-8].minor);
#line 1316 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(VectorValue, expr, yymsp[-7].minor.yy154);
	expr->Operation = PEX_Vector;
	expr->Type = TypeVector4;
	expr->X = yymsp[-7].minor.yy154;
	expr->Y = yymsp[-5].minor.yy154;
	expr->Z = yymsp[-3].minor.yy154;
	expr->W = yymsp[-1].minor.yy154;
	yymsp[-8].minor.yy154 = expr;
}
#line 4438 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-6].minor);
  yy_destructor(yypParser,50,&yymsp[-4].minor);
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 222: /* primary ::= LPAREN expr COMMA expr COMMA expr RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-6].minor);
#line 1327 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(VectorValue, expr, yymsp[-5].minor.yy154);
	expr->Operation = PEX_Vector;
	expr->Type = TypeVector3;
	expr->X = yymsp[-5].minor.yy154;
	expr->Y = yymsp[-3].minor.yy154;
	expr->Z = yymsp[-1].minor.yy154;
	expr->W = nullptr;
	yymsp[-6].minor.yy154 = expr;
}
#line 4458 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-4].minor);
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 223: /* primary ::= LPAREN expr COMMA expr RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-4].minor);
#line 1338 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(VectorValue, expr, yymsp[-3].minor.yy154);
	expr->Operation = PEX_Vector;
	expr->Type = TypeVector2;
	expr->X = yymsp[-3].minor.yy154;
	expr->Y = yymsp[-1].minor.yy154;
	expr->Z = nullptr;
	expr->W = nullptr;
	yymsp[-4].minor.yy154 = expr;
}
#line 4477 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 224: /* primary ::= LPAREN expr RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-2].minor);
#line 1349 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[-2].minor.yy154 = yymsp[-1].minor.yy154; /*yymsp[-2].minor.yy154-overwrites-yymsp[-1].minor.yy154*/
}
#line 4488 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 225: /* primary ::= primary LPAREN func_expr_list RPAREN */
#line 1354 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprFuncCall, expr, yymsp[-3].minor.yy154);
	expr->Operation = PEX_FuncCall;
	expr->Type = NULL;
	expr->Function = yymsp[-3].minor.yy154;
	expr->Parameters = yymsp[-1].minor.yy74;
	yylhsminor.yy154 = expr;
}
#line 4502 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
  yymsp[-3].minor.yy154 = yylhsminor.yy154;
        break;
      case 226: /* primary ::= LPAREN CLASS LT IDENTIFIER GT RPAREN LPAREN func_expr_list RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-8].minor);
#line 1363 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ClassCast, expr, yymsp[-5].minor.yy0);
	expr->Operation = PEX_ClassCast;
	expr->ClassName = ENamedName(yymsp[-5].minor.yy0.Int);
	expr->Parameters = yymsp[-1].minor.yy74;
	yymsp[-8].minor.yy154 = expr;
}
#line 4517 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,53,&yymsp[-7].minor);
  yy_destructor(yypParser,20,&yymsp[-6].minor);
  yy_destructor(yypParser,21,&yymsp[-4].minor);
  yy_destructor(yypParser,62,&yymsp[-3].minor);
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      case 227: /* primary ::= primary LBRACKET expr RBRACKET */
#line 1371 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprBinary, expr, yymsp[-1].minor.yy154);
	expr->Operation = PEX_ArrayAccess;
	expr->Type = NULL;
	expr->Left = yymsp[-3].minor.yy154;
	expr->Right = yymsp[-1].minor.yy154;
	yylhsminor.yy154 = expr;
}
#line 4536 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,46,&yymsp[-2].minor);
  yy_destructor(yypParser,117,&yymsp[0].minor);
  yymsp[-3].minor.yy154 = yylhsminor.yy154;
        break;
      case 228: /* primary ::= primary DOT IDENTIFIER */
#line 1380 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprMemberAccess, expr, yymsp[0].minor.yy0);
	expr->Operation = PEX_MemberAccess;
	expr->Type = NULL;
	expr->Left = yymsp[-2].minor.yy154;
	expr->Right = ENamedName(yymsp[0].minor.yy0.Int);
	yylhsminor.yy154 = expr;
}
#line 4551 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,44,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 229: /* primary ::= primary ADDADD */
#line 1389 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[-1].minor.yy154,PEX_PostInc);
	yylhsminor.yy154 = expr1;
}
#line 4561 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,42,&yymsp[0].minor);
  yymsp[-1].minor.yy154 = yylhsminor.yy154;
        break;
      case 230: /* primary ::= primary SUBSUB */
#line 1394 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[-1].minor.yy154,PEX_PostDec);
	yylhsminor.yy154 = expr1;
}
#line 4571 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,43,&yymsp[0].minor);
  yymsp[-1].minor.yy154 = yylhsminor.yy154;
        break;
      case 231: /* unary_expr ::= SUB unary_expr */
{  yy_destructor(yypParser,33,&yymsp[-1].minor);
#line 1410 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	ZCC_ExprConstant *con = static_cast<ZCC_ExprConstant *>(yymsp[0].minor.yy154);
	if (yymsp[0].minor.yy154->Operation == PEX_ConstValue && (con->Type->isInt() || con->Type->isFloat()))
	{ // For constants, manipulate the child node directly, and don't create a new node.
		if (con->Type->isInt())
		{
			con->IntVal = -con->IntVal;
		}
		else
		{
			con->DoubleVal = -con->DoubleVal;
		}
		yymsp[-1].minor.yy154 = yymsp[0].minor.yy154;
	}
	else
	{ // For everything else, create a new node and do the negation later.
		UNARY_EXPR(yymsp[0].minor.yy154,PEX_Negate);
		yymsp[-1].minor.yy154 = expr1;
	}
}
#line 4598 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 232: /* unary_expr ::= ADD unary_expr */
{  yy_destructor(yypParser,34,&yymsp[-1].minor);
#line 1431 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	// Even though this is really a no-op, we still need to make a node for
	// it so we can type check that it is being applied to something numeric.
	// But we can do that right now for constant numerals.
	ZCC_ExprConstant *con = static_cast<ZCC_ExprConstant *>(yymsp[0].minor.yy154);
	if (yymsp[0].minor.yy154->Operation != PEX_ConstValue || (!con->Type->isInt() && !con->Type->isFloat()))
	{
		UNARY_EXPR(yymsp[0].minor.yy154,PEX_AntiNegate);
		yymsp[-1].minor.yy154 = expr1;
	}
	else
	{
		yymsp[-1].minor.yy154 = yymsp[0].minor.yy154;
	}
}
#line 4619 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 233: /* unary_expr ::= SUBSUB unary_expr */
{  yy_destructor(yypParser,43,&yymsp[-1].minor);
#line 1447 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[0].minor.yy154,PEX_PreDec);
	yymsp[-1].minor.yy154 = expr1;
}
#line 4629 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 234: /* unary_expr ::= ADDADD unary_expr */
{  yy_destructor(yypParser,42,&yymsp[-1].minor);
#line 1452 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[0].minor.yy154,PEX_PreInc);
	yymsp[-1].minor.yy154 = expr1;
}
#line 4639 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 235: /* unary_expr ::= TILDE unary_expr */
{  yy_destructor(yypParser,136,&yymsp[-1].minor);
#line 1457 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[0].minor.yy154,PEX_BitNot);
	yymsp[-1].minor.yy154 = expr1;
}
#line 4649 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 236: /* unary_expr ::= BANG unary_expr */
{  yy_destructor(yypParser,137,&yymsp[-1].minor);
#line 1462 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[0].minor.yy154,PEX_BoolNot);
	yymsp[-1].minor.yy154 = expr1;
}
#line 4659 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 237: /* unary_expr ::= SIZEOF unary_expr */
{  yy_destructor(yypParser,138,&yymsp[-1].minor);
#line 1467 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[0].minor.yy154,PEX_SizeOf);
	yymsp[-1].minor.yy154 = expr1;
}
#line 4669 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 238: /* unary_expr ::= ALIGNOF unary_expr */
{  yy_destructor(yypParser,139,&yymsp[-1].minor);
#line 1472 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	UNARY_EXPR(yymsp[0].minor.yy154,PEX_AlignOf);
	yymsp[-1].minor.yy154 = expr1;
}
#line 4679 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 239: /* expr ::= expr ADD expr */
#line 1485 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Add);
	yylhsminor.yy154 = expr2;
}
#line 4688 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,34,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 240: /* expr ::= expr SUB expr */
#line 1490 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Sub);
	yylhsminor.yy154 = expr2;
}
#line 4698 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,33,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 241: /* expr ::= expr MUL expr */
#line 1495 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Mul);
	yylhsminor.yy154 = expr2;
}
#line 4708 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,35,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 242: /* expr ::= expr DIV expr */
#line 1500 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Div);
	yylhsminor.yy154 = expr2;
}
#line 4718 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,36,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 243: /* expr ::= expr MOD expr */
#line 1505 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Mod);
	yylhsminor.yy154 = expr2;
}
#line 4728 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,37,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 244: /* expr ::= expr POW expr */
#line 1510 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Pow);
	yylhsminor.yy154 = expr2;
}
#line 4738 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,40,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 245: /* expr ::= expr CROSSPROD expr */
#line 1515 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_CrossProduct);
	yylhsminor.yy154 = expr2;
}
#line 4748 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,38,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 246: /* expr ::= expr DOTPROD expr */
#line 1520 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_DotProduct);
	yylhsminor.yy154 = expr2;
}
#line 4758 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,39,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 247: /* expr ::= expr LSH expr */
#line 1525 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_LeftShift);
	yylhsminor.yy154 = expr2;
}
#line 4768 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,30,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 248: /* expr ::= expr RSH expr */
#line 1530 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_RightShift);
	yylhsminor.yy154 = expr2;
}
#line 4778 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,31,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 249: /* expr ::= expr URSH expr */
#line 1535 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_URightShift);
	yylhsminor.yy154 = expr2;
}
#line 4788 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,32,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 250: /* expr ::= expr DOTDOT expr */
#line 1540 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Concat);
	yylhsminor.yy154 = expr2;
}
#line 4798 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,26,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 251: /* expr ::= expr LT expr */
#line 1546 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_LT);
	yylhsminor.yy154 = expr2;
}
#line 4808 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,20,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 252: /* expr ::= expr GT expr */
#line 1551 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_GT);
	yylhsminor.yy154 = expr2;
}
#line 4818 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,21,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 253: /* expr ::= expr LTEQ expr */
#line 1556 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_LTEQ);
	yylhsminor.yy154 = expr2;
}
#line 4828 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,22,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 254: /* expr ::= expr GTEQ expr */
#line 1561 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_GTEQ);
	yylhsminor.yy154 = expr2;
}
#line 4838 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,23,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 255: /* expr ::= expr LTGTEQ expr */
#line 1566 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_LTGTEQ);
	yylhsminor.yy154 = expr2;
}
#line 4848 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,24,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 256: /* expr ::= expr IS expr */
#line 1571 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Is);
	yylhsminor.yy154 = expr2;
}
#line 4858 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,25,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 257: /* expr ::= expr EQEQ expr */
#line 1577 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_EQEQ);
	yylhsminor.yy154 = expr2;
}
#line 4868 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,17,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 258: /* expr ::= expr NEQ expr */
#line 1582 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_NEQ);
	yylhsminor.yy154 = expr2;
}
#line 4878 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,18,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 259: /* expr ::= expr APPROXEQ expr */
#line 1587 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_APREQ);
	yylhsminor.yy154 = expr2;
}
#line 4888 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,19,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 260: /* expr ::= expr AND expr */
#line 1593 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_BitAnd);
	yylhsminor.yy154 = expr2;
}
#line 4898 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,29,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 261: /* expr ::= expr XOR expr */
#line 1598 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_BitXor);
	yylhsminor.yy154 = expr2;
}
#line 4908 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,28,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 262: /* expr ::= expr OR expr */
#line 1603 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_BitOr);
	yylhsminor.yy154 = expr2;
}
#line 4918 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,27,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 263: /* expr ::= expr ANDAND expr */
#line 1608 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_BoolAnd);
	yylhsminor.yy154 = expr2;
}
#line 4928 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,16,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 264: /* expr ::= expr OROR expr */
#line 1613 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_BoolOr);
	yylhsminor.yy154 = expr2;
}
#line 4938 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,15,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 265: /* expr ::= expr EQ expr */
#line 1618 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Assign);
	yylhsminor.yy154 = expr2;
}
#line 4948 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 266: /* expr ::= expr ADDEQ expr */
#line 1623 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_AddAssign);
	yylhsminor.yy154 = expr2;
}
#line 4958 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,5,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 267: /* expr ::= expr SUBEQ expr */
#line 1628 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_SubAssign);
	yylhsminor.yy154 = expr2;
}
#line 4968 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,6,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 268: /* expr ::= expr MULEQ expr */
#line 1633 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_MulAssign);
	yylhsminor.yy154 = expr2;
}
#line 4978 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,2,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 269: /* expr ::= expr DIVEQ expr */
#line 1638 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_DivAssign);
	yylhsminor.yy154 = expr2;
}
#line 4988 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,3,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 270: /* expr ::= expr MODEQ expr */
#line 1643 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_ModAssign);
	yylhsminor.yy154 = expr2;
}
#line 4998 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,4,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 271: /* expr ::= expr LSHEQ expr */
#line 1648 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_LshAssign);
	yylhsminor.yy154 = expr2;
}
#line 5008 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,7,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 272: /* expr ::= expr RSHEQ expr */
#line 1653 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_RshAssign);
	yylhsminor.yy154 = expr2;
}
#line 5018 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,8,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 273: /* expr ::= expr URSHEQ expr */
#line 1658 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_URshAssign);
	yylhsminor.yy154 = expr2;
}
#line 5028 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,12,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 274: /* expr ::= expr ANDEQ expr */
#line 1663 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_AndAssign);
	yylhsminor.yy154 = expr2;
}
#line 5038 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,9,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 275: /* expr ::= expr OREQ expr */
#line 1668 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_OrAssign);
	yylhsminor.yy154 = expr2;
}
#line 5048 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,10,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 276: /* expr ::= expr XOREQ expr */
#line 1673 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_XorAssign);
	yylhsminor.yy154 = expr2;
}
#line 5058 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,11,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 277: /* expr ::= expr SCOPE expr */
#line 1680 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	BINARY_EXPR(yymsp[-2].minor.yy154,yymsp[0].minor.yy154,PEX_Scope);
	yylhsminor.yy154 = expr2;
}
#line 5068 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,47,&yymsp[-1].minor);
  yymsp[-2].minor.yy154 = yylhsminor.yy154;
        break;
      case 278: /* expr ::= expr QUESTION expr COLON expr */
#line 1688 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprTrinary, expr, yymsp[-4].minor.yy154);
	expr->Operation = PEX_Trinary;
	expr->Type = NULL;
	expr->Test = yymsp[-4].minor.yy154;
	expr->Left = yymsp[-2].minor.yy154;
	expr->Right = yymsp[0].minor.yy154;
	yylhsminor.yy154 = expr;
}
#line 5083 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,13,&yymsp[-3].minor);
  yy_destructor(yypParser,14,&yymsp[-1].minor);
  yymsp[-4].minor.yy154 = yylhsminor.yy154;
        break;
      case 279: /* expr_list ::= expr_list COMMA expr */
#line 1704 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[-2].minor.yy154 = yymsp[-2].minor.yy154;	/*X-overwrites-A*/
	AppendTreeNodeSibling(yymsp[-2].minor.yy154, yymsp[0].minor.yy154);
}
#line 5094 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 280: /* func_expr_list ::= func_expr_list COMMA func_expr_item */
#line 1720 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	// Omitted parameters still need to appear as nodes in the list.
	if (yymsp[-2].minor.yy74 == NULL)
	{
		NEW_AST_NODE(FuncParm,nil_a,yymsp[-1].minor.yy0);
		nil_a->Value = NULL;
		nil_a->Label = NAME_None;
		yymsp[-2].minor.yy74 = nil_a;
	}
	if (yymsp[0].minor.yy74 == NULL)
	{
		NEW_AST_NODE(FuncParm,nil_b,yymsp[-1].minor.yy0);
		nil_b->Value = NULL;
		nil_b->Label = NAME_None;
		yymsp[0].minor.yy74 = nil_b;
	}
	yymsp[-2].minor.yy74 = yymsp[-2].minor.yy74;	/*X-overwrites-A*/
	AppendTreeNodeSibling(yymsp[-2].minor.yy74, yymsp[0].minor.yy74);
}
#line 5118 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 281: /* func_expr_item ::= */
#line 1741 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[1].minor.yy74 = NULL;
}
#line 5125 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 282: /* named_expr ::= IDENTIFIER COLON expr */
#line 1747 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FuncParm, parm, yymsp[-2].minor.yy0);
	parm->Value = yymsp[0].minor.yy154;
	parm->Label = ENamedName(yymsp[-2].minor.yy0.Int);
	yylhsminor.yy74 = parm;
}
#line 5135 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,14,&yymsp[-1].minor);
  yymsp[-2].minor.yy74 = yylhsminor.yy74;
        break;
      case 283: /* named_expr ::= expr */
#line 1754 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(FuncParm, parm, yymsp[0].minor.yy154);
	parm->Value = yymsp[0].minor.yy154;
	parm->Label = NAME_None;
	yylhsminor.yy74 = parm;
}
#line 5147 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy74 = yylhsminor.yy74;
        break;
      case 285: /* string_constant ::= string_constant STRCONST */
#line 1775 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprConstant, strconst, yymsp[-1].minor.yy215);
	strconst->Operation = PEX_ConstValue;
	strconst->Type = TypeString;
	strconst->StringVal = stat->Strings.Alloc(*(yymsp[-1].minor.yy215->StringVal) + *(yymsp[0].minor.yy0.String));
	yylhsminor.yy215 = strconst;
}
#line 5159 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy215 = yylhsminor.yy215;
        break;
      case 286: /* constant ::= INTCONST */
#line 1785 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_INTCONST_NODE(intconst, TypeSInt32, yymsp[0].minor.yy0.Int, yymsp[0].minor.yy0);
	yylhsminor.yy215 = intconst;
}
#line 5168 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 287: /* constant ::= UINTCONST */
#line 1790 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_INTCONST_NODE(intconst, TypeUInt32, yymsp[0].minor.yy0.Int, yymsp[0].minor.yy0);
	yylhsminor.yy215 = intconst;
}
#line 5177 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 288: /* constant ::= FLOATCONST */
#line 1795 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprConstant, floatconst, yymsp[0].minor.yy0);
	floatconst->Operation = PEX_ConstValue;
	floatconst->Type = TypeFloat64;
	floatconst->DoubleVal = yymsp[0].minor.yy0.Float;
	yylhsminor.yy215 = floatconst;
}
#line 5189 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 289: /* constant ::= NAMECONST */
#line 1803 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprConstant, floatconst, yymsp[0].minor.yy0);
	floatconst->Operation = PEX_ConstValue;
	floatconst->Type = TypeName;
	floatconst->IntVal = yymsp[0].minor.yy0.Int;
	yylhsminor.yy215 = floatconst;
}
#line 5201 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 290: /* constant ::= FALSE */
#line 1811 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_INTCONST_NODE(boolconst, TypeBool, false, yymsp[0].minor.yy0);
	yylhsminor.yy215 = boolconst;
}
#line 5210 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 291: /* constant ::= TRUE */
#line 1816 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_INTCONST_NODE(boolconst, TypeBool, true, yymsp[0].minor.yy0);
	yylhsminor.yy215 = boolconst;
}
#line 5219 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 292: /* constant ::= NULLPTR */
#line 1821 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExprConstant, nullptrconst, yymsp[0].minor.yy0);
	nullptrconst->Operation = PEX_ConstValue;
	nullptrconst->Type = TypeNullPtr;
	nullptrconst->StringVal = nullptr;
	yylhsminor.yy215 = nullptrconst;
}
#line 5231 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy215 = yylhsminor.yy215;
        break;
      case 294: /* statement ::= labeled_statement */
#line 1835 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy296; /*X-overwrites-A*/ }
#line 5237 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 295: /* statement ::= compound_statement */
#line 1836 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy51; /*X-overwrites-A*/ }
#line 5242 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 296: /* statement ::= expression_statement SEMICOLON */
#line 1837 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy367 = yymsp[-1].minor.yy372; /*X-overwrites-A*/ }
#line 5247 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
        break;
      case 297: /* statement ::= assign_statement SEMICOLON */
#line 1842 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy367 = yymsp[-1].minor.yy25; /*X-overwrites-A*/ }
#line 5253 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
        break;
      case 298: /* statement ::= local_var SEMICOLON */
#line 1843 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-1].minor.yy367 = yymsp[-1].minor.yy32; /*X-overwrites-A*/ }
#line 5259 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
        break;
      case 300: /* statement ::= staticarray_statement */
#line 1845 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy502; /*X-overwrites-A*/ }
#line 5265 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 301: /* staticarray_statement ::= STATICCONST type IDENTIFIER LBRACKET RBRACKET EQ LBRACE expr_list RBRACE SEMICOLON */
{  yy_destructor(yypParser,146,&yymsp[-9].minor);
#line 1852 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StaticArrayStatement, stmt, yymsp[-8].minor.yy256);
	stmt->Type = yymsp[-8].minor.yy256;
	stmt->Id = ENamedName(yymsp[-7].minor.yy0.Int);
	stmt->Values = yymsp[-2].minor.yy154;
	yymsp[-9].minor.yy502 = stmt;
}
#line 5277 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,46,&yymsp[-6].minor);
  yy_destructor(yypParser,117,&yymsp[-5].minor);
  yy_destructor(yypParser,1,&yymsp[-4].minor);
  yy_destructor(yypParser,65,&yymsp[-3].minor);
  yy_destructor(yypParser,66,&yymsp[-1].minor);
  yy_destructor(yypParser,49,&yymsp[0].minor);
}
        break;
      case 302: /* staticarray_statement ::= STATICCONST type LBRACKET RBRACKET IDENTIFIER EQ LBRACE expr_list RBRACE SEMICOLON */
{  yy_destructor(yypParser,146,&yymsp[-9].minor);
#line 1861 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(StaticArrayStatement, stmt, yymsp[-8].minor.yy256);
	stmt->Type = yymsp[-8].minor.yy256;
	stmt->Id = ENamedName(yymsp[-5].minor.yy0.Int);
	stmt->Values = yymsp[-2].minor.yy154;
	yymsp[-9].minor.yy502 = stmt;
}
#line 5296 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,46,&yymsp[-7].minor);
  yy_destructor(yypParser,117,&yymsp[-6].minor);
  yy_destructor(yypParser,1,&yymsp[-4].minor);
  yy_destructor(yypParser,65,&yymsp[-3].minor);
  yy_destructor(yypParser,66,&yymsp[-1].minor);
  yy_destructor(yypParser,49,&yymsp[0].minor);
}
        break;
      case 303: /* jump_statement ::= CONTINUE SEMICOLON */
#line 1874 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ContinueStmt, stmt, yymsp[-1].minor.yy0);
	yylhsminor.yy367 = stmt;
}
#line 5311 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-1].minor.yy367 = yylhsminor.yy367;
        break;
      case 304: /* jump_statement ::= BREAK SEMICOLON */
#line 1879 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(BreakStmt, stmt, yymsp[-1].minor.yy0);
	yylhsminor.yy367 = stmt;
}
#line 5321 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-1].minor.yy367 = yylhsminor.yy367;
        break;
      case 305: /* jump_statement ::= RETURN SEMICOLON */
#line 1884 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ReturnStmt, stmt, yymsp[-1].minor.yy0);
	stmt->Values = NULL;
	yylhsminor.yy367 = stmt;
}
#line 5332 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-1].minor.yy367 = yylhsminor.yy367;
        break;
      case 306: /* jump_statement ::= RETURN expr_list SEMICOLON */
#line 1890 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ReturnStmt, stmt, yymsp[-2].minor.yy0);
	stmt->Values = yymsp[-1].minor.yy154;
	yylhsminor.yy367 = stmt;
}
#line 5343 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
  yymsp[-2].minor.yy367 = yylhsminor.yy367;
        break;
      case 307: /* compound_statement ::= LBRACE RBRACE */
#line 1902 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(CompoundStmt,stmt,yymsp[-1].minor.yy0);
	stmt->Content = NULL;
	yylhsminor.yy51 = stmt;
}
#line 5354 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-1].minor.yy51 = yylhsminor.yy51;
        break;
      case 308: /* compound_statement ::= LBRACE statement_list RBRACE */
#line 1908 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(CompoundStmt,stmt,yymsp[-2].minor.yy0);
	stmt->Content = yymsp[-1].minor.yy367;
	yylhsminor.yy51 = stmt;
}
#line 5365 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-2].minor.yy51 = yylhsminor.yy51;
        break;
      case 309: /* compound_statement ::= LBRACE error RBRACE */
#line 1914 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(CompoundStmt,stmt,yymsp[-2].minor.yy0);
	stmt->Content = NULL;
	yylhsminor.yy51 = stmt;
}
#line 5376 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-2].minor.yy51 = yylhsminor.yy51;
        break;
      case 312: /* expression_statement ::= expr */
#line 1934 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ExpressionStmt, stmt, yymsp[0].minor.yy154);
	stmt->Expression = yymsp[0].minor.yy154;
	yylhsminor.yy372 = stmt;
}
#line 5387 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy372 = yylhsminor.yy372;
        break;
      case 313: /* iteration_statement ::= while_or_until LPAREN expr RPAREN statement */
#line 1946 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(IterationStmt, iter, yymsp[-4].minor.yy0);
	if (yymsp[-4].minor.yy0.Int == ZCC_UNTIL)
	{ // Negate the loop condition
		UNARY_EXPR(yymsp[-2].minor.yy154,PEX_BoolNot);
		iter->LoopCondition = expr1;
	}
	else
	{
		iter->LoopCondition = yymsp[-2].minor.yy154;
	}
	iter->LoopStatement = yymsp[0].minor.yy367;
	iter->LoopBumper = NULL;
	iter->CheckAt = ZCC_IterationStmt::Start;
	yylhsminor.yy367 = iter;
}
#line 5408 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-3].minor);
  yy_destructor(yypParser,62,&yymsp[-1].minor);
  yymsp[-4].minor.yy367 = yylhsminor.yy367;
        break;
      case 314: /* iteration_statement ::= DO statement while_or_until LPAREN expr RPAREN */
#line 1964 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(IterationStmt, iter, yymsp[-5].minor.yy0);
	if (yymsp[-3].minor.yy0.Int == ZCC_UNTIL)
	{ // Negate the loop condition
		UNARY_EXPR(yymsp[-1].minor.yy154,PEX_BoolNot);
		iter->LoopCondition = expr1;
	}
	else
	{
		iter->LoopCondition = yymsp[-1].minor.yy154;
	}
	iter->LoopStatement = yymsp[-4].minor.yy367;
	iter->LoopBumper = NULL;
	iter->CheckAt = ZCC_IterationStmt::End;
	yylhsminor.yy367 = iter;
}
#line 5431 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-2].minor);
  yy_destructor(yypParser,62,&yymsp[0].minor);
  yymsp[-5].minor.yy367 = yylhsminor.yy367;
        break;
      case 315: /* iteration_statement ::= FOR LPAREN for_init SEMICOLON opt_expr SEMICOLON for_bump RPAREN statement */
#line 1982 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(IterationStmt, iter, yymsp[-8].minor.yy0);
	iter->LoopCondition = yymsp[-4].minor.yy154;
	iter->LoopStatement = yymsp[0].minor.yy367;
	iter->LoopBumper = yymsp[-2].minor.yy367;
	iter->CheckAt = ZCC_IterationStmt::Start;
	// The initialization expression appears outside the loop
	// for_init may be NULL if there is no initialization.
	SAFE_APPEND(yymsp[-6].minor.yy367, iter);
	// And the whole thing gets wrapped inside a compound statement in case the loop
	// initializer defined any variables.
	NEW_AST_NODE(CompoundStmt, wrap, yymsp[-8].minor.yy0);
	wrap->Content = yymsp[-6].minor.yy367;
	yylhsminor.yy367 = wrap;
}
#line 5453 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-7].minor);
  yy_destructor(yypParser,49,&yymsp[-5].minor);
  yy_destructor(yypParser,49,&yymsp[-3].minor);
  yy_destructor(yypParser,62,&yymsp[-1].minor);
  yymsp[-8].minor.yy367 = yylhsminor.yy367;
        break;
      case 316: /* array_iteration_statement ::= FOREACH LPAREN variable_name COLON expr RPAREN statement */
#line 2001 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(ArrayIterationStmt, iter, yymsp[-6].minor.yy0);
	iter->ItName = yymsp[-4].minor.yy134;
	iter->ItArray = yymsp[-2].minor.yy154;
	iter->LoopStatement = yymsp[0].minor.yy367;
	yylhsminor.yy367 = iter;
}
#line 5469 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-5].minor);
  yy_destructor(yypParser,14,&yymsp[-3].minor);
  yy_destructor(yypParser,62,&yymsp[-1].minor);
  yymsp[-6].minor.yy367 = yylhsminor.yy367;
        break;
      case 317: /* while_or_until ::= WHILE */
#line 2010 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yylhsminor.yy0.Int = ZCC_WHILE;
	yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc;
}
#line 5481 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 318: /* while_or_until ::= UNTIL */
#line 2015 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yylhsminor.yy0.Int = ZCC_UNTIL;
	yylhsminor.yy0.SourceLoc = yymsp[0].minor.yy0.SourceLoc;
}
#line 5490 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 319: /* for_init ::= local_var */
#line 2021 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy32 /*X-overwrites-A*/; }
#line 5496 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 320: /* for_init ::= for_bump */
#line 2022 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy367 /*X-overwrites-A*/; }
#line 5501 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 321: /* for_bump ::= */
#line 2025 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[1].minor.yy367 = NULL; }
#line 5506 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 322: /* for_bump ::= expression_statement */
#line 2026 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[0].minor.yy367 = yymsp[0].minor.yy372; /*X-overwrites-A*/ }
#line 5511 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 323: /* for_bump ::= for_bump COMMA expression_statement */
#line 2027 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ yymsp[-2].minor.yy367 = yymsp[-2].minor.yy367; /*X-overwrites-A*/ AppendTreeNodeSibling(yymsp[-2].minor.yy367, yymsp[0].minor.yy372); }
#line 5516 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 324: /* selection_statement ::= if_front */
#line 2041 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[0].minor.yy367 = yymsp[0].minor.yy17; /*X-overwrites-A*/
}
#line 5524 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
        break;
      case 325: /* selection_statement ::= if_front ELSE statement */
#line 2045 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	yymsp[-2].minor.yy17->FalsePath = yymsp[0].minor.yy367;
	yymsp[-2].minor.yy367 = yymsp[-2].minor.yy17; /*X-overwrites-A*/
}
#line 5532 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,156,&yymsp[-1].minor);
        break;
      case 326: /* if_front ::= IF LPAREN expr RPAREN statement */
#line 2051 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(IfStmt,stmt,yymsp[-4].minor.yy0);
	stmt->Condition = yymsp[-2].minor.yy154;
	stmt->TruePath = yymsp[0].minor.yy367;
	stmt->FalsePath = NULL;
	yylhsminor.yy17 = stmt;
}
#line 5544 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-3].minor);
  yy_destructor(yypParser,62,&yymsp[-1].minor);
  yymsp[-4].minor.yy17 = yylhsminor.yy17;
        break;
      case 327: /* selection_statement ::= SWITCH LPAREN expr RPAREN statement */
#line 2062 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(SwitchStmt,stmt,yymsp[-4].minor.yy0);
	stmt->Condition = yymsp[-2].minor.yy154;
	stmt->Content = yymsp[0].minor.yy367;
	yylhsminor.yy367 = stmt;
}
#line 5557 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,45,&yymsp[-3].minor);
  yy_destructor(yypParser,62,&yymsp[-1].minor);
  yymsp[-4].minor.yy367 = yylhsminor.yy367;
        break;
      case 328: /* labeled_statement ::= CASE expr COLON */
#line 2074 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(CaseStmt,stmt,yymsp[-2].minor.yy0);
	stmt->Condition = yymsp[-1].minor.yy154;
	yylhsminor.yy296 = stmt;
}
#line 5569 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,14,&yymsp[0].minor);
  yymsp[-2].minor.yy296 = yylhsminor.yy296;
        break;
      case 329: /* labeled_statement ::= DEFAULT COLON */
#line 2080 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(CaseStmt,stmt,yymsp[-1].minor.yy0);
	stmt->Condition = NULL;
	yylhsminor.yy296 = stmt;
}
#line 5580 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,14,&yymsp[0].minor);
  yymsp[-1].minor.yy296 = yylhsminor.yy296;
        break;
      case 330: /* assign_statement ::= LBRACKET expr_list RBRACKET EQ expr */
{  yy_destructor(yypParser,46,&yymsp[-4].minor);
#line 2093 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(AssignStmt,stmt,yymsp[-3].minor.yy154);
	stmt->AssignOp = ZCC_EQ;
	stmt->Dests = yymsp[-3].minor.yy154;
	stmt->Sources = yymsp[0].minor.yy154;
	yymsp[-4].minor.yy25 = stmt;
}
#line 5594 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,117,&yymsp[-2].minor);
  yy_destructor(yypParser,1,&yymsp[-1].minor);
}
        break;
      case 331: /* local_var ::= type variable_list_with_init */
#line 2106 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(LocalVarStmt,vardef,yymsp[-1].minor.yy256);
	vardef->Type = yymsp[-1].minor.yy256;
	vardef->Vars = yymsp[0].minor.yy83;
	yylhsminor.yy32 = vardef;
}
#line 5607 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy32 = yylhsminor.yy32;
        break;
      case 332: /* var_init ::= IDENTIFIER */
#line 2115 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ 
	NEW_AST_NODE(VarInit,var,yymsp[0].minor.yy0);
	var->Name = ENamedName(yymsp[0].minor.yy0.Int);
	var->ArraySize = NULL;
	var->Init = NULL;
	var->InitIsArray = false;
	yylhsminor.yy83 = var;
}
#line 5620 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[0].minor.yy83 = yylhsminor.yy83;
        break;
      case 333: /* var_init ::= IDENTIFIER array_size */
#line 2125 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	NEW_AST_NODE(VarInit,var,yymsp[-1].minor.yy0);
	var->Name = ENamedName(yymsp[-1].minor.yy0.Int);
	var->ArraySize = yymsp[0].minor.yy154;
	var->Init = NULL;
	var->InitIsArray = false;
	yylhsminor.yy83 = var;
}
#line 5633 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yymsp[-1].minor.yy83 = yylhsminor.yy83;
        break;
      case 334: /* var_init ::= IDENTIFIER EQ expr */
#line 2135 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ 
	NEW_AST_NODE(VarInit,var,yymsp[-2].minor.yy0);
	var->Name = ENamedName(yymsp[-2].minor.yy0.Int);
	var->ArraySize = NULL;
	var->Init = yymsp[0].minor.yy154;
	var->InitIsArray = false;
	yylhsminor.yy83 = var;
}
#line 5646 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-1].minor);
  yymsp[-2].minor.yy83 = yylhsminor.yy83;
        break;
      case 335: /* var_init ::= IDENTIFIER EQ LBRACE expr_list RBRACE */
#line 2145 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ 
	NEW_AST_NODE(VarInit,var,yymsp[-4].minor.yy0);
	var->Name = ENamedName(yymsp[-4].minor.yy0.Int);
	var->ArraySize = NULL;
	var->Init = yymsp[-1].minor.yy154;
	var->InitIsArray = true;
	yylhsminor.yy83 = var;
}
#line 5660 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-3].minor);
  yy_destructor(yypParser,65,&yymsp[-2].minor);
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-4].minor.yy83 = yylhsminor.yy83;
        break;
      case 336: /* var_init ::= IDENTIFIER array_size EQ LBRACE expr_list RBRACE */
#line 2155 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ 
	NEW_AST_NODE(VarInit,var,yymsp[-5].minor.yy0);
	var->Name = ENamedName(yymsp[-5].minor.yy0.Int);
	var->ArraySize = yymsp[-4].minor.yy154;
	var->Init = yymsp[-1].minor.yy154;
	var->InitIsArray = true;
	yylhsminor.yy83 = var;
}
#line 5676 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-3].minor);
  yy_destructor(yypParser,65,&yymsp[-2].minor);
  yy_destructor(yypParser,66,&yymsp[0].minor);
  yymsp[-5].minor.yy83 = yylhsminor.yy83;
        break;
      case 337: /* var_init ::= IDENTIFIER EQ LBRACE error RBRACE */
{  yy_destructor(yypParser,54,&yymsp[-4].minor);
#line 2165 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{ 
	yymsp[-4].minor.yy83 = NULL;
}
#line 5688 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,1,&yymsp[-3].minor);
  yy_destructor(yypParser,65,&yymsp[-2].minor);
  yy_destructor(yypParser,66,&yymsp[0].minor);
}
        break;
      case 338: /* variable_list_with_init ::= variable_list_with_init COMMA var_init */
#line 2173 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
	AppendTreeNodeSibling(yymsp[-2].minor.yy83, yymsp[0].minor.yy83);
	yymsp[-2].minor.yy83 = yymsp[-2].minor.yy83; /*X-overwrites-A*/
}
#line 5700 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,50,&yymsp[-1].minor);
        break;
      case 339: /* translation_unit ::= translation_unit EOF */
#line 178 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
}
#line 5707 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,48,&yymsp[0].minor);
        break;
      case 341: /* opt_semicolon ::= SEMICOLON */
{  yy_destructor(yypParser,49,&yymsp[0].minor);
#line 191 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
}
#line 5715 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 343: /* opt_comma ::= COMMA */
{  yy_destructor(yypParser,50,&yymsp[0].minor);
#line 194 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
}
#line 5723 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
}
        break;
      case 349: /* state_flow ::= state_flow_type scanner_mode SEMICOLON */
#line 650 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
}
#line 5730 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,49,&yymsp[0].minor);
        break;
      case 359: /* primary ::= LPAREN error RPAREN */
{  yy_destructor(yypParser,45,&yymsp[-2].minor);
#line 1352 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
{
}
#line 5738 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
  yy_destructor(yypParser,62,&yymsp[0].minor);
}
        break;
      default:
      /* (340) opt_semicolon ::= */ yytestcase(yyruleno==340);
      /* (342) opt_comma ::= */ yytestcase(yyruleno==342);
      /* (344) opt_expr ::= expr */ yytestcase(yyruleno==344);
      /* (345) opt_struct_body ::= struct_body */ yytestcase(yyruleno==345);
      /* (346) struct_body ::= struct_member */ yytestcase(yyruleno==346);
      /* (347) enum_list ::= enumerator */ yytestcase(yyruleno==347);
      /* (348) opt_enum_list ::= enum_list opt_comma */ yytestcase(yyruleno==348);
      /* (350) type_name1 ::= int_type */ yytestcase(yyruleno==350);
      /* (351) type_or_array ::= type */ yytestcase(yyruleno==351);
      /* (352) type_list ::= type_or_array */ yytestcase(yyruleno==352);
      /* (353) type_list_or_void ::= type_list */ yytestcase(yyruleno==353);
      /* (354) array_size ::= array_size_expr */ yytestcase(yyruleno==354);
      /* (355) variable_list ::= variable_name */ yytestcase(yyruleno==355);
      /* (356) opt_func_body ::= function_body */ yytestcase(yyruleno==356);
      /* (357) func_params ::= func_param_list */ yytestcase(yyruleno==357);
      /* (358) func_param_list ::= func_param */ yytestcase(yyruleno==358);
      /* (360) unary_expr ::= primary */ yytestcase(yyruleno==360);
      /* (361) expr ::= unary_expr */ yytestcase(yyruleno==361);
      /* (362) expr_list ::= expr */ yytestcase(yyruleno==362);
      /* (363) func_expr_list ::= func_expr_item */ yytestcase(yyruleno==363);
      /* (364) func_expr_item ::= named_expr */ yytestcase(yyruleno==364);
      /* (365) constant ::= string_constant */ yytestcase(yyruleno==365);
      /* (366) function_body ::= compound_statement */ yytestcase(yyruleno==366);
      /* (367) statement ::= selection_statement */ yytestcase(yyruleno==367);
      /* (368) statement ::= iteration_statement */ yytestcase(yyruleno==368);
      /* (369) statement ::= array_iteration_statement */ yytestcase(yyruleno==369);
      /* (370) statement ::= jump_statement */ yytestcase(yyruleno==370);
      /* (371) variable_list_with_init ::= var_init */ yytestcase(yyruleno==371);
       break;
/********** End reduce actions ************************************************/
  };
  assert( yyruleno<sizeof(yyRuleInfo)/sizeof(yyRuleInfo[0]) );
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact <= YY_MAX_SHIFTREDUCE ){
    if( yyact>YY_MAX_SHIFT ){
      yyact += YY_MIN_REDUCE - YY_MIN_SHIFTREDUCE;
    }
    yymsp -= yysize-1;
    yypParser->yytos = yymsp;
    yymsp->stateno = (YYACTIONTYPE)yyact;
    yymsp->major = (YYCODETYPE)yygoto;
    yyTraceShift(yypParser, yyact);
  }else{
    assert( yyact == YY_ACCEPT_ACTION );
    yypParser->yytos -= yysize;
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  ZCCParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
    fflush(yyTraceFILE);
  }
#endif
  while( yypParser->yytos>yypParser->yystack ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
/************ Begin %parse_failure code ***************************************/
#line 148 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
 /**failed = true;*/ 
#line 5814 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
/************ End %parse_failure code *****************************************/
  ZCCParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  ZCCParseTOKENTYPE yyminor         /* The minor type of the error token */
){
  ZCCParseARG_FETCH;
#define TOKEN yyminor
/************ Begin %syntax_error code ****************************************/
#line 121 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"

	FString unexpected, expecting;

	int i;
	int stateno = yypParser->yytos->stateno;

	unexpected << "Unexpected " << ZCCTokenName(yymajor);

	// Determine all the terminals that the parser would have accepted at this point
	// (see yy_find_shift_action). This list can get quite long. Is it worthwhile to
	// print it when not debugging the grammar, or would that be too confusing to
	// the average user?
	if (stateno < YY_SHIFT_MAX && (i = yy_shift_ofst[stateno])!=YY_SHIFT_USE_DFLT)
	{
		for (int j = 1; j < YYERRORSYMBOL; ++j)
		{
			int k = i + j;
			if (k >= 0 && k < YY_ACTTAB_COUNT && yy_lookahead[k] == j)
			{
				expecting << (expecting.IsEmpty() ? "Expecting " : " or ") << ZCCTokenName(j);
			}
		}
	}
	stat->sc->ScriptMessage("%s\n%s\n", unexpected.GetChars(), expecting.GetChars());
	FScriptPosition::ErrorCounter++;
#line 5857 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
/************ End %syntax_error code ******************************************/
  ZCCParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  ZCCParseARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
    fflush(yyTraceFILE);
  }
#endif
#ifndef YYNOERRORRECOVERY
  yypParser->yyerrcnt = -1;
#endif
#if 0
  assert( yypParser->yytos==yypParser->yystack );
#else
  while (yypParser->yytos>yypParser->yystack) yy_pop_parser_stack(yypParser);
#endif
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
/*********** Begin %parse_accept code *****************************************/
#line 147 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/src/common/scripting/frontend/zcc-parse.lemon"
 DPrintf(DMSG_SPAMMY, "Input accepted\n"); 
#line 5888 "/home/abuild/rpmbuild/BUILD/gzdoom-g4.10.0/build/src/zcc-parse.c"
/*********** End %parse_accept code *******************************************/
  ZCCParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "ZCCParseAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void ZCCParse(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  ZCCParseTOKENTYPE yyminor       /* The value for the token */
  ZCCParseARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  unsigned int yyact;   /* The parser action. */
#if !defined(YYERRORSYMBOL) && !defined(YYNOERRORRECOVERY)
  int yyendofinput;     /* True if we are at the end of input */
#endif
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  yypParser = (yyParser*)yyp;
  assert( yypParser->yytos!=0 );
#if !defined(YYERRORSYMBOL) && !defined(YYNOERRORRECOVERY)
  yyendofinput = (yymajor==0);
#endif
  ZCCParseARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput '%s'\n",yyTracePrompt,yyTokenName[yymajor]);
    fflush(yyTraceFILE);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact <= YY_MAX_SHIFTREDUCE ){
      yy_shift(yypParser,yyact,yymajor,yyminor);
#ifndef YYNOERRORRECOVERY
      yypParser->yyerrcnt--;
#endif
      yymajor = YYNOCODE;
    }else if( yyact <= YY_MAX_REDUCE ){
      yy_reduce(yypParser,yyact-YY_MIN_REDUCE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
      yyminorunion.yy0 = yyminor;
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
        fflush(yyTraceFILE);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminor);
      }
      yymx = yypParser->yytos->major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
          fflush(yyTraceFILE);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor, &yyminorunion);
        yymajor = YYNOCODE;
      }else{
        while( yypParser->yytos >= yypParser->yystack
            && yymx != YYERRORSYMBOL
            && (yyact = yy_find_reduce_action(
                        yypParser->yytos->stateno,
                        YYERRORSYMBOL)) >= YY_MIN_REDUCE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yytos < yypParser->yystack || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
#ifndef YYNOERRORRECOVERY
          yypParser->yyerrcnt = -1;
#endif
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          yy_shift(yypParser,yyact,YYERRORSYMBOL,yyminor);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor, yyminor);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor, yyminor);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
#ifndef YYNOERRORRECOVERY
        yypParser->yyerrcnt = -1;
#endif
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yytos>yypParser->yystack );
#ifndef NDEBUG
  if( yyTraceFILE ){
    yyStackEntry *i;
    char cDiv = '[';
    fprintf(yyTraceFILE,"%sReturn. Stack=",yyTracePrompt);
    for(i=&yypParser->yystack[1]; i<=yypParser->yytos; i++){
      fprintf(yyTraceFILE,"%c%s", cDiv, yyTokenName[i->major]);
      cDiv = ' ';
    }
    fprintf(yyTraceFILE,"]\n");
    fflush(yyTraceFILE);
  }
#endif
  return;
}
